@@ -49,9 +49,6 @@
 #if SLRN_HAS_CHARACTER_MAP
 char *Slrn_Charset;
 
-static char File_Error [] = "File error:";
-static char Not_Posted [] = "--- message not posted.";
-
 static unsigned char *ChMap_To_Iso_Map;
 static unsigned char *ChMap_From_Iso_Map;
 
@@ -152,14 +149,14 @@ int slrn_chmap_fix_file (char *file, int reverse)
 
    if (NULL == (fp = fopen (file, "r")))
      {
-        slrn_error ("%s %s %s", File_Error, file, Not_Posted);
+        slrn_error (_("File error: %s --- message not posted."), file);
         return -1;
      }
 
    if (NULL == (tmpfp = slrn_open_tmpfile_in_dir (tmp_file, tmp_file,
 						  sizeof (tmp_file))))
      {
-	slrn_error ("%s %s %s", File_Error, tmp_file, Not_Posted);
+	slrn_error (_("File error: %s --- message not posted."), tmp_file);
 	fclose (fp);
 	return -1;
      }
@@ -171,7 +168,7 @@ int slrn_chmap_fix_file (char *file, int reverse)
 	else chmap_map_string_to_iso (buf);
 	if (EOF == fputs (buf, tmpfp))
 	  {
-	     slrn_error ("Write Error. Disk Full? %s", Not_Posted);
+	     slrn_error (_("Write Error. Disk Full? --- message not posted."));
 	     ret = -1;
 	     break;
 	  }
@@ -201,9 +198,9 @@ void slrn_chmap_show_supported (void)
    CharMap_Type *map;
    unsigned int i;
 
-   printf ("  Default character set:     %s\n", DEFAULT_CHARSET_NAME);
+   printf (_("  Default character set:     %s\n"), DEFAULT_CHARSET_NAME);
    
-   printf (" SUPPORTED CHARACTER SETS:\n");
+   printf (_(" SUPPORTED CHARACTER SETS:\n"));
    printf ("  isolatin");
 
    for (i = 0; i < MAX_CHARMAPS; i++)
@@ -244,7 +241,7 @@ int slrn_set_charset (char *name)
 	  }
      }
 
-   slrn_error ("Unsupport character set: %s", name);
+   slrn_error (_("Unsupport character set: %s"), name);
    return -1;
 
 #else
