@@ -58,18 +58,34 @@ int slrn_edit_score (Slrn_Header_Type *h, char *newsgroup)
    time_t myclock;
    int file_modified = 0, re_error = 0;
    SLRegexp_Type re;
+   /* Note to translators: The translated string needs to have 10 characters.
+    * Each pair becomes a valid response for "Subject", "From", "References",
+    * "Edit" and "Cancel" (in that order); you cannot use any of the default
+    * characters for any other field than they originally stood for!
+    */
+   char *typeresp = _("SsFfRrEeCc");
+   /* Note to translators: The translated string needs to have 4 characters.
+    * The two pairs become valid responses for "This group" and "All groups",
+    * respectively. You cannot use any of the default characters for any other
+    * field than they originally stood for!
+    */
+   char *scoperesp = _("TtAa");
 
    if (Slrn_Score_File == NULL)
      {
-	slrn_error ("A Score file has not been specified.");
+	slrn_error (_("A Score file has not been specified."));
 	return -1;
      }
    
    if (Slrn_Prefix_Arg_Ptr == NULL)
      {
-	ch = slrn_get_response ("SsFfRrEeCc\007", "Pick Score type: \001Subject, \001From, \001References, \001Edit, \001Cancel");
+	char rsp;
+	
+	if (strlen(typeresp) != 10) /* Translator messed it up */
+	  typeresp = "";
+	ch = slrn_get_response ("SsFfRrEeCc\007", typeresp, _("Pick Score type: \001Subject, \001From, \001References, \001Edit, \001Cancel"));
 	if (ch == 7) return -1;
-	ch |= 0x20;
+	ch = slrn_map_translated_char ("SsFfRrEeCc", typeresp, ch) | 0x20;
 	if (ch == 'c') return -1;
    
 	while (1)
@@ -82,13 +98,17 @@ int slrn_edit_score (Slrn_Header_Type *h, char *newsgroup)
 	       break;
 	  }
 	
-	if ('a' == (0x20 | slrn_get_response ("TtaA", "Which newsgroups: \001This group, \001All groups")))
+	if (strlen (scoperesp) != 4) /* Translator messed it up */
+	  scoperesp = "";
+	rsp = slrn_get_response ("TtaA\007", scoperesp, _("Which newsgroups: \001This group, \001All groups"));
+	if (rsp == 7) return -1;
+	if ((slrn_map_translated_char ("TtAa", scoperesp, rsp) | 0x20) == 'a')
 	  ng = "*";
 
 	while (1)
 	  {
 	     *qregexp = 0;
-	     if (-1 == slrn_read_input ("Expires (MM/DD/YYYY, DD-MM-YYYY, +NN (days) or leave blank): ", NULL, qregexp, 1, 0))
+	     if (-1 == slrn_read_input (_("Expires (MM/DD/YYYY, DD-MM-YYYY, +NN (days) or leave blank): "), NULL, qregexp, 1, 0))
 	       return -1;
 	     if (*qregexp)
 	       {
@@ -130,12 +150,12 @@ int slrn_edit_score (Slrn_Header_Type *h, char *newsgroup)
        && (NULL == (fp = slrn_open_home_file (Slrn_Score_File, "w+", file,
 					      sizeof (file), 1))))
      {
-	slrn_error ("Unable to open %s", file);
+	slrn_error (_("Unable to open %s"), file);
 	return -1;
      }
    
-   re.pat = qregexp;
-   re.buf = buf;
+   re.pat = (unsigned char*) qregexp;
+   re.buf = (unsigned char*) buf;
    re.buf_len = sizeof (buf);
    re.case_sensitive = 0;
    
@@ -159,6 +179,7 @@ int slrn_edit_score (Slrn_Header_Type *h, char *newsgroup)
 	
 	subject = slrn_safe_strmalloc (h->subject);
 	slrn_subject_strip_was (subject);
+	
 	if ((NULL == (q = SLregexp_quote_string (subject, qregexp, sizeof (qregexp)))) ||
 	    (ch != 's') ||
 	    (0 != SLang_regexp_compile (&re)))
@@ -236,9 +257,8 @@ int slrn_edit_score (Slrn_Header_Type *h, char *newsgroup)
 	
 	/* drop */
      }
-   slrn_message_now ("Reloading score file ...");
-   if (-1 == slrn_read_score_file (file))
-     return -1;
-
+   if (Slrn_Scorefile_Open != NULL)
+     slrn_free (Slrn_Scorefile_Open);
+   Slrn_Scorefile_Open = slrn_safe_strmalloc (file);
    return 1;
 }
