@@ -62,6 +62,7 @@
 #include "slrndir.h"
 #include "vfile.h"
 #include "snprintf.h"
+#include "hooks.h"
 
 /*}}}*/
 
@@ -83,6 +84,7 @@ static SLscroll_Window_Type Group_Window;
 static Slrn_Group_Type *Groups;
 
 int Slrn_No_Backups = 0;
+int Slrn_No_Autosave = 0;
 
 int Slrn_Unsubscribe_New_Groups = 0;
 int Slrn_Check_New_Groups = 1;
@@ -119,7 +121,7 @@ static void group_update_screen (void);
 static void group_quick_help (void);
 static void save_newsrc_cmd (void);
 static void read_and_parse_active (int);
-static int  parse_active_line (char *, unsigned int *, int *, int *);
+static int  parse_active_line (unsigned char *, unsigned int *, int *, int *);
 static void remove_group_entry (Slrn_Group_Type *);
 
 /*}}}*/
@@ -289,8 +291,9 @@ static int group_update_range (Slrn_Group_Type *g, int min, int max) /*{{{*/
 	nmin = g->range.min;
 	
 	/* Server database inconsistent. */
-	for (n = nmin; n <= nmax; n++)
-	  group_mark_article_as_read (g, n);
+	if (nmax > 0)
+	  for (n = nmin; n <= nmax; n++)
+	    group_mark_article_as_read (g, n);
 	
 	/* g->unread = 0; */
 	Slrn_Full_Screen_Update = 1;
@@ -368,7 +371,7 @@ static int group_sync_group_with_server (Slrn_Group_Type *g, int *minp, int *max
    
    group = g->group_name;
    
-   slrn_message_now ("Selecting %s ...", group);
+   slrn_message_now (_("Selecting %s ..."), group);
 
    if (((reselect = Kill_After_Max) == 0) &&
        (strcmp (group, Slrn_Server_Obj->sv_current_group ())))
@@ -380,15 +383,15 @@ static int group_sync_group_with_server (Slrn_Group_Type *g, int *minp, int *max
    
    if (status == ERR_NOGROUP)
      {
-	slrn_error ("Group %s is bogus%s.", group,
-		    Slrn_Drop_Bogus_Groups ? " - dropping it" : "");
+	slrn_error (_("Group %s is bogus%s."), group,
+		    Slrn_Drop_Bogus_Groups ? _(" - dropping it") : "");
 	if (Slrn_Drop_Bogus_Groups)
 	  remove_group_entry (g);
 	return -1;
      }
    else if (status != OK_GROUP)
      {
-	slrn_error ("Could not enter group %s.", group);
+	slrn_error (_("Could not enter group %s."), group);
 	return -1;
      }
    
@@ -518,8 +521,8 @@ static Slrn_Group_Type *create_group_entry (char *name, unsigned int len, /*{{{*
 	status = Slrn_Server_Obj->sv_select_group (g->group_name, &min, &max);
 	if (status == ERR_NOGROUP)
 	  {
-	     slrn_error ("Group %s is bogus%s.", g->group_name,
-			 Slrn_Drop_Bogus_Groups ? " - ignoring it" : "");
+	     slrn_error (_("Group %s is bogus%s."), g->group_name,
+			 Slrn_Drop_Bogus_Groups ? _(" - ignoring it") : "");
 	     if (Slrn_Drop_Bogus_Groups)
 	       {
 		  Slrn_Groups_Dirty = 1;
@@ -613,7 +616,7 @@ static int add_group (char *name, unsigned int len, /*{{{*/
 	     
 	     if (NULL != (tmp_name = slrn_strnmalloc (name, len, 1)))
 	       {
-		  slrn_error ("Group %s is bogus - ignoring it.", tmp_name);
+		  slrn_error (_("Group %s is bogus - ignoring it."), tmp_name);
 		  slrn_free (tmp_name);
 	       }
 	     return -1;
@@ -799,10 +802,10 @@ static SLRegexp_Type *read_group_regexp (char *prompt, char *origpat) /*{{{*/
 
 /*}}}*/
 
-static void add_unsubscribed_group (char *name) /*{{{*/
+static void add_unsubscribed_group (unsigned char *name) /*{{{*/
 {
    Unsubscribed_Slrn_Group_Type *g;
-   char *p;
+   unsigned char *p;
    unsigned int len;
    
    g = (Unsubscribed_Slrn_Group_Type *) slrn_safe_malloc (sizeof (Unsubscribed_Slrn_Group_Type));
@@ -815,7 +818,7 @@ static void add_unsubscribed_group (char *name) /*{{{*/
    *p = 0;
    len = p - name;
 
-   g->group_name = slrn_safe_strmalloc (name);
+   g->group_name = slrn_safe_strmalloc ((char*)name);
 }
 
 /*}}}*/
@@ -921,7 +924,7 @@ int slrn_group_select_group (void) /*{{{*/
 
    if (-1 == group_sync_group_with_server (Slrn_Group_Current_Group, &min, &max))
      {
-	slrn_message ("No articles to read.");
+	slrn_message (_("No articles to read."));
 	return -1;
      }
    
@@ -943,7 +946,7 @@ int slrn_group_select_group (void) /*{{{*/
      {
 	char int_prompt_buf[256];
 	slrn_snprintf (int_prompt_buf, sizeof (int_prompt_buf),
-		       "%s: Read how many? ",
+		       _("%s: Read how many? "),
 		       Slrn_Group_Current_Group->group_name);
 	if ((-1 == slrn_read_integer (int_prompt_buf, &n, &n))
 	    || (n <= 0))
@@ -1028,16 +1031,16 @@ void slrn_group_quit (void) /*{{{*/
 {
    if (Slrn_User_Wants_Confirmation
        && (Slrn_Batch == 0)
-       && (slrn_get_yesno (1, "Do you really want to quit") <= 0)) return;
+       && (slrn_get_yesno (1, _("Do you really want to quit")) <= 0)) return;
 
-   if ((Slrn_Groups_Dirty) && (-1 == slrn_write_newsrc ()))
+   if ((Slrn_Groups_Dirty) && (-1 == slrn_write_newsrc (0)))
      {
 	if (Slrn_Batch)
 	  slrn_quit (1);
 
 	slrn_smg_refresh ();
 	if (Slrn_Batch == 0) slrn_sleep (2);
-	if (slrn_get_yesno (0, "Write to newsrc file failed.  Quit anyway") <= 0)
+	if (slrn_get_yesno (0, _("Write to newsrc file failed.  Quit anyway")) <= 0)
 	  return;
      }
    slrn_quit (0);
@@ -1050,7 +1053,7 @@ static void group_pagedown (void) /*{{{*/
    Slrn_Full_Screen_Update = 1;
    
    if (-1 == SLscroll_pagedown (&Group_Window))
-     slrn_error ("End of Buffer.");
+     slrn_error (_("End of Buffer."));
    Slrn_Group_Current_Group = (Slrn_Group_Type *) Group_Window.current_line;
 }
 
@@ -1061,7 +1064,7 @@ static void group_pageup (void) /*{{{*/
    Slrn_Full_Screen_Update = 1;
    
    if (-1 == SLscroll_pageup (&Group_Window))
-     slrn_error ("Top of Buffer.");
+     slrn_error (_("Top of Buffer."));
    
    Slrn_Group_Current_Group = (Slrn_Group_Type *) Group_Window.current_line;
 }
@@ -1072,7 +1075,7 @@ static void group_up (void) /*{{{*/
 {
    if (0 == slrn_group_up_n (1))
      {
-	slrn_error ("Top of buffer.");
+	slrn_error (_("Top of buffer."));
      }
 }
 
@@ -1122,7 +1125,7 @@ static void refresh_groups (Slrn_Group_Type **c) /*{{{*/
 	  }
 	if (Slrn_Server_Obj->sv_refresh_groups (ranges, i))
 	  {
-	     slrn_error ("Server connection dropped.");
+	     slrn_error (_("Server connection dropped."));
 	     return;
 	  }
 	g = start;
@@ -1135,8 +1138,8 @@ static void refresh_groups (Slrn_Group_Type **c) /*{{{*/
 	       }
 	     if (ranges[j].min == -1)
 	       {
-		  slrn_error ("Group %s is bogus%s.", g->group_name,
-			      Slrn_Drop_Bogus_Groups ? " - dropping it" : "");
+		  slrn_error (_("Group %s is bogus%s."), g->group_name,
+			      Slrn_Drop_Bogus_Groups ? _(" - dropping it") : "");
 		  if (Slrn_Drop_Bogus_Groups)
 		    {
 		       Slrn_Group_Type *tmp = g->next;
@@ -1163,8 +1166,8 @@ static void refresh_groups_cmd (void) /*{{{*/
    Slrn_Group_Type *c = Slrn_Group_Current_Group;
    Slrn_Group_Current_Group = NULL;
    
-   slrn_message_now ("Checking news%s ...",
-		     Slrn_List_Active_File ? " via active file" : "");
+   slrn_message_now (_("Checking news%s ..."),
+		     Slrn_List_Active_File ? _(" via active file") : "");
    
    if (Slrn_List_Active_File)
      {
@@ -1209,26 +1212,30 @@ static void refresh_groups_cmd (void) /*{{{*/
 static void generic_group_search (int dir) /*{{{*/
 {
    static char search_str[256];
-   char prompt[20];
+   char* prompt;
    Slrn_Group_Type *g;
    unsigned int n;
+   int ret;
    
    g = Slrn_Group_Current_Group;
    if (g == NULL) return;
    
-   sprintf (prompt, "%s Search: ", dir > 0 ? "Forward" : "Backward"); /* safe */
-   if (slrn_read_input (prompt, search_str, NULL, 1, 0) <= 0) return;
+   prompt = slrn_strdup_strcat ((dir > 0 ? _("Forward") : _("Backward")),
+				_(" Search: "), NULL);
+   ret = slrn_read_input (prompt, search_str, NULL, 1, 0);
+   slrn_free (prompt);
+   if (ret <= 0) return;
    
    n = Group_Window.line_num;
    if (0 == slrn_group_search (search_str, dir))
      {
-        slrn_error ("Not found.");
+        slrn_error (_("Not found."));
 	return;
      }
    
    if (((dir > 0) && (n > Group_Window.line_num)) ||
        ((dir < 0) && (n < Group_Window.line_num)))
-     slrn_message ("Search wrapped.");
+     slrn_message (_("Search wrapped."));
 }
 
 /*}}}*/
@@ -1257,7 +1264,7 @@ int slrn_add_group (char *group) /*{{{*/
 	  }
 	else
 	  {
-	     slrn_error ("Group %s does not exist.", group);
+	     slrn_error (_("Group %s does not exist."), group);
 	     retval = -1;
 	  }
      }
@@ -1273,7 +1280,7 @@ static void add_group_cmd (void) /*{{{*/
    char group[256];
    
    *group = 0;
-   if (slrn_read_input ("Add group: ", NULL, group, 1, 0) > 0)
+   if (slrn_read_input (_("Add group: "), NULL, group, 1, 0) > 0)
      (void) slrn_add_group (group);
 }
 
@@ -1283,7 +1290,7 @@ static void group_down (void) /*{{{*/
 {
    if (1 != slrn_group_down_n (1))
      {
-	slrn_error ("End of Buffer.");
+	slrn_error (_("End of Buffer."));
      }
 }
 
@@ -1344,7 +1351,7 @@ static void move_group_cmd (void) /*{{{*/
    
    while (1)
      {
-	slrn_message_now ("Moving %s. Press RETURN when finished.", Slrn_Group_Current_Group->group_name);
+	slrn_message_now (_("Moving %s. Press RETURN when finished."), Slrn_Group_Current_Group->group_name);
 	
 	key = SLang_do_key (Slrn_Group_Keymap, (int (*)(void)) SLang_getkey);
 	
@@ -1427,7 +1434,7 @@ static void subscribe (void) /*{{{*/
    
    Slrn_Prefix_Arg_Ptr = NULL;
    	
-   if (NULL == (re = read_group_regexp ("Subscribe pattern: ", NULL)))
+   if (NULL == (re = read_group_regexp (_("Subscribe pattern: "), NULL)))
      return;
 
    g = Groups;
@@ -1456,11 +1463,11 @@ static void catch_up (void) /*{{{*/
    if ((Slrn_Group_Current_Group == NULL)
        || (Slrn_User_Wants_Confirmation
 	   && (Slrn_Batch == 0)
-	   && slrn_get_yesno(1, "Mark %s as read", Slrn_Group_Current_Group->group_name) <= 0))
+	   && slrn_get_yesno(1, _("Mark %s as read"), Slrn_Group_Current_Group->group_name) <= 0))
      return;
    
    slrn_catchup_group ();
-   slrn_message ("Group marked as read.");
+   slrn_message (_("Group marked as read."));
    (void) slrn_group_down_n (1);
 }
 
@@ -1471,11 +1478,11 @@ static void uncatch_up (void) /*{{{*/
    if ((Slrn_Group_Current_Group == NULL)
        || (Slrn_User_Wants_Confirmation 
 	   && (Slrn_Batch == 0)
-	   && slrn_get_yesno(1, "Mark %s as un-read", Slrn_Group_Current_Group->group_name) <= 0))
+	   && slrn_get_yesno(1, _("Mark %s as un-read"), Slrn_Group_Current_Group->group_name) <= 0))
      return;
  
    slrn_uncatchup_group ();   
-   slrn_message ("Group marked as un-read.");
+   slrn_message (_("Group marked as un-read."));
    (void) slrn_group_down_n (1);
 }
 
@@ -1498,7 +1505,7 @@ static void unsubscribe (void) /*{{{*/
    
    Slrn_Prefix_Arg_Ptr = NULL;
    	
-   if (NULL == (re = read_group_regexp ("Un-Subscribe pattern: ", NULL)))
+   if (NULL == (re = read_group_regexp (_("Un-Subscribe pattern: "), NULL)))
      return;
 
    g = Groups;
@@ -1565,7 +1572,7 @@ static void toggle_list_all_groups1 (int hide_flag) /*{{{*/
 	SLRegexp_Type *re;
 	char origpat[256];
 	
-	if (NULL == (re = read_group_regexp ("List Groups (e.g., comp*unix*): ",
+	if (NULL == (re = read_group_regexp (_("List Groups (e.g., comp*unix*): "),
 					     origpat)))
 	  {
 	     all_hidden = 1;
@@ -1584,7 +1591,7 @@ static void toggle_list_all_groups1 (int hide_flag) /*{{{*/
 		  unsigned int len;
 		  int min, max;
 
-		  parse_active_line (buf, &len, &min, &max);
+		  parse_active_line ((unsigned char *)buf, &len, &min, &max);
 		  g = create_group_entry (buf, len, min, max, 0, 0);
 		  
 		  if (g != NULL)
@@ -1714,7 +1721,7 @@ void slrn_hide_groups (int mode)
 static void select_group_cmd (void)
 {
    if (-1 == slrn_group_select_group ())
-     slrn_error ("No unread articles.");
+     slrn_error (_("No unread articles."));
 }
 
 void slrn_post_cmd (void) /*{{{*/
@@ -1726,18 +1733,18 @@ void slrn_post_cmd (void) /*{{{*/
    
    if (Slrn_Post_Obj->po_can_post == 0)
      {
-	slrn_error ("Posting not allowed.");
+	slrn_error (_("Posting not allowed."));
 	return;
      }
 
 #if SLRN_HAS_SLANG
-   SLang_run_hooks ("post_hook", 0);
+   slrn_run_hooks (HOOK_POST, 0);
    if (SLang_Error)
      return;
 #endif
 
    if (Slrn_User_Wants_Confirmation && (Slrn_Batch == 0) &&
-       (slrn_get_yesno (1, "Are you sure that you want to post") <= 0))
+       (slrn_get_yesno (1, _("Are you sure that you want to post")) <= 0))
      return;
    
    if (Slrn_Group_Current_Group == NULL)
@@ -1747,16 +1754,16 @@ void slrn_post_cmd (void) /*{{{*/
    if (strlen (name) >= sizeof (group))
      name = "";
    slrn_strncpy (group, name, sizeof (group));
-   if (slrn_read_input ("Newsgroup: ", NULL, group, 1, 1) <= 0) return;
+   if (slrn_read_input (_("Newsgroup: "), NULL, group, 1, 1) <= 0) return;
    if (strchr (group, ',') != NULL)
      {
 	slrn_strncpy (followupto, name, sizeof (followupto));
-	(void) slrn_read_input ("Followup-To: ", NULL, followupto, 1, 1);
+	(void) slrn_read_input (_("Followup-To: "), NULL, followupto, 1, 1);
      }
    else
      *followupto = '\0';
    
-   *subj = 0; if (slrn_read_input ("Subject: ", NULL, subj, 1, 0) <= 0) return;
+   *subj = 0; if (slrn_read_input (_("Subject: "), NULL, subj, 1, 0) <= 0) return;
    
    (void) slrn_post (group, followupto, subj);
 }
@@ -1765,28 +1772,37 @@ void slrn_post_cmd (void) /*{{{*/
 
 static void toggle_scoring (void) /*{{{*/
 {
+   /* Note to translators: Here, "fF" means "full", "sS" is "simple",
+    * "nN" means "none" and "cC" is "cancel".
+    * As always, don't change the length of the string; you cannot use
+    * the default characters for different fields.
+    */
+   char rsp, *responses = _("fFsSnNcC");
+   
    if (-1 == slrn_check_batch ())
      return;
-
-   switch (slrn_get_response ("FfSsNnCc\007",
-			      "Select scoring mode: \001Full, \001Simple, \001None, \001Cancel"))
+   if (strlen (responses) != 8)
+     responses = "";
+   rsp = slrn_get_response ("fFsSnNcC\007", responses,
+    _("Select scoring mode: \001Full, \001Simple, \001None, \001Cancel"));
+   
+   if (rsp != 7)
+     rsp = slrn_map_translated_char ("fFsSnNcC", responses, rsp) | 0x20;
+   switch (rsp)
      {
-      case 'F':
       case 'f':
 	Slrn_Perform_Scoring = SLRN_XOVER_SCORING | SLRN_EXPENSIVE_SCORING;
-	slrn_message ("Full Header Scoring enabled.");
+	slrn_message (_("Full Header Scoring enabled."));
 	break;
 	
-      case 'S':
       case 's':
 	Slrn_Perform_Scoring = SLRN_XOVER_SCORING;
-	slrn_message ("Expensive Scoring disabled.");
+	slrn_message (_("Expensive Scoring disabled."));
 	break;
 	
-      case 'N':
       case 'n':
 	Slrn_Perform_Scoring = 0;
-	slrn_message ("Scoring disabled.");
+	slrn_message (_("Scoring disabled."));
 	break;
 	
       default:
@@ -1796,16 +1812,15 @@ static void toggle_scoring (void) /*{{{*/
 }
 
 /*}}}*/
-
 static void save_newsrc_cmd (void) /*{{{*/
 {
    if (Slrn_Groups_Dirty)
      {
-	slrn_write_newsrc ();
+	slrn_write_newsrc (0);
      }
    else
      {
-	slrn_message ("No changes need to be saved.");
+	slrn_message (_("No changes need to be saved."));
      }
    slrn_smg_refresh ();
 }
@@ -1818,14 +1833,14 @@ static void save_newsrc_cmd (void) /*{{{*/
 
 static void slrn_group_hup (int sig)
 {
-   slrn_write_newsrc ();
+   slrn_write_newsrc (0);
    slrn_quit (sig);
 }
 
 static void enter_group_mode_hook (void)
 {
 #if SLRN_HAS_SLANG
-   SLang_run_hooks ("group_mode_hook", 0);
+   slrn_run_hooks (HOOK_GROUP_MODE, 0);
 #endif
 }
 
@@ -1996,7 +2011,7 @@ static void test_function (void)
      
 void slrn_init_group_mode (void) /*{{{*/
 {
-   char  *err = "Unable to create group keymap!";
+   char  *err = _("Unable to create group keymap!");
    
    if (NULL == (Slrn_Group_Keymap = SLang_create_keymap ("group", NULL)))
      slrn_exit_error (err);
@@ -2082,6 +2097,8 @@ void slrn_init_group_mode (void) /*{{{*/
    SLkm_define_key  ("\033[M\040", (FVOID_STAR) group_mouse_left, Slrn_Group_Keymap);
    SLkm_define_key  ("\033[M\041", (FVOID_STAR) group_mouse_middle, Slrn_Group_Keymap);
    SLkm_define_key  ("\033[M\042", (FVOID_STAR) group_mouse_right, Slrn_Group_Keymap);
+   SLkm_define_key  ("\033[M\043", (FVOID_STAR) group_mouse_left, Slrn_Group_Keymap);
+   SLkm_define_key  ("\033[M\044", (FVOID_STAR) group_mouse_left, Slrn_Group_Keymap);
 #if USE_TEST_FUNCTION
    SLkm_define_key  ("y", (FVOID_STAR) test_function, Slrn_Group_Keymap);
 #endif
@@ -2152,19 +2169,19 @@ void slrn_get_group_descriptions (void) /*{{{*/
    
    if (NULL == (fp = fopen (file, "w")))
      {
-	slrn_exit_error ("\
-Unable to create newsgroup description file:\n%s\n", file);
+	slrn_exit_error (_("\
+Unable to create newsgroup description file:\n%s\n"), file);
      }
    
-   fprintf (stdout, "\nCreating description file %s.\n", file);
-   fprintf (stdout, "Getting newsgroup descriptions from server.\n\
-Note: This step may take some time if you have a slow connection!!!\n");
+   fprintf (stdout, _("\nCreating description file %s.\n"), file);
+   fprintf (stdout, _("Getting newsgroup descriptions from server.\n\
+Note: This step may take some time if you have a slow connection!!!\n"));
    
    fflush (stdout);
    
    if (OK_GROUPS != Slrn_Server_Obj->sv_list_newsgroups ())
      {
-	slrn_error ("Server failed on list newsgroups command.");
+	slrn_error (_("Server failed on list newsgroups command."));
 	slrn_fclose (fp);
 	return;
      }
@@ -2179,7 +2196,7 @@ Note: This step may take some time if you have a slow connection!!!\n");
 	status = Slrn_Server_Obj->sv_read_line (line, sizeof (line));
 	if (status == -1)
 	  {
-	     slrn_error ("Error reading line from server");
+	     slrn_error (_("Error reading line from server"));
 	     slrn_fclose (fp);
 	     return;
 	  }
@@ -2238,23 +2255,20 @@ int slrn_read_group_descriptions (void) /*{{{*/
    
    if (NULL == (fp = fopen (file, "r")))
      {
-	if (Slrn_Lib_Dir != NULL)
-	  {
 #ifdef VMS
-	     slrn_snprintf (file, sizeof (file), "%snewsgroups.dsc",
-			    Slrn_Lib_Dir);
-	     if (NULL == (fp = fopen (file, "r")))
-	       {
-		  slrn_snprintf (file, sizeof (file), "%snewsgroups-dsc",
-				 Slrn_Lib_Dir);
-		  fp = fopen (file, "r");
-	       }
-#else
-	     slrn_snprintf (file, sizeof (file), "%s/newsgroups.dsc",
-			    Slrn_Lib_Dir);
+	slrn_snprintf (file, sizeof (file), "%snewsgroups.dsc",
+		       SHAREDIR);
+	if (NULL == (fp = fopen (file, "r")))
+	  {
+	     slrn_snprintf (file, sizeof (file), "%snewsgroups-dsc",
+			    SHAREDIR);
 	     fp = fopen (file, "r");
-#endif
 	  }
+#else
+	slrn_snprintf (file, sizeof (file), "%s/newsgroups.dsc",
+		       SHAREDIR);
+	fp = fopen (file, "r");
+#endif
 	if (fp == NULL) return -1;
      }
    
@@ -2313,7 +2327,7 @@ int slrn_get_new_groups (int create_flag) /*{{{*/
 	fgets (line, sizeof (line), fp);
 	slrn_fclose (fp);
 	
-	slrn_message_now ("Checking for new groups ...");
+	slrn_message_now (_("Checking for new groups ..."));
 	
 	time (&tloc);
 	parse_error = 1;
@@ -2371,11 +2385,11 @@ int slrn_get_new_groups (int create_flag) /*{{{*/
 	     return -1;
 	     
 	   case ERR_COMMAND:
-	     slrn_message ("Server does not implement NEWGROUPS command.");
+	     slrn_message (_("Server does not implement NEWGROUPS command."));
 	     return 0;
 	     
 	   default:
-	     slrn_message ("Server failed to return proper response to NEWGROUPS:\n%s\n",
+	     slrn_message (_("Server failed to return proper response to NEWGROUPS:\n%s\n"),
 			   line);
 	     goto parse_error_label;
 	  }
@@ -2389,18 +2403,18 @@ int slrn_get_new_groups (int create_flag) /*{{{*/
 	       break;
 	     if (status == -1)
 	       {
-		  slrn_error ("Read from server failed");
+		  slrn_error (_("Read from server failed"));
 		  return -1;
 	       }
 
 	     /* line contains new newsgroup name */
-	     add_unsubscribed_group (line);
+	     add_unsubscribed_group ((unsigned char*)line);
 	     num++;
 	  }
 	
 	if (num)
 	  {
-	     slrn_message ("%d new newsgroup(s) found.", num);
+	     slrn_message (_("%d new newsgroup(s) found."), num);
 	  }
      }
    else time (&tloc);
@@ -2408,10 +2422,10 @@ int slrn_get_new_groups (int create_flag) /*{{{*/
    parse_error_label:
    if (parse_error)
      {
-	slrn_message ("\
+	slrn_message (_("\
 %s appears corrupt.\n\
-I expected to see see: NEWGROUPS yymmdd hhmmss GMT\n\
-I will patch the file up for you.\n", file);
+I expected to see: NEWGROUPS yymmdd hhmmss GMT\n\
+I will patch the file up for you.\n"), file);
      }
    
 #ifdef VMS
@@ -2420,7 +2434,7 @@ I will patch the file up for you.\n", file);
    if (NULL == (fp = fopen (file, "w")))
 #endif
      {
-	slrn_exit_error ("Unable to open %s to record date.", file);
+	slrn_exit_error (_("Unable to open %s to record date."), file);
      }
 
    /* According to rfc977, the year must be in the form YY with the closest
@@ -2449,19 +2463,19 @@ I will patch the file up for you.\n", file);
 
 /*}}}*/
 
-static int parse_active_line (char *name, unsigned int *lenp, /*{{{*/
+static int parse_active_line (unsigned char *name, unsigned int *lenp, /*{{{*/
 			      int *minp, int *maxp)
 {
-   char *p;
+   unsigned char *p;
    
    p = name;
    while (*p > ' ') p++;
    *lenp = (unsigned int) (p - name);
    
    while (*p == ' ') p++;
-   *maxp = atoi (p);
+   *maxp = atoi ((char*)p);
    while (*p > ' ') p++;  while (*p == ' ') p++;
-   *minp = atoi(p);
+   *minp = atoi((char*)p);
    if (*maxp < *minp) *minp = *maxp + 1;
    return 0;
 }
@@ -2488,8 +2502,8 @@ static void read_and_parse_active (int create_flag) /*{{{*/
    if (OK_GROUPS != (ret = Slrn_Server_Obj->sv_list_active (NULL)))
      {
 	if (ret == ERR_NOAUTH)
-	  slrn_exit_error ("Server failed LIST ACTIVE - authorization missing.");
-	slrn_exit_error ("Server failed LIST ACTIVE.");
+	  slrn_exit_error (_("Server failed LIST ACTIVE - authorization missing."));
+	slrn_exit_error (_("Server failed LIST ACTIVE."));
      }
    
    while (1)
@@ -2502,12 +2516,12 @@ static void read_and_parse_active (int create_flag) /*{{{*/
 	if (status == -1)
 	  {
 	     Slrn_Groups_Dirty = 0;
-	     slrn_exit_error ("Read from server failed");
+	     slrn_exit_error (_("Read from server failed"));
 	  }
 	if (status == 0)
 	  break;
 
-	parse_active_line (line, &len, &min, &max);
+	parse_active_line ((unsigned char *)line, &len, &min, &max);
 	
 	if (!initial_run)
 	  {
@@ -2581,13 +2595,43 @@ static int read_and_parse_newsrc_file (void)
    char *vline;
    unsigned int vlen;
    char file[SLRN_MAX_PATH_LEN];
+   char *newsrc_filename = Slrn_Newsrc_File;
    Slrn_Group_Type *last_group = NULL;
+   int ret_stat_o, ret_stat_as;
+   struct stat st_o, st_as;
 
-   if ((NULL == (vp = vopen (Slrn_Newsrc_File, 4096, 0)))
+#ifdef VMS
+   slrn_snprintf (file, sizeof (file), "%s-as", Slrn_Newsrc_File);
+#else
+# ifdef SLRN_USE_OS2_FAT
+   slrn_os2_make_fat (file, sizeof (file), Slrn_Newsrc_File, ".as");
+# else
+   slrn_snprintf (file, sizeof (file), "%s.as", Slrn_Newsrc_File);
+# endif
+#endif
+
+   ret_stat_o = stat (newsrc_filename, &st_o);
+   ret_stat_as = stat (file, &st_as); 
+   
+   if ((ret_stat_as != -1) &&
+       (((ret_stat_o == -1) || (st_as.st_mtime > st_o.st_mtime))))
+     {
+	slrn_message (_("\n* The autosave file of %s is newer than the file "
+		      "itself.\n"), newsrc_filename);
+	if (slrn_get_yesno (1, _("Do you want to restore your newsrc from "
+			    "the autosave version")))
+	  {
+	     newsrc_filename = file;
+	  }
+     }
+   
+   if (NULL == (vp = vopen (newsrc_filename, 4096, 0)))
+#if 0
        && (NULL == (vp = slrn_open_home_vfile (".newsrc", file,
-					       sizeof (file)))))
-     slrn_exit_error ("Unable to open %s.", file);
-
+					       sizeof (file))))
+#endif
+       slrn_exit_error (_("Unable to open %s."), newsrc_filename);
+   
    while (NULL != (vline = vgets (vp, &vlen)))
      {
 	char *p = vline;
@@ -2614,14 +2658,15 @@ static int read_and_parse_newsrc_file (void)
 	  last_group = place_group_in_newsrc_order (last_group);
      }
    vclose (vp);
-   refresh_groups (&Slrn_Group_Current_Group);
+   if (!Slrn_List_Active_File)
+     refresh_groups (&Slrn_Group_Current_Group);
    return 0;
 }
    
 int slrn_read_newsrc (int create_flag) /*{{{*/
 {
-   slrn_message_now ("Checking news%s ...",
-		     Slrn_List_Active_File ? " via active file" : "");
+   slrn_message_now (_("Checking news%s ..."),
+		     Slrn_List_Active_File ? _(" via active file") : "");
    
    if (create_flag)
      {
@@ -2634,11 +2679,11 @@ int slrn_read_newsrc (int create_flag) /*{{{*/
 	if (NULL == (fp = fopen (Slrn_Newsrc_File, "w")))
 #endif
 	  {
-	     slrn_exit_error ("Unable to create %s.", Slrn_Newsrc_File);
+	     slrn_exit_error (_("Unable to create %s."), Slrn_Newsrc_File);
 	  }
 	fclose (fp);
-	fputs ("\n--The next step may take a while if the NNTP connection is slow.--\n\n", stdout);
-	fprintf (stdout, "Creating %s.", Slrn_Newsrc_File);
+	fputs (_("\n--The next step may take a while if the NNTP connection is slow.--\n\n"), stdout);
+	fprintf (stdout, _("Creating %s."), Slrn_Newsrc_File);
 	fflush (stdout);
      }
    
@@ -2649,7 +2694,7 @@ int slrn_read_newsrc (int create_flag) /*{{{*/
    
    if ((create_flag == 0)
        && (-1 == read_and_parse_newsrc_file ()))
-     slrn_exit_error ("Unable to read newsrc file");
+     slrn_exit_error (_("Unable to read newsrc file"));
    
    insert_new_groups ();
    
@@ -2684,17 +2729,18 @@ int slrn_read_newsrc (int create_flag) /*{{{*/
 
 /*}}}*/
 
-int slrn_write_newsrc (void) /*{{{*/
+int slrn_write_newsrc (int auto_save) /*{{{*/
+/* auto_save == 0: save to newsrc, == 1: save to autosave file. */
 {
    Slrn_Group_Type *g;
    Slrn_Range_Type *r;
-   char backup_file[SLRN_MAX_PATH_LEN];
+   char backup_file[SLRN_MAX_PATH_LEN], autosave_file[SLRN_MAX_PATH_LEN];
+   char *newsrc_filename;
    static FILE *fp;
-   int pass;
    int max;
    struct stat filestat;
-   int stat_worked;
-   int have_backup;
+   int stat_worked = 0;
+   int have_backup = 0;
 
    slrn_init_hangup_signals (0);
    
@@ -2708,38 +2754,63 @@ int slrn_write_newsrc (void) /*{{{*/
     * This will not hurt since we are going to do it again anyway.
     */
    if (fp != NULL) slrn_fclose (fp); fp = NULL;
-   
-   slrn_message_now ("Writing %s ...", Slrn_Newsrc_File);
-   
-   /* Try to preserve .newsrc permissions and owner/group.  This also 
-    * confirms existence of file.
-    */
-   stat_worked = (-1 != stat (Slrn_Newsrc_File, &filestat));
-  
+
 #ifdef VMS
-   slrn_snprintf (backup_file, sizeof (backup_file), "%s-bak", Slrn_Newsrc_File);
+   slrn_snprintf (autosave_file, sizeof (autosave_file), "%s-as",
+		  Slrn_Newsrc_File);
+   slrn_snprintf (backup_file, sizeof (backup_file), "%s-bak",
+		  Slrn_Newsrc_File);
 #else
 # ifdef SLRN_USE_OS2_FAT
-   slrn_os2_make_fat (backup_file, sizeof (backup_file), Slrn_Newsrc_File, ".bak");
+   slrn_os2_make_fat (autosave_file, sizeof (autosave_file), Slrn_Newsrc_File,
+		      ".as");
+   slrn_os2_make_fat (backup_file, sizeof (backup_file), Slrn_Newsrc_File,
+		      ".bak");
 # else
+   slrn_snprintf (autosave_file, sizeof (autosave_file), "%s.as",
+		  Slrn_Newsrc_File);
    slrn_snprintf (backup_file, sizeof (backup_file), "%s~", Slrn_Newsrc_File);
 # endif
 #endif
    
-   /* Create a temp backup file.  Delete it later if user 
-    * does not want backups.
-    */
-   have_backup = 1;
-   if (-1 == rename (Slrn_Newsrc_File, backup_file))
+   if ((auto_save == 1) && (Slrn_No_Autosave != 2))
      {
-	have_backup = 0;
+	if (Slrn_No_Autosave)
+	  {
+	     slrn_init_hangup_signals (1);
+	     return 0;
+	  }
+	
+	newsrc_filename = autosave_file;
+	stat_worked = (-1 != stat (newsrc_filename, &filestat));
      }
+   else
+     newsrc_filename = Slrn_Newsrc_File;
+
+   slrn_message_now (_("Writing %s ..."), newsrc_filename);
+   /* Try to preserve .newsrc permissions and owner/group.  This also
+    * confirms existence of file.  If an autosave file does not (yet)
+    * exist, use permissions of the .newsrc.
+    */
+   if (stat_worked == 0)
+     stat_worked = (-1 != stat (Slrn_Newsrc_File, &filestat));
    
-   
-   if (NULL == (fp = fopen (Slrn_Newsrc_File, "w")))
+   if (newsrc_filename == Slrn_Newsrc_File)
      {
-	slrn_error ("Unable to open file %s for writing.", Slrn_Newsrc_File);
-	if (have_backup) (void) rename (backup_file, Slrn_Newsrc_File);
+	/* Create a temp backup file.  Delete it later if user 
+	 * does not want backups.
+	 */
+	have_backup = 1;
+	if (-1 == rename (newsrc_filename, backup_file))
+	  {
+	     have_backup = 0;
+	  }
+     }
+
+   if (NULL == (fp = fopen (newsrc_filename, "w")))
+     {
+	slrn_error (_("Unable to open file %s for writing."), newsrc_filename);
+	if (have_backup) (void) rename (backup_file, newsrc_filename);
 	slrn_init_hangup_signals (1);
 	return -1;
      }
@@ -2754,105 +2825,75 @@ int slrn_write_newsrc (void) /*{{{*/
 #  endif
    if (stat_worked)
      {
-	if (-1 == chmod (Slrn_Newsrc_File, filestat.st_mode & (S_IRUSR | S_IWUSR | S_IXUSR)))
-	  (void) chmod (Slrn_Newsrc_File, S_IWUSR | S_IRUSR);
-
- 	(void) chown (Slrn_Newsrc_File, filestat.st_uid, filestat.st_gid);
+	if (-1 == chmod (newsrc_filename, filestat.st_mode & (S_IRUSR | S_IWUSR | S_IXUSR)))
+	  (void) chmod (newsrc_filename, S_IWUSR | S_IRUSR);
+	
+	(void) chown (newsrc_filename, filestat.st_uid, filestat.st_gid);
      }
 # endif
 #endif
    
-   
-   /* We are going to do this in 2 passes.  The first pass writes out just
-    * the subscribed groups.  The second pass takes care of the rest.
-    */
-   for (pass = 0; pass < 2; pass++)
+   g = Groups;
+   while (g != NULL)
      {
-	g = Groups;
-	while (g != NULL)
+	if ((g->flags & GROUP_UNSUBSCRIBED) &&
+	    ((Slrn_Write_Newsrc_Flags == 1) ||
+	    ((Slrn_Write_Newsrc_Flags == 2) && (g->range.next == NULL))))
 	  {
-	     if (pass == 0)
-	       {
-		  if (g->flags & GROUP_UNSUBSCRIBED)
-		    {
-		       g = g->next;
-		       continue;
-		    }
-		  if ((EOF == fputs (g->group_name, fp))
-		      || (EOF == putc (':', fp)))
-		    goto write_error;
-	       }
-	     else if (pass == 1)
-	       {
-		  if ((g->flags & GROUP_UNSUBSCRIBED) == 0)
-		    {
-		       g = g->next;
-		       continue;
-		    }
-		  
-		  if (Slrn_Write_Newsrc_Flags)
-		    {
-		       if ((Slrn_Write_Newsrc_Flags == 1)
-			   || ((Slrn_Write_Newsrc_Flags == 2)
-			       && (g->range.next == NULL)))
-			 {
-			    g = g->next;
-			    continue;
-			 }
-		    }
-		  if ((EOF == fputs (g->group_name, fp))
-		      || (EOF == putc ('!', fp)))
-		    goto write_error;
-	       }
-	     
-	     r = g->range.next;
-	     max = g->range.max;
-	     if (r != NULL)
-	       {
-		  if (EOF == putc (' ', fp))
-		    goto write_error;
-		  
-		  while (1)
-		    {
-		       /* Make this check because the unsubscribed group
-			* range may not have been initialized from the server.
-			*/
-		       if ((max != -1) && (g->range.min != -1)
-			   && ((g->flags & GROUP_UNSUBSCRIBED) == 0))
-			 {
-			    if (r->min > max) break;
-			    if (r->max > max) r->max = max;
-			 }
-		       
-		       if (r->min != r->max)
-			 {
-			    if (fprintf (fp, "%d-%d", r->min, r->max) < 0)
-			      goto write_error;
-			 }
-		       else if (fprintf (fp, "%d", r->min) < 0)
-			 goto write_error;
-		       
-		       r = r->next;
-		       if (r == NULL) break;
-		       if (EOF == putc (',', fp))
-			 goto write_error;
-		    }
-	       }
-	     else if (g->range.min == 2)
-	       {
-		  if (EOF == fputs (" 1", fp))
-		    goto write_error;
-	       }
-	     else if (g->range.min > 2)
-	       {
-		  if (fprintf (fp, " %d-%d", 1, g->range.min - 1) < 0)
-		    goto write_error;
-	       }
-	     
-	     if (EOF == putc ('\n', fp))
-	       goto write_error;
 	     g = g->next;
+	     continue;
 	  }
+	if ((EOF == fputs (g->group_name, fp)) ||
+	    (EOF == putc (((g->flags & GROUP_UNSUBSCRIBED) ? '!' : ':'), fp)))
+	  goto write_error;
+	
+	r = g->range.next;
+	max = g->range.max;
+	if (r != NULL)
+	  {
+	     if (EOF == putc (' ', fp))
+	       goto write_error;
+	     
+	     while (1)
+	       {
+		  /* Make this check because the unsubscribed group
+		   * range may not have been initialized from the server.
+		   */
+		  if ((max != -1) && (g->range.min != -1)
+		      && ((g->flags & GROUP_UNSUBSCRIBED) == 0))
+		    {
+		       if (r->min > max) break;
+		       if (r->max > max) r->max = max;
+		    }
+		  
+		  if (r->min != r->max)
+		    {
+		       if (fprintf (fp, "%d-%d", r->min, r->max) < 0)
+			 goto write_error;
+		    }
+		  else if (fprintf (fp, "%d", r->min) < 0)
+		    goto write_error;
+		  
+		  r = r->next;
+		  if (r == NULL) break;
+		  if (EOF == putc (',', fp))
+		    goto write_error;
+	       }
+	  }
+	else if (g->range.min == 2)
+	  {
+	     if (EOF == fputs (" 1", fp))
+	       goto write_error;
+	  }
+	else if (g->range.min > 2)
+	  {
+	     if (fprintf (fp, " %d-%d", 1, g->range.min - 1) < 0)
+	       goto write_error;
+	  }
+	
+	if (EOF == putc ('\n', fp))
+	  goto write_error;
+	g = g->next;
      }
    
    if (-1 == slrn_fclose (fp))
@@ -2865,9 +2906,14 @@ int slrn_write_newsrc (void) /*{{{*/
 	if (have_backup) slrn_delete_file (backup_file);
      }
    
-   Slrn_Groups_Dirty = 0;
+   if (newsrc_filename == Slrn_Newsrc_File)
+     {
+	Slrn_Groups_Dirty = 0;
+	if (Slrn_No_Autosave == 0)
+	  slrn_delete_file (autosave_file);
+     }
    if (Slrn_TT_Initialized & SLRN_TTY_INIT)
-     slrn_message ("Writing %s ... done.", Slrn_Newsrc_File);
+     slrn_message (_("Writing %s ... done."), newsrc_filename);
 
    slrn_init_hangup_signals (1);
    return 0;
@@ -2875,13 +2921,10 @@ int slrn_write_newsrc (void) /*{{{*/
    write_error:
    
    slrn_fclose (fp); fp = NULL;
-   slrn_error ("Write to %s failed! Disk Full?", Slrn_Newsrc_File);
+   slrn_error (_("Write to %s failed! Disk Full?"), newsrc_filename);
    
-   if (stat_worked)
-     {
-	/* Put back orginal file */
-	if (have_backup) (void) rename (backup_file, Slrn_Newsrc_File);
-     }
+   /* Put back orginal file */
+   if (have_backup) (void) rename (backup_file, newsrc_filename);
    
    slrn_init_hangup_signals (1);
    return -1;
@@ -2893,7 +2936,7 @@ int slrn_write_newsrc (void) /*{{{*/
 
 static void group_quick_help (void) /*{{{*/
 {
-   char *hlp = "SPC:Select  p:Post  c:CatchUp  l:List  q:Quit  ^R:Redraw  (u)s:(Un)Subscribe";
+   char *hlp = _("SPC:Select  p:Post  c:CatchUp  l:List  q:Quit  ^R:Redraw  (u)s:(Un)Subscribe");
    
    if (Slrn_Batch)
      return;
@@ -2997,7 +3040,7 @@ static char *group_status_line_cb (char ch, void *data, int *len, int *color) /*
 	retval = slrn_print_percent (buf, &Group_Window, 0);
 	break;
       case 'D':
-	retval = Slrn_Groups_Dirty ? "*" : " ";
+	retval = Slrn_Groups_Dirty ? "*" : "-";
 	*len = 1;
 	break;
       case 's':
@@ -3065,7 +3108,7 @@ static void group_update_screen (void) /*{{{*/
    
    fmt = Slrn_Group_Status_Line;
    if ((fmt == NULL) || (*fmt == 0))
-     fmt = "-%D-News Groups: %s %-20g -- %L (%P)";
+     fmt = _("-%D-News Groups: %s %-20g -- %L (%P)");
    slrn_custom_printf (fmt, group_status_line_cb, NULL, SLtt_Screen_Rows - 2,
 		       STATUS_COLOR);
    
@@ -3097,7 +3140,7 @@ static void group_update_screen (void) /*{{{*/
 void slrn_intr_get_group_order (void) /*{{{*/
 {
 # if SLANG_VERSION < 10400
-   slrn_error ("To use this feature, please update s-lang and recompile slrn.");
+   slrn_error (_("To use this feature, please update s-lang and recompile slrn."));
 # else
    Slrn_Group_Type *g;
    SLang_Array_Type *retval;
@@ -3135,7 +3178,7 @@ void slrn_intr_get_group_order (void) /*{{{*/
 void slrn_intr_set_group_order (void) /*{{{*/
 {
 #if SLANG_VERSION < 10400
-   slrn_error ("To use this feature, please update s-lang and recompile slrn.");
+   slrn_error (_("To use this feature, please update s-lang and recompile slrn."));
 # else
    SLang_Array_Type *at;
    Slrn_Group_Type *last = NULL, *rest = Groups;
@@ -3143,13 +3186,13 @@ void slrn_intr_set_group_order (void) /*{{{*/
    
    if (-1 == SLang_pop_array_of_type (&at, SLANG_STRING_TYPE))
      {
-	slrn_error ("Array of string expected.");
+	slrn_error (_("Array of string expected."));
 	return;
      }
    
    if (at->num_dims != 1)
      {
-	slrn_error ("One-dimensional array expected.");
+	slrn_error (_("One-dimensional array expected."));
 	SLang_free_array (at);
 	return;
      }
