@@ -27,7 +27,7 @@ extern int slrn_group_select_group (void);
 extern void slrn_list_all_groups (int);
 extern void slrn_hide_groups (int);
 extern int slrn_read_newsrc (int);
-extern int slrn_write_newsrc (void);
+extern int slrn_write_newsrc (int);
 extern void slrn_mark_article_as_read (char *, long);
 extern int slrn_get_new_groups (int);
 extern void slrn_init_group_mode (void);
@@ -88,6 +88,7 @@ extern char *Slrn_Group_Status_Line;
 extern int Slrn_Groups_Dirty;
 extern int *Slrn_Prefix_Arg_Ptr;
 extern int Slrn_No_Backups;
+extern int Slrn_No_Autosave;
 extern int Slrn_Unsubscribe_New_Groups;
 extern int Slrn_Check_New_Groups;
 extern int Slrn_List_Active_File;
