@@ -54,6 +54,7 @@
 #include "snprintf.h"
 #include "mime.h"
 #include "version.h"
+#include "hooks.h"
 
 /*}}}*/
 
@@ -100,7 +101,7 @@ static char **pop_argv_list (unsigned int *argcp)
 
    if (n < 0)
      {
-	slrn_error ("positive integer expected");
+	slrn_error (_("positive integer expected"));
 	return NULL;
      }
 
@@ -136,7 +137,7 @@ static int interp_select_box (void) /*{{{*/
 
    if (Slrn_Batch)
      {
-	slrn_error ("select box function not available in batch mode.");
+	slrn_error (_("select box function not available in batch mode."));
 	return -1;
      }
 
@@ -270,7 +271,7 @@ int slrn_eval_slang_file (char *name) /*{{{*/
 	     if ((-1 != slrn_dircat (dir, name, file, sizeof (file)))
 		 && (1 == slrn_file_exists (file)))
 	       {
-		  slrn_message_now ("loading %s", file);
+		  slrn_message_now (_("loading %s"), file);
 		  if (-1 == SLang_load_file (file))
 		    return -1;
 		  return 0;
@@ -280,7 +281,7 @@ int slrn_eval_slang_file (char *name) /*{{{*/
      }
    
    slrn_make_home_filename (name, file, sizeof (file));
-   slrn_message_now ("loading %s", file);
+   slrn_message_now (_("loading %s"), file);
    if (0 == SLang_load_file (file))
      return -1;
    return 0;
@@ -297,7 +298,7 @@ int slrn_eval_slang_file (char *name) /*{{{*/
 static void set_string_variable (char *s1, char *s2) /*{{{*/
 {
    if (-1 == slrn_set_string_variable (s1, s2))
-     slrn_error ("%s is not a valid variable name.", s1);
+     slrn_error (_("%s is not a valid variable name."), s1);
 }
 
 /*}}}*/
@@ -305,7 +306,7 @@ static void set_string_variable (char *s1, char *s2) /*{{{*/
 static void set_integer_variable (char *s1, int *val) /*{{{*/
 {
    if (-1 == slrn_set_integer_variable (s1, *val))
-     slrn_error ("%s is not a valid variable name.", s1);
+     slrn_error (_("%s is not a valid variable name."), s1);
 }
 
 /*}}}*/
@@ -318,7 +319,7 @@ static void get_variable_value (char *name) /*{{{*/
    
    if (-1 == slrn_get_variable_value (name, &type, &s, &ip))
      {
-	slrn_error ("%s is not a valid variable name.", name);
+	slrn_error (_("%s is not a valid variable name."), name);
 	return;
      }
    
@@ -346,7 +347,7 @@ static char *interp_setlocale (int *category, char *locale) /*{{{*/
    return (ret == NULL) ? "" : ret;
 #else
    (void) category; (void) locale;
-   slrn_error ("setlocale is not available on this system.");
+   slrn_error (_("setlocale is not available on this system."));
    return "";
 #endif
 }
@@ -356,23 +357,23 @@ static char *interp_setlocale (int *category, char *locale) /*{{{*/
 static void set_utf8_conversion_table (void) /*{{{*/
 {
 #if SLANG_VERSION < 10400
-   slrn_error ("To use this feature, please update s-lang and recompile slrn.");
+   slrn_error (_("To use this feature, please update s-lang and recompile slrn."));
 #else
 # if ! SLRN_HAS_MIME
-   slrn_error ("This version of slrn lacks MIME support.");
+   slrn_error (_("This version of slrn lacks MIME support."));
 # else
    SLang_Array_Type *at;
    int rows;
    
    if (-1 == SLang_pop_array_of_type (&at, SLANG_INT_TYPE))
      {
-	slrn_error ("Array of integer expected.");
+	slrn_error (_("Array of integer expected."));
 	return;
      }
    
    if ((at->num_dims != 2) || (at->dims[0] != 2))
      {
-	slrn_error ("Two-dimensional array with two columns expected.");
+	slrn_error (_("Two-dimensional array with two columns expected."));
 	goto free_and_return;
      }
    
@@ -384,7 +385,7 @@ static void set_utf8_conversion_table (void) /*{{{*/
    
    if (NULL == (Slrn_Utf8_Table = slrn_malloc (65536, 1, 0)))
      {
-	slrn_error ("Could not get memory for conversion table.");
+	slrn_error (_("Could not get memory for conversion table."));
 	goto free_and_return;
      }
    
@@ -399,7 +400,7 @@ static void set_utf8_conversion_table (void) /*{{{*/
 	
 	if (((unicode != 0) && (unicode < 128)) || (unicode > 65535))
 	  {
-	     slrn_error ("Unicode value in row %d out of range.", rows);
+	     slrn_error (_("Unicode value in row %d out of range."), rows);
 	     goto free_and_return;
 	  }
 	
@@ -408,7 +409,7 @@ static void set_utf8_conversion_table (void) /*{{{*/
 	
 	if ((local < 1) || (local > 255))
 	  {
-	     slrn_error ("Local charset value in row %d out of range.",
+	     slrn_error (_("Local charset value in row %d out of range."),
 			 rows);
 	     goto free_and_return;
 	  }
@@ -453,7 +454,7 @@ static void definekey (char *fun, char *key, char *map) /*{{{*/
    
    if (NULL == (kmap = SLang_find_keymap (map)))
      {
-	slrn_error ("definekey: no such keymap.");
+	slrn_error (_("definekey: no such keymap."));
 	return;
      }
    
@@ -475,9 +476,12 @@ static void undefinekey (char *key, char *map) /*{{{*/
 {
    SLKeyMap_List_Type *kmap;
    
+   if (!strcmp (map, "readline"))
+     map = "ReadLine";
+   
    if (NULL == (kmap = SLang_find_keymap (map)))
      {
-	error ("undefinekey: no such keymap.");
+	error (_("undefinekey: no such keymap."));
 	return;
      }
    
@@ -488,6 +492,7 @@ static void undefinekey (char *key, char *map) /*{{{*/
 
 #define PROMPT_NO_ECHO	1
 #define PROMPT_FILENAME	2
+#define PROMPT_VARIABLE 3
 static void generic_read_mini (int mode, char *prompt, char *dfl, char *init) /*{{{*/
 {
    char str[256];
@@ -496,16 +501,24 @@ static void generic_read_mini (int mode, char *prompt, char *dfl, char *init) /*
    strncpy (str, init, sizeof (str));
    str[sizeof(str) - 1] = 0;
    
-   if (mode == PROMPT_NO_ECHO)
-     ret = slrn_read_input_no_echo (prompt, dfl, str, 0, 0);
-   else if (mode == PROMPT_FILENAME)
-     ret = slrn_read_filename (prompt, dfl, str, 0, 0);
-   else
-     ret = slrn_read_input (prompt, dfl, str, 0, 0);
+   switch (mode)
+     {
+      case PROMPT_NO_ECHO:
+	ret = slrn_read_input_no_echo (prompt, dfl, str, 0, 0);
+	break;
+      case PROMPT_FILENAME:
+	ret = slrn_read_filename (prompt, dfl, str, 0, 0);
+	break;
+      case PROMPT_VARIABLE:
+	ret = slrn_read_variable (prompt, dfl, str, 0, 0);
+	break;
+      default:
+	ret = slrn_read_input (prompt, dfl, str, 0, 0);
+     }
    
    if (-1 == ret)
      {
-	error ("Quit!");
+	error (_("Quit!"));
 	return;
      }
    SLang_push_string (str);
@@ -532,6 +545,23 @@ static void read_mini_filename (char *prompt, char *dfl, char *init) /*{{{*/
 
 /*}}}*/
 
+static void read_mini_variable (char *prompt, char *dfl, char *init) /*{{{*/
+{
+   generic_read_mini (PROMPT_VARIABLE, prompt, dfl, init);
+}
+/*}}}*/
+
+static int read_mini_integer (char *prompt, int *dfl) /*{{{*/
+{
+   int status, retval;
+   status = slrn_read_integer (prompt, dfl, &retval);
+   if (-1 == status)
+     error (_("Quit!"));
+   slrn_clear_message ();
+   return retval;
+}
+/*}}}*/
+
 static void set_prefix_arg (int *arg) /*{{{*/
 {
    slrn_set_prefix_argument (*arg);
@@ -561,7 +591,7 @@ static int check_tty (void)
    if (Slrn_TT_Initialized & SLRN_TTY_INIT)
      return 0;
    
-   error ("Terminal not initialized.");
+   error (_("Terminal not initialized."));
    return -1;
 }
 
@@ -612,7 +642,7 @@ static int check_article_mode (void) /*{{{*/
    if ((Slrn_Current_Mode == NULL)
        || (Slrn_Current_Mode->mode != SLRN_ARTICLE_MODE))
      {
-	error ("Not in article mode.");
+	error (_("Not in article mode."));
 	return -1;
      }
    return 0;
@@ -722,7 +752,7 @@ static char *extract_current_article_header (char *h) /*{{{*/
      return NULL;
    if (NULL == Slrn_Current_Article)
      {
-	slrn_error ("No article selected");
+	slrn_error (_("No article selected"));
 	return NULL;
      }
 
@@ -910,14 +940,11 @@ static int get_header_flags (void) /*{{{*/
 
 static void set_header_flags (int *flagsp) /*{{{*/
 {
-   unsigned int flags;
    if ((-1 == check_article_mode ())
        || (Slrn_Current_Header == NULL))
      return;
    
-   Slrn_Current_Header->flags &= ~HEADER_HARMLESS_FLAGS_MASK;
-   flags = ((unsigned int) *flagsp) & HEADER_HARMLESS_FLAGS_MASK;
-   Slrn_Current_Header->flags |= flags;
+   slrn_set_header_flags (Slrn_Current_Header, (unsigned int) *flagsp);
 }
 
 /*}}}*/
@@ -1036,7 +1063,7 @@ static int check_group_mode (void)
    if ((Slrn_Current_Mode == NULL)
        || (Slrn_Current_Mode->mode != SLRN_GROUP_MODE))
      {
-	error ("Not in group mode.");
+	error (_("Not in group mode."));
 	return -1;
      }
    return 0;
@@ -1134,6 +1161,16 @@ static void set_color (char *obj, char *fg, char *bg)
    slrn_set_object_color (obj, fg, bg);
 }
 
+static char *get_fg_color (char *name)
+{
+   return slrn_get_object_color (name, 0);
+}
+
+static char *get_bg_color (char *name)
+{
+   return slrn_get_object_color (name, 1);
+}
+
 static FILE *Log_File_Ptr;
 static void close_log_file (void)
 {	
@@ -1150,7 +1187,7 @@ static void open_log_file (char *file)
 
    Log_File_Ptr = fopen (file, "w");
    if (Log_File_Ptr == NULL)
-     slrn_error ("Unable to open log file %s", file);
+     slrn_error (_("Unable to open log file %s"), file);
 }
 
 static void log_message (char *buf)
@@ -1229,10 +1266,9 @@ static void reload_scorefile (int *apply_now) /*{{{*/
    
    slrn_make_home_filename (Slrn_Score_File, file, sizeof (file));
 
-   slrn_message_now ("Reloading score file ...");
-   
-   if (-1 == slrn_read_score_file (file))
-     slrn_error ("Error processing score file %s.", file);
+   if (Slrn_Scorefile_Open != NULL)
+     slrn_free (Slrn_Scorefile_Open);
+   Slrn_Scorefile_Open = slrn_safe_strmalloc (file);
    
    if (is_article_mode() && *apply_now)
      slrn_apply_scores (*apply_now);
@@ -1265,6 +1301,8 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_SSS("definekey", definekey, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("extract_article_header", extract_article_header, SLANG_STRING_TYPE),
    MAKE_INTRINSIC_S("extract_displayed_article_header", extract_current_article_header, SLANG_STRING_TYPE),
+   MAKE_INTRINSIC_S("get_bg_color", get_bg_color, SLANG_STRING_TYPE),
+   MAKE_INTRINSIC_S("get_fg_color", get_fg_color, SLANG_STRING_TYPE),
    MAKE_INTRINSIC_0("get_group_flags", get_group_flags, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("get_grouplens_score", get_grouplens_score, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("get_header_flags", get_header_flags, SLANG_INT_TYPE),
@@ -1305,6 +1343,9 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_SSS("read_mini", read_mini, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SSS("read_mini_no_echo", read_mini_no_echo, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SSS("read_mini_filename", read_mini_filename, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_SSS("read_mini_variable", read_mini_variable, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_SI("read_mini_integer", read_mini_integer, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_SS("register_hook", slrn_register_hook_by_name, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("reload_scorefile", reload_scorefile, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("save_current_article", save_current_article, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("search_article", search_article, SLANG_INT_TYPE),
@@ -1334,6 +1375,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_0("uncollapse_threads", uncollapse_threads, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SS("undefinekey", undefinekey, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_I("ungetkey", ungetkey, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_SS("unregister_hook", slrn_unregister_hook_by_name, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("update", update, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0(NULL, NULL, 0)
 };
@@ -1411,4 +1453,3 @@ int slrn_reset_slang (void)
 }
 
 #endif
-
