@@ -1,3 +1,4 @@
+/* -*- mode: C; mode: fold -*- */
 /* MIME handling routines.
  *
  * Author: Michael Elkins <elkins@aero.org>
@@ -37,13 +38,13 @@
 #include <slang.h>
 #include "jdmacros.h"
 
-#include "server.h"
 #include "slrn.h"
 #include "misc.h"
 #include "slrn.h"
 #include "group.h"
 #include "art.h"
 #include "util.h"
+#include "server.h"
 #include "snprintf.h"
 #include "mime.h"
 
@@ -53,6 +54,7 @@ int Slrn_Use_Mime = 0;
 
 int Slrn_Use_Mime = 5;
 int Slrn_Use_Meta_Mail = 1;
+int Slrn_Fold_Headers = 1;
 char *Slrn_MetaMail_Cmd;
 char *Slrn_Utf8_Table = NULL;
 
@@ -620,7 +622,11 @@ static void rfc1522_decode_headers (Slrn_Article_Type *a)
 
    while ((line != NULL) && (line->flags & HEADER_LINE))
      {
-	if (slrn_rfc1522_decode_string (line->buf))
+	if (slrn_case_strncmp ((unsigned char *)line->buf,
+			       (unsigned char *)"Newsgroups:", 11) &&
+	    slrn_case_strncmp ((unsigned char *)line->buf,
+			       (unsigned char *)"Followup-To:", 12) &&
+	    slrn_rfc1522_decode_string (line->buf))
 	  {
 	     a->is_modified = 1;
 	     a->mime_was_modified = 1;
@@ -934,7 +940,7 @@ int slrn_mime_call_metamail (void)
      
    if ((Slrn_Use_Meta_Mail == 0)
        || Slrn_Batch
-       || (slrn_get_yesno (1, "Process this MIME article with metamail") <= 0))
+       || (slrn_get_yesno (1, _("Process this MIME article with metamail")) <= 0))
      return 0;
 
 # if defined(__os2__) || defined(__NT__)
@@ -948,7 +954,7 @@ int slrn_mime_call_metamail (void)
 
    if (fp == NULL)
      {
-	slrn_error ("Unable to open tmp file for metamail.");
+	slrn_error (_("Unable to open tmp file for metamail."));
 	return 0;
      }
    
@@ -967,7 +973,7 @@ int slrn_mime_call_metamail (void)
        || (strlen (mm) > SLRN_MAX_PATH_LEN))
      mm = "metamail";
 
-   cmd = slrn_strdup_printf ("%s %s", mm, tempfile);
+   cmd = slrn_strdup_strcat (mm, " ", tempfile, NULL);
    
    /* Make sure that metamail has a normal environment */
    slrn_set_display_state (0);
@@ -976,7 +982,7 @@ int slrn_mime_call_metamail (void)
    slrn_delete_file (tempfile);
    slrn_free (cmd);
    
-   printf("Press return to continue ...");
+   printf(_("Press return to continue ..."));
    getchar();
    fflush(stdin); /* get rid of any pending input! */
    
@@ -1069,199 +1075,431 @@ int slrn_mime_scan_file (FILE *fp)
    return 0;
 }
 
-#define IS_RFC850_SPECIAL(c) \
- (((c) == '(') || ((c) == ')') || ((c) == '<') || ((c) == '>') || ((c) == '"'))
-
-/* This routine returns -1 if d is not big enough to encode s.
- * Technically, this routine is incorrect.  It should encode whitespace
- * separated words and not arbitrary sequences of text.
- */
-static int rfc1522_encode (unsigned char *d, unsigned int len,
-			   unsigned char *s, int ignore_specials)
+static int get_word_len (char *s, int *is_encoded) /*{{{*/
 {
-   /* This routine encodes the a string containing 8-bit characters according
-    * to the conventions of RFC1522.  The strategy here is to print up to
-    * the first word which contains 8-bit chars before starting the "Q"
-    * encoding.  When it has been determined that all of the 8-bit
-    * characters have been encoded, switch back to normal mode.  This
-    * approach should generate relatively short encoded words and should be
-    * more generally readable.
-    */
+   char *e = s;
+   int qmarks = 0;
    
-   unsigned int hibit;		   /* How many 8-bit characters are left? */
-   unsigned char *p, ch, *dmax;
-   char charset[256];
-   unsigned int charset_len;
+   while (*e && ((*e == ' ') || (*e == '\t')))
+     e++;
+   if ((e[0] == '=') && (e[1] == '?'))
+     {
+	e += 2;
+	qmarks = 1;
+     }
+   while (*e && (*e != ' ') && (*e != '\t'))
+     {
+	if ((*e++ == '?') && qmarks) qmarks++;
+     }
+   if ((qmarks == 4) && (e > s+1) && (e[-1] == '=') && (e[-2] == '?'))
+     *is_encoded = 1;
+   return e - s;
+}
+/*}}}*/
 
-   /* First scan the string to see if there are 8-bit characters
-    * Count them for later use.
-    */
-   p = s;
-   hibit = 0;
-   while ((ch = *p) != 0)
+static int fold_line (char *s, unsigned int bytes, unsigned int line_len) /*{{{*/
+{
+   int line_enc = 0, word_len, word_enc = 0, retval = -1;
+   char *copy, *p;
+   
+   if (NULL == (copy = slrn_strmalloc (s, 0)))
+     return -1;
+   p = copy;
+   
+   /* skip the first word (folding after the keyword is not allowed) */
+   word_len = get_word_len (p, &line_enc);
+   strncpy (s, p, word_len);
+   s += word_len; p += word_len;
+   line_len += word_len; bytes -= word_len;
+   
+   while (*p)
      {
-	if (ch & 0x80) hibit++;
-	p++;
+	word_len = get_word_len (p, &word_enc);
+	if ((line_enc || word_enc) && (line_len + word_len > 76))
+	  {
+	     if (bytes-- <= 0) goto free_and_return;
+	     *s++ = '\n';
+	     line_len = 0;
+	     line_enc = 0;
+	  }
+	if ((int)bytes <= word_len) goto free_and_return;
+	strncpy (s, p, word_len);
+	s += word_len; p += word_len;
+	line_len += word_len; bytes -= word_len;
+	line_enc |= word_enc; word_enc = 0;
      }
    
-   if (hibit == 0)
+   if (bytes)
      {
-	if (s + len < p) return -1;
-	
-	strcpy ((char *)d, (char *)s); /* safe */
-	return 0;
+	retval = 0;
+	*s = '\0';
      }
    
+   free_and_return:
+   slrn_free (copy);
+   return retval;
+}
+/*}}}*/
+
+/* These functions return the number of characters written or
+ * -1 if dest is too small to hold the result. */
+
+static int copy_whitespace (unsigned char *from, unsigned char *to, /*{{{*/
+			    unsigned char *dest, size_t max)
+{
+   int len = 0;
+   while ((from <= to) && ((*from == ' ') || (*from == '\t')))
+     {
+	if (len == (int)max) return -1;
+	*dest++ = *from++;
+	len++;
+     }
+   return len;
+}
+/*}}}*/
+
+/* Hack to keep encoded words shorter if they directly follow the keyword. */
+static int Keyword_Len;
+
+#define ENCODE_COMMENT	1
+#define ENCODE_PHRASE	2
+
+/* Do the actual encoding.
+ * Note: This function does not generate encoded-words that are longer
+ *       than 75 chars (or shorter, depending on Keyword_Len).  The line
+ *       folding is performed by a separate function. */
+static int encode_string (unsigned char *from, unsigned char *to, /*{{{*/
+			  int flags, unsigned char *dest, size_t max)
+{
+   int len = 0, total = 0;
+   char charset[64];
+   
    if (0 == slrn_case_strcmp ((unsigned char *)Slrn_Mime_Display_Charset,
 			      (unsigned char *)"us-ascii"))
      {
 	strcpy (charset, "=?iso-8859-1?Q?"); /* safe */
      }
    else slrn_snprintf (charset, sizeof (charset), "=?%s?Q?", Slrn_Mime_Display_Charset);
-   charset_len = strlen (charset);
    
-   p = s;
-   dmax = d + len;
-   /* leave room for the 0 character */
-   dmax--;
-   
-   /* algorithm:
-    *  1. copy up to first word that contains one or more eight bit chars.
-    *  2. encode to last char of last word that contains eight bit chars,
-    *     or until a RFC822 special is encountered: <>()"
-    *  3. Repeat
-    * The rationale behind this is that 7 bit characters are allowed only
-    * in certain portions of the header where an 8 bit character would never
-    * appear.  Thus by encoding only in regions of 8 bit charcters delimited
-    * by the RFC822 specials, we are sure not to decode into forbidden regions.
-    *
-    * RFC1522 forbids strarting or ending a =?..?..?..?= sequence in mid-word.
-    */
-   while (1)
+   while (from < to)
      {
-	/* 1. Copy to first 8 bit character */
-	while (((ch = *p) != 0) && (d < dmax) && (0 == (ch & 0x80)))
+	/* Start the encoded-word */
+	len = strlen (charset);
+	if (len > (int) max - 1) return -1;
+	strcpy (dest, charset); /* safe */
+	dest += len; max -= len; total += len;
+	
+	/* Write the data */
+	while (max && (from < to) && (len < 71 - Keyword_Len))
 	  {
-	     *d++ = ch;
-	     p++;
+	     unsigned char ch;
+	     if (flags && (*from == '\\') && (++from == to))
+	       break;
+	     if (((ch = *from) & 0x80) || (ch < 32) ||
+		 (ch == '?') || (ch == '\t') || (ch == '=') || (ch == '_') ||
+		 ((flags & ENCODE_COMMENT) &&
+		  ((ch == '(') || (ch == ')') || (ch == '"'))) ||
+		 ((flags & ENCODE_PHRASE) &&
+		  (NULL != strchr ("\"(),.:;<>@[\\]", ch))))
+	       {
+		  if (max < 3) return -1;
+		  sprintf ((char *) dest, "=%02X", (int) ch); /* safe */
+		  len += 3; max -= 3; dest += 3; total += 3;
+	       }
+	     else
+	       {
+		  if (ch == ' ') *dest = '_';
+		  else *dest = ch;
+		  len++; max--; dest++; total++;
+	       }
+	     from++;
 	  }
 	
-	if (ch == 0)
-	  break;
-	
-	if (d == dmax) 
+	/* Finish the encoded-word */
+	if (max < 3) return -1;
+	Keyword_Len = 0;
+	*dest++ = '?'; *dest++ = '=';
+	if (from < to)
 	  {
-	     *d = 0;
-	     return -1;
+	     *dest++ = ' ';
+	     len += 3; max -= 3; total += 3;
 	  }
+	else
+	  {
+	     len += 2; max -= 2; total += 2;
+	  }
+     }
+   
+   if (max)
+     {
+	*dest = '\0';
+	return total;
+     }
+   return -1;
+}
+/*}}}*/
+
+/* Try to cause minimal overhead when encoding. */
+static int min_encode (unsigned char *from, unsigned char *to, /*{{{*/
+		       int flags, unsigned char *dest, size_t max)
+{
+   int len, total = 0;
+   
+   if (-1 == (len = copy_whitespace (from, to, dest, max)))
+     return -1;
+   from += len; dest += len; max -= len; total += len;
+   
+   while (from < to)
+     {
+	unsigned char *beg = NULL, *end = NULL;
+	unsigned char *eword = from, *bword = from;
+	int count;
 	
-	/* walk back until we find a word boundary or begin-of-line */
 	do
 	  {
-	     p--;
-	     ch = *p;
-	     if ((ch == ' ') || (ch == '\t')
-		 || ((ignore_specials == 0) && IS_RFC850_SPECIAL(ch)))
+	     count = 0;
+	     while ((eword < to) && (*eword != ' ') && (*eword != '\t'))
 	       {
-		  p++;
-		  break;
+		  if (*eword & 0x80) count++;
+		  eword++;
 	       }
- 	     d--;
-	  }
-	while (p != s);
-
- 	
-	/* 2. Now start encoding.  Work up to the first occurance of an RFC822
-	 *    special character or until there are no more eight bit characters
-	 *    left.  RFC1522 
-	 */
-	if (d + charset_len >= dmax)
-	  return -1;
-	
-	strcpy ((char *)d, charset); /* safe */
-	d += charset_len;
-
-	/* Note: rfc1522 places a limit on the length of the encoded word.  
-	 * That limit is ignored here.  Perhaps someday I will recode this to 
-	 * take the limit into account.
-	 */
-	while ((d < dmax) 
-	       && ((ch = *p) != 0)
-	       && ((ignore_specials != 0) || (0 == IS_RFC850_SPECIAL(ch)))
-	       && ((hibit != 0) 
-		   || ((ch != ' ') && (ch != '\t') && (ch != '\n'))))
-	  {
-	     if (ch == ' ') *d++ = '_';
-	     else if ((ch & 0x80) 
-		      || ((ch < 32) && (ch != '\n'))
-		      || IS_RFC850_SPECIAL(ch)
-		      || (ch == '=')  /* = cannot occur as part of encoded word */
-		      || (ch == '?')  /* ? cannot occur as part of encoded word */
-		      || (ch == '_'))  /* encode this since SPACE gets mapped to it */
+	     
+	     if (count)
 	       {
-		  /* We need 3 characters to encode this. */
-		  if (d + 3 >= dmax)
-		    {
-		       *d = 0;
-		       return -1;
-		    }
-		  sprintf ((char *)d, "=%02X", (int) ch); /* safe */
-		  d += 3;
-		  if (ch & 0x80) hibit--;
+		  if (beg == NULL) beg = bword;
+		  bword = end = eword;
 	       }
-	     else *d++ = ch;
-	     p++;
+	     
+	     while ((eword < to) && ((*eword == ' ') || (*eword == '\t')))
+	       eword++;
 	  }
+	while ((eword < to) && count);
 	
-	/* Remove encoded spaces at the end (' '--> '_') */
-	while ((p > s) && (*(p-1) == ' '))
+	if (beg != NULL)
 	  {
-	     d--; 
-	     p--;
+	     if (-1 == (len = encode_string (beg, end, flags, dest, max)))
+	       return -1;
+	     dest += len; max -= len; total += len;
 	  }
-
-	/* Now turn off encoding.  We need two characters */
-	if (d + 1 >= dmax) 
+	if (bword != eword)
 	  {
-	     *d = 0;
-	     return -1;
+	     len = eword - bword;
+	     if ((int)max < len) return -1;
+	     strncpy (dest, bword, len);
+	     dest += len; max -= len; total += len;
+	     Keyword_Len = 0;
 	  }
-	
-	d[0] = '?';
-	d[1] = '=';
-	d += 2;
+	from = eword;
      }
-   *d = 0;
-   return 0;
+   
+   if (max)
+     {
+	*dest = '\0';
+	return total; /* don't count trailing zero */
+     }
+   
+   return -1;
 }
+/*}}}*/
 
-void slrn_mime_header_encode (char *s, unsigned int bytes)
+static int encode_quoted_string (unsigned char **from, unsigned char *to, /*{{{*/
+				 unsigned char *dest, size_t max)
 {
-   char buf[1024];
-   unsigned int len = strlen (s);
+   unsigned char *end = *from + 1;
+   int count = 0;
+   
+   while ((end < to) && (*end != '"'))
+     {
+	if ((*end == '\\') && (++end == to))
+	  break;
+	if (*end & 0x80)
+	  count++;
+	end++;
+     }
+   
+   if (count)
+     {
+	int retval = encode_string (*from + 1, end, ENCODE_PHRASE, dest, max);
+	*from = end + 1;
+	return retval;
+     }
+   else
+     {
+	int len = end + 1 - *from;
+	if ((int)max <= len) return -1;
+	strncpy (dest, *from, len);
+	dest[len] = '\0';
+	*from = end + 1;
+	return len;
+     }
+}
+/*}}}*/
+
+static int encode_comment (unsigned char **from, unsigned char *to, /*{{{*/
+			   unsigned char *dest, size_t max)
+{
+   unsigned char *end = *from + 1;
+   int len, total = 0;
+   
+   while ((end < to) && (*end != ')'))
+     {
+	if (*end == '(')
+	  {
+	     if (**from == '(')
+	       {
+		  if (!max) return -1;
+		  *dest++ = '('; total++; max--; (*from)++;
+	       }
+	     
+	     if (-1 == (len = min_encode (*from, end, ENCODE_COMMENT, dest, max)))
+	       return -1;
+	     total += len; max -= len; dest += len;
+	     
+	     if (-1 == (len = encode_comment (&end, to, dest, max)))
+	       return -1;
+	     total += len; max -= len; dest += len;
+	     *from = end;
+	  }
+	else
+	  {
+	     if ((*end == '\\') && (++end == to))
+	       break;
+	     end++;
+	  }
+     }
+   
+   if (**from == '(')
+     {
+	if (!max) return -1;
+	*dest++ = '('; total++; max--; (*from)++;
+     }
+   if (-1 == (len = min_encode (*from, end, ENCODE_COMMENT, dest, max)))
+     return -1;
+   dest += len; total += len; max -= len;
+   *from = end + 1;
+   if (max < 2) return -1;
+   *dest++ = ')'; *dest = '\0';
+   
+   return total + 1; /* don't count trailing zero */
+}
+/*}}}*/
+
+/* Encode structured header fields ("From:", "To:", "Cc:" and such) */
+static int from_encode (unsigned char *from, unsigned char *to, /*{{{*/
+			unsigned char *dest, size_t max)
+{
+   int len, total = 0;
+   
+   if (-1 == (len = copy_whitespace (from, to, dest, max)))
+     return -1;
+   from += len; dest += len; max -= len; total += len;
+   
+   while (from < to)
+     {
+	if (*from == '(')
+	  {
+	     if (-1 == (len = encode_comment (&from, to, dest, max)))
+	       return -1;
+	     dest += len; max -= len; total += len;
+	  }
+	else if (*from == '"')
+	  {
+	     if (-1 == (len = encode_quoted_string (&from, to, dest, max)))
+	       return -1;
+	     dest += len; max -= len; total += len;
+	  }
+	else if (NULL != strchr ("),.:;<>@[\\]", *from))
+	  {
+	     if (!max) return -1;
+	     *dest++ = *from++;
+	     max--; total++;
+	  }
+	else
+	  {
+	     unsigned char *next = from + 1;
+	     while ((next < to) && (NULL == strchr ("\"(),.:;<>@[\\]", *next)))
+	       next++;
+	     if (-1 == (len = min_encode (from, next, ENCODE_PHRASE, dest, max)))
+	       return -1;
+	     from = next; dest += len; max -= len; total += len;
+	  }
+     }
+   
+   if (!max) return -1;
+   *dest = '\0';
+   
+   return total;
+}
+/*}}}*/
+
+void slrn_mime_header_encode (char *s, unsigned int bytes) /*{{{*/
+{
+   char buf[1024], *colon;
+   unsigned int len, max = bytes;
+   
+   /* preserve 8bit characters in those headers */
+   if (!slrn_case_strncmp ((unsigned char *) s,
+			   (unsigned char *) "Newsgroups: ", 12) ||
+       !slrn_case_strncmp ((unsigned char *) s,
+			   (unsigned char *) "Followup-To: ", 13))
+     return;
+   
+   if ((NULL != (colon = strchr (s, ':'))) && (colon[1] == ' '))
+     {
+	max -= (colon + 2 - s);
+	colon += 2; /* skip the keyword */
+     }
+   else
+     colon = s;
+   len = strlen (colon);
+   Keyword_Len = colon - s;
    
    if (len < sizeof (buf))
      {
-	int ignore_specials;
+	int ret;
 	
-	/* Perhaps this test should be extended to include other headers. */
-	ignore_specials = !slrn_case_strncmp ((unsigned char *)s, 
-					      (unsigned char *)"Subject: ", 9);
+	strcpy (buf, colon); /* safe */
+	     
+	if (len && (buf[len-1] == '\n')) len--; /* save \n from being encoded */
 	
-	strcpy (buf, s); /* safe */
-	if (0 == rfc1522_encode ((unsigned char *)s, bytes, 
-				 (unsigned char *)buf,
-				 ignore_specials))
-	    return;
+	if (!slrn_case_strncmp ((unsigned char *) s,
+				(unsigned char *) "From: ", 6) ||
+	    !slrn_case_strncmp ((unsigned char *) s,
+				(unsigned char *) "Cc: ", 4) ||
+	    !slrn_case_strncmp ((unsigned char *) s,
+				(unsigned char *) "To: ", 4) ||
+	    !slrn_case_strncmp ((unsigned char *) s,
+				(unsigned char *) "Mail-Copies-To: ", 16))
+	  ret = from_encode ((unsigned char *) buf, (unsigned char *) buf + len,
+			     (unsigned char *) colon, max);
+	else
+	  ret = min_encode ((unsigned char *) buf, (unsigned char *) buf + len,
+			    0, (unsigned char *) colon, max);
+	
+	if ((ret != -1) && (max - ret > 1))
+	  {
+	     if (len && (buf[len] == '\n'))
+	       {
+		  colon[ret] = '\n';
+		  colon[ret+1] = '\0';
+	       }
+	     if ((0 == Slrn_Fold_Headers) ||
+		 (-1 != fold_line (colon, max, colon - s)))
+	       return;
+	  }
+	strcpy (colon, buf); /* safe */
      }
-   
+
+#if 0
    /* Cannot do it so strip it to 8 bits. */
-   while (*s)
+   while (*colon)
      {
-	*s = *s & 0x7F;
-	s++;
+	*colon = *colon & 0x7F;
+	colon++;
      }
+#endif
 }
+/*}}}*/
 
 void slrn_mime_add_headers (FILE *fp)
 {
@@ -1320,5 +1558,3 @@ FILE *slrn_mime_encode (FILE *fp)
 #endif /* NOT SLRNPULL_CODE */
 
 #endif  /* SLRN_HAS_MIME */
-
-
