@@ -15,12 +15,13 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #ifndef _SLRN_MISC_H
 #define _SLRN_MISC_H
 #include <stdio.h>
+#include <stdarg.h>
 #include <slang.h>
 
 #include "ttymsg.h"
@@ -49,10 +50,13 @@ extern void slrn_suspend_cmd (void);
 extern int slrn_read_integer (char *, int *, int *);
 extern int slrn_read_input (char *, char *, char *, int, int);
 extern int slrn_read_input_no_echo (char *, char *, char *, int, int);
+extern int slrn_read_filename (char *, char *, char *, int, int);
+extern int slrn_read_variable (char *, char *, char *, int, int);
 extern void slrn_evaluate_cmd (void);
 extern void slrn_update_top_status_line (void);
 extern void slrn_set_color (int);
-extern char slrn_get_response (char *, char *str, ...);
+extern char slrn_map_translated_char (char *, char *, char);
+extern char slrn_get_response (char *, char *, char *str, ...);
 extern int slrn_is_fqdn (char *);
 extern int slrn_init_readline (void);
 extern int slrn_check_batch (void);
@@ -95,6 +99,8 @@ extern void slrn_get_mouse_rc (int *, int *);
 extern char *Slrn_SendMail_Command;
 #endif
 extern SLKeyMap_List_Type *Slrn_RLine_Keymap;
+extern SLang_RLine_Info_Type *Slrn_Keymap_RLI;
+extern SLKeymap_Function_Type Slrn_Custom_Readline_Functions [];
 
 extern void slrn_va_message (char *, va_list);
 
@@ -114,6 +120,8 @@ extern char *Slrn_Editor;
 extern char *Slrn_Editor_Post;
 extern char *Slrn_Editor_Score;
 extern char *Slrn_Editor_Mail;
+extern int Slrn_Editor_Uses_Mime_Charset;
+extern int Slrn_Mail_Editor_Is_Mua;
 
 extern char *Slrn_Failed_Post_Filename;
 
