@@ -55,6 +55,7 @@ int Slrn_Broken_Xref;
 
 static int _nntp_connect_server (NNTP_Type *);
 
+extern int Slrn_Force_Authentication;
 extern FILE *Slrn_Debug_Fp; /* we need this here when compiling slrnpull */
 
 static NNTP_Type *nntp_allocate_nntp (void)
@@ -192,7 +193,7 @@ int nntp_puts_server (NNTP_Type *s, char *buf)
 static void _nntp_error_response (NNTP_Type *s, char *fmt)
 {
    slrn_error ("%s", fmt);
-   slrn_error ("Reason: %s", s->rspbuf);
+   slrn_error (_("Reason: %s"), s->rspbuf);
 }
 
 
@@ -262,7 +263,7 @@ static int _nntp_reconnect (NNTP_Type *s)
 
    s->flags &= ~NNTP_RECONNECT_OK;
 
-   slrn_message_now ("Server %s dropped connection.  Reconnecting...", s->host);
+   slrn_message_now (_("Server %s dropped connection.  Reconnecting..."), s->host);
 
    if (-1 == _nntp_connect_server (s))
      {
@@ -405,14 +406,14 @@ int nntp_authorization (NNTP_Type *s)
    char *name, *pass;
 
    if ((NNTP_Authorization_Hook == NULL) ||
-       (-1 == (*NNTP_Authorization_Hook) (s->host, &name, &pass)) ||
+       (-1 == (*NNTP_Authorization_Hook) (s->name, &name, &pass)) ||
        ((name == NULL) || (pass == NULL)))
      {
-	slrn_exit_error ("Authorization needed, but could not determine username / password.");
+	slrn_exit_error (_("Authorization needed, but could not determine username / password."));
 	return -1;
      }
 
-   slrn_message_now ("Authenticating %s ...", name);
+   slrn_message_now (_("Authenticating %s ..."), name);
 
    if (-1 == nntp_server_vcmd (s, "AUTHINFO USER %s", name))
      return -1;
@@ -442,7 +443,7 @@ int nntp_authorization (NNTP_Type *s)
 	break;
       case ERR_ACCESS:
       default:
-	_nntp_error_response (s, "Authorization failed.");
+	_nntp_error_response (s, _("Authorization failed."));
 	return -1;
      }
    return 0;
@@ -453,7 +454,7 @@ static int _nntp_connect_server (NNTP_Type *s)
    if (s->tcp != NULL)
      sltcp_close (s->tcp);
 
-   slrn_message_now ("Connecting to host %s ...", s->host);
+   slrn_message_now (_("Connecting to host %s ..."), s->host);
    if (Slrn_Debug_Fp != NULL)
      fputs ("!Connecting to server...\n", Slrn_Debug_Fp);
 
@@ -462,7 +463,7 @@ static int _nntp_connect_server (NNTP_Type *s)
 
    s->init_state = 1;
    s->number_bytes_received = 0;
-   NNTP_Try_Authentication = 1;
+   NNTP_Try_Authentication = 2;
    
    /* Read logon message. */
    switch (nntp_get_server_response (s))
@@ -485,14 +486,18 @@ static int _nntp_connect_server (NNTP_Type *s)
    
    if (s->code == OK_NOPOST)
      s->can_post = 0;
+   
+   if (Slrn_Force_Authentication && (-1 == nntp_authorization (s)))
+     return -1;
 
-   slrn_message_now ("Connected to host.  Posting %sOk.",
-		     (s->can_post ? "" : "NOT "));
+   slrn_message_now (_("Connected to host.  %s"),
+		     (s->can_post ? _("Posting ok.") : _("Posting NOT ok.")));
+   
    return 0;
 
    failed:
 
-   _nntp_error_response (s, "Failed to initialize server");
+   _nntp_error_response (s, _("Failed to initialize server"));
    if (Slrn_Debug_Fp != NULL)
      fputs ("!Server connect failed.\n", Slrn_Debug_Fp);
    
@@ -556,15 +561,15 @@ char *nntp_get_server_name (void)
    if (NULL != (host = _nntp_getserverbyfile(NNTPSERVER_FILE)))
      return host;
 
-   fprintf (stderr, "You need to set the NNTPSERVER environment variable to your server name.\r\n");
+   slrn_stderr_strcat ("\n", _("You need to set the NNTPSERVER environment variable to your server name."), "\n", NULL);
 #ifdef VMS
-   fprintf (stderr, "Example: $ define/job NNTPSERVER my.news.server\r\n");
+   slrn_stderr_strcat (_("Example: $ define/job NNTPSERVER my.news.server"), "\n", NULL);
 #else
 # if defined(IBMPC_SYSTEM)
-   fprintf (stderr, "Example: set NNTPSERVER=my.news.server\r\n");
+   slrn_stderr_strcat (_("Example: set NNTPSERVER=my.news.server"), "\n", NULL);
 # else
-   fprintf (stderr, "Example (csh): setenv NNTPSERVER my.news.server\r\n");
-   fprintf (stderr, "Example (sh) : NNTPSERVER='my.news.server' && export NNTPSERVER\r\n");
+   slrn_stderr_strcat (_("Example (csh): setenv NNTPSERVER my.news.server"), "\n",
+		       _("Example (sh) : NNTPSERVER='my.news.server' && export NNTPSERVER"), "\n", NULL);
 # endif
 #endif
    return NULL;
@@ -597,6 +602,7 @@ static int _nntp_setup_host_and_port (char *host, int port, NNTP_Type *s)
    while ((*a != 0) && (*a != ':') && (b < bmax))
      *b++ = *a++;
    *b = 0;
+   slrn_strncpy (s->name, host, NNTP_MAX_HOST_LEN);
 
    if (s->use_ssh)
      {
@@ -741,7 +747,7 @@ int nntp_has_cmd (NNTP_Type *s, char *cmd)
 	     else
 	       s->can_xhdr = 1;
 	  }
-	else if ((ret == ERR_CMDSYN) && (strlen (cmd) == 4))
+	else if (strlen (cmd) == 4)
 	  s->can_xhdr = 1;
 	
 	return s->can_xhdr;
@@ -829,20 +835,36 @@ int nntp_select_group (NNTP_Type *s, char *name, int *minp, int *maxp)
 	break;
      }
 
+   if (NNTP_Try_Authentication == 2)
+     NNTP_Try_Authentication = 1;
    return s->code;
 }
 
 int nntp_refresh_groups (NNTP_Type *s, Slrn_Group_Range_Type *gr, int n)
 {
-   int reconnect, i, max_tries = 3;
+   int reconnect, i, prev = 0, max_tries = 3;
    
    start_over:
+   
+   /* If we might still need authentication, use nntp_select_group first. */
+   if ((NNTP_Try_Authentication == 2) && n)
+     {
+	int status;
+	NNTP_Try_Authentication = 1;
+	
+	status = nntp_select_group (s, gr->name, &(gr->min), &(gr->max));
+	if (status == -1) return -1;
+	else if (status == ERR_NOGROUP)
+	  gr->min = -1;
+	n--; gr++;
+     }
+   
    max_tries--;
    reconnect = s->flags & NNTP_RECONNECT_OK;
    s->flags &= ~NNTP_RECONNECT_OK;
    /* disallow reconnect for the moment as we batch commands */
    
-   i = 0;
+   i = prev;
    while (i < n)
      {
 	if (-1 == nntp_start_server_vcmd (s, "GROUP %s", gr[i].name))
@@ -854,10 +876,11 @@ int nntp_refresh_groups (NNTP_Type *s, Slrn_Group_Range_Type *gr, int n)
 	  }
 	i++;
      }
-   i = 0;
+   i = prev;
    while (i < n)
      {
 	int estim;
+	prev = i;
 	switch (nntp_get_server_response (s))
 	  {
 	   case -1:
@@ -866,11 +889,30 @@ int nntp_refresh_groups (NNTP_Type *s, Slrn_Group_Range_Type *gr, int n)
 	       return -1;
 	     goto start_over;
 	     
+	   case ERR_NOAUTH:
+	     s->flags |= reconnect;
+	     while (++i < n)
+	       nntp_get_server_response (s);
+	     if (NNTP_Try_Authentication)
+	       {
+		  NNTP_Try_Authentication = 0;
+		  if (-1 == nntp_authorization (s))
+		    return -1;
+		  max_tries++;
+		  goto start_over;
+	       }
+	     else
+	       return -1;
+	     
 	   case OK_GROUP:
 	     if (3 != sscanf (s->rspbuf + 4, "%d %d %d", &estim,
 			      &(gr[i].min), &(gr[i].max)))
-	       return -1;
-	     
+	       {
+		  s->flags |= reconnect;
+		  while (++i < n)
+		    nntp_get_server_response (s);
+		  return -1;
+	       }
 	     slrn_strncpy (s->group_name, gr[i].name, NNTP_MAX_GROUP_NAME_LEN);
 	     break;
 	     
@@ -1076,99 +1118,98 @@ char *nntp_map_code_to_string (int code)
    switch (code)
      {
       case INF_HELP:	/* 100 */	
-	return "Help text on way";
+	return _("Help text on way");
       case INF_AUTH:	/* 180 */	
-	return "Authorization capabilities";
+	return _("Authorization capabilities");
       case INF_DEBUG:	/* 199 */	
-	return "Debug output";
+	return _("Debug output");
       case OK_CANPOST:	/* 200 */	
-	return "Hello; you can post";
+	return _("Hello; you can post");
       case OK_NOPOST:	/* 201 */	
-	return "Hello; you can't post";
+	return _("Hello; you can't post");
       case OK_SLAVE:	/* 202 */	
-	return "Slave status noted";
+	return _("Slave status noted");
       case OK_GOODBYE:	/* 205 */	
-	return "Closing connection";
+	return _("Closing connection");
       case OK_GROUP:	/* 211 */	
-	return "Group selected";
+	return _("Group selected");
       case OK_GROUPS:	/* 215 */	
-	return "Newsgroups follow";
+	return _("Newsgroups follow");
       case OK_ARTICLE:	/* 220 */	
-	return "Article (head & body) follows";
+	return _("Article (head & body) follows");
       case OK_HEAD:	/* 221 */
-	return "Head follows";
+	return _("Head follows");
       case OK_BODY:	/* 222 */
-	return "Body follows";
+	return _("Body follows");
       case OK_NOTEXT:	/* 223 */
-	return "No text sent -- stat, next, last";
+	return _("No text sent -- stat, next, last");
       case OK_NEWNEWS:	/* 230 */
-	return "New articles by message-id follow";
+	return _("New articles by message-id follow");
       case OK_NEWGROUPS:/* 231 */
-	return "New newsgroups follow";
+	return _("New newsgroups follow");
       case OK_XFERED:	/* 235 */
-	return "Article transferred successfully";
+	return _("Article transferred successfully");
       case OK_POSTED:	/* 240 */
-	return "Article posted successfully";
+	return _("Article posted successfully");
       case OK_AUTHSYS:	/* 280 */
-	return "Authorization system ok";
+	return _("Authorization system ok");
       case OK_AUTH:	/* 281 */
-	return "Authorization (user/pass) ok";
+	return _("Authorization (user/pass) ok");
       case OK_XGTITLE:	/* 282 */
-	return "Ok, XGTITLE info follows";
+	return _("Ok, XGTITLE info follows");
       case OK_XOVER:	/* 224 */
-	return "ok -- overview data follows";
+	return _("ok -- overview data follows");
       case CONT_XFER:	/* 335 */
-	return "Continue to send article";
+	return _("Continue to send article");
       case CONT_POST:	/* 340 */
-	return "Continue to post article";
+	return _("Continue to post article");
       case NEED_AUTHINFO:/* 380 */
-	return "authorization is required";
+	return _("authorization is required");
       case NEED_AUTHDATA:/* 381 */
-	return "<type> authorization data required";
+	return _("<type> authorization data required");
       case ERR_GOODBYE:	/* 400 */
-	return "Have to hang up for some reason";
+	return _("Have to hang up for some reason");
       case ERR_NOGROUP:	/* 411 */
-	return "No such newsgroup";
+	return _("No such newsgroup");
       case ERR_NCING:	/* 412 */
-	return "Not currently in newsgroup";
+	return _("Not currently in newsgroup");
       case ERR_NOCRNT:	/* 420 */
-	return "No current article selected";
+	return _("No current article selected");
       case ERR_NONEXT:	/* 421 */
-	return "No next article in this group";
+	return _("No next article in this group");
       case ERR_NOPREV:	/* 422 */
-	return "No previous article in this group";
+	return _("No previous article in this group");
       case ERR_NOARTIG:	/* 423 */
-	return "No such article in this group";
+	return _("No such article in this group");
       case ERR_NOART:	/* 430 */
-	return "No such article at all";
+	return _("No such article at all");
       case ERR_GOTIT:	/* 435 */
-	return "Already got that article, don't send";
+	return _("Already got that article, don't send");
       case ERR_XFERFAIL:/* 436 */
-	return "Transfer failed";
+	return _("Transfer failed");
       case ERR_XFERRJCT:/* 437 */
-	return "Article rejected, don't resend";
+	return _("Article rejected, don't resend");
       case ERR_NOPOST:	/* 440 */
-	return "Posting not allowed";
+	return _("Posting not allowed");
       case ERR_POSTFAIL:/* 441 */
-	return "Posting failed";
+	return _("Posting failed");
       case ERR_NOAUTH:	/* 480 */
-	return "authorization required for command";
+	return _("authorization required for command");
       case ERR_AUTHSYS:	/* 481 */  /* ERR_XGTITLE has same code */
-	return "Authorization system invalid";
+	return _("Authorization system invalid");
       case ERR_AUTHREJ:	/* 482 */
-	return "Authorization data rejected";
+	return _("Authorization data rejected");
       case ERR_COMMAND:	/* 500 */
-	return "Command not recognized";
+	return _("Command not recognized");
       case ERR_CMDSYN:	/* 501 */
-	return "Command syntax error";
+        return _("Command syntax error");
       case ERR_ACCESS:	/* 502 */
-	return "Access to server denied";
+	return _("Access to server denied");
       case ERR_FAULT:	/* 503 */
-	return "Program fault, command not performed";
+	return _("Program fault, command not performed");
       case ERR_AUTHBAD:	/* 580 */
-	return "Authorization Failed";
+	return _("Authorization Failed");
      }
 
-   return "Unknown NNTP code";
+   return _("Unknown NNTP code");
 }
-
