@@ -15,8 +15,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 /* post an article */
 #include "config.h"
@@ -39,24 +39,27 @@
 #include <slang.h>
 #include "jdmacros.h"
 
-
 #ifdef VMS
 # include "vms.h"
 #endif
+#ifdef __MINGW32__
+# include <process.h>
+#endif
 
 #include "slrn.h"
+#include "util.h"
 #include "server.h"
 #include "misc.h"
 #include "post.h"
 #include "group.h"
 #include "art.h"
-#include "uudecode.h"
-#include "util.h"
+#include "decode.h"
 #include "snprintf.h"
 #include "chmap.h"
 #include "menu.h"
 #include "version.h"
 #include "mime.h"
+#include "hooks.h"
 
 #define MAX_LINE_BUFLEN	2048
 
@@ -69,10 +72,11 @@ char *Slrn_Post_Custom_Headers;
 char *Slrn_Failed_Post_Filename;
 char *Slrn_Signoff_String;
 
-int Slrn_Reject_Long_Lines = 1;
+int Slrn_Reject_Long_Lines = 2;
 int Slrn_Netiquette_Warnings = 1;
 char *Slrn_Postpone_Dir;
 int Slrn_Generate_Message_Id = 1;
+int Slrn_Generate_Date_Header = 0;
 
 static int postpone_file (char *);
 
@@ -95,7 +99,7 @@ static char *slrn_create_message_id (void)
 	  return NULL;
 	
 	if (now != last_now) break;
-	sleep (1);
+	slrn_sleep (1);
      }
    last_now = now;
    
@@ -128,25 +132,62 @@ static char *slrn_create_message_id (void)
 	*b++ = *t;
      }
    
-   *b++ = '.';
    t = (unsigned char *) Slrn_User_Info.username;
-   if (t != NULL)
+   if (t != NULL && *t != 0)
      {
-	unsigned char ch;
-	
-	while ((*t != 0) && (b < baseid + sizeof(baseid) - 1))
+	unsigned char ch = 0;
+
+	*b++ = '.';
+	while ((*t != 0) && (b < baseid + sizeof(baseid) - 2))
 	  {
 	     if (((ch = *t) > ' ') && ((ch & 0x80) == 0)
-		 && (ch != '>') && (ch != '<'))
+		 && (NULL == strchr ("<>()@,;:\\\"[]", ch)))
 	       *b++ = ch;
 	     t++;
 	  }
+	if (*(b-1) == '.') /* may not be last character, so append '_' */
+	  *b++ = '_';
      }
    *b = 0;
    
    return (char *) baseid;
 }
 #endif
+
+const char *Weekdays[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
+const char *Months[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec", "ERR" };
+
+void slrn_add_date_header (FILE *fp)
+{
+   char buf[64];
+   time_t now;
+   struct tm *t;
+   long int tz = 0;
+   
+   time (&now);
+   t = localtime (&now);
+   
+#ifdef HAVE_TIMEZONE
+   tz = - timezone / 60;
+   if (t->tm_isdst == 1) tz += 60;
+#else
+# ifdef HAVE_TM_GMTOFF
+   tz = t->tm_gmtoff / 60;
+# endif
+#endif
+   
+   slrn_snprintf (buf, sizeof (buf),
+		  "Date: %s, %d %s %d %02d:%02d:%02d %+03d%02d\n",
+		  Weekdays[t->tm_wday], t->tm_mday, Months[t->tm_mon],
+		  t->tm_year + 1900, t->tm_hour, t->tm_min, t->tm_sec,
+		  (int) tz / 60, (int) abs (tz) % 60);
+   
+   if (fp != NULL)
+     fputs (buf, fp);
+   else if (Slrn_Generate_Date_Header)
+     Slrn_Post_Obj->po_puts (buf);
+}
+
 void slrn_add_signature (FILE *fp)
 {
    FILE *sfp;
@@ -218,7 +259,7 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
 
    if (NULL == (fp = fopen (file, "r")))
      {
-	slrn_error ("Unable to open %s.");
+	slrn_error (_("Unable to open %s."));
 	return -1;
      }
    
@@ -328,6 +369,7 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
 	  fputs (line, pp);
      }
 
+   slrn_add_date_header (pp);
 # if SLRN_HAS_MIME
    if (Slrn_Use_Mime & MIME_DISPLAY) slrn_mime_add_headers (pp);
 # endif
@@ -384,7 +426,7 @@ static int check_file_for_posting (char *file)
    
    if (fp == NULL)
      {
-	slrn_error ("Unable to open %s", file);
+	slrn_error (_("Unable to open %s"), file);
 	return -1;
      }
    
@@ -400,7 +442,7 @@ static int check_file_for_posting (char *file)
 	  {
 	     if (num == 1) 
 	       {
-		  err = "The first line must begin with a header.";
+		  err = _("The first line must begin with a header.");
 		  break;
 	       }
 	     if (ch == '\n') break;
@@ -410,7 +452,7 @@ static int check_file_for_posting (char *file)
 	
 	if (NULL == (colon = slrn_strchr (line, ':')))
 	  {
-	     err = "Expecting a header.  This is not a header line.";
+	     err = _("Expecting a header.  This is not a header line.");
 	     break;
 	  }
 	
@@ -418,7 +460,7 @@ static int check_file_for_posting (char *file)
 	  {
 	     if (is_empty_header (line))
 	       {
-		  err = "The subject header is not allowed be to empty.";
+		  err = _("The subject header is not allowed be to empty.");
 		  break;
 	       }
 	     subject_found = 1;
@@ -430,7 +472,7 @@ static int check_file_for_posting (char *file)
 	     char *p = line;
 	     if (is_empty_header (line)) 
 	       {
-		  err = "The Newsgroups header is not allowed be to empty.";
+		  err = _("The Newsgroups header is not allowed be to empty.");
 		  break;
 	       }
 	     newsgroups_found = 1;
@@ -458,21 +500,21 @@ static int check_file_for_posting (char *file)
 	
 	if (*(colon + 1) != ' ') 
 	  {
-	     err = "A space must follow the ':' in a header";
+	     err = _("A space must follow the ':' in a header");
 	     break;
 	  }
 	
 #if SLRN_HAS_STRICT_FROM
 	if (!slrn_case_strncmp ((unsigned char *) line, (unsigned char *) "From:", 5))
 	  {
-	     err = "This news reader will not accept user generated From lines.";
+	     err = _("This news reader will not accept user generated From lines.");
 	     break;
 	  }
 #endif
 #if ! SLRN_HAS_GEN_MSGID
 	if (!slrn_case_strncmp ((unsigned char *) line, (unsigned char *) "Message-Id:", 11))
 	  {
-	     err = "This news reader will not accept user generated Message-Ids.";
+	     err = _("This news reader will not accept user generated Message-Ids.");
 	     break;
 	  }
 #endif
@@ -482,25 +524,25 @@ static int check_file_for_posting (char *file)
      {
 	if (subject_found == 0)
 	  {
-	     err = "Subject header is required.";
+	     err = _("Subject header is required.");
 	     num = 0;
 	  }
 	else if (newsgroups_found == 0)
 	  {
-	     err = "Newsgroups header is required.";
+	     err = _("Newsgroups header is required.");
 	     num = 0;
 	  }
 	else
 	  {
 	     if (newsgroups_found > 4)
 	       warnings[warn++ % MAX_WARNINGS] =
-	       "Please re-consider if posting to a large number of groups is necessary.";
+	       _("Please re-consider if posting to a large number of groups is necessary.");
 	     if (followupto_found > 1)
 	       warnings[warn++ % MAX_WARNINGS] =
-	       "In most cases, setting a \"Followup-To:\" multiple groups is a bad idea.";
+	       _("In most cases, setting a \"Followup-To:\" multiple groups is a bad idea.");
 	     else if ((newsgroups_found > 1) && (followupto_found == 0))
 	       warnings[warn++ % MAX_WARNINGS] =
-	       "Setting a \"Followup-To:\" is recommended when crossposting.";
+	       _("Setting a \"Followup-To:\" is recommended when crossposting.");
 	  }
      }
    
@@ -516,7 +558,7 @@ static int check_file_for_posting (char *file)
 	if (qs == NULL) qs = ">";
 	qlen = strlen (qs);
 	
-	err = "Your message does not appear to have any unquoted text.";
+	err = _("Your message does not appear to have any unquoted text.");
 	the_line = NULL;
 	while (NULL != fgets (line, sizeof (line), fp))
 	  {
@@ -567,15 +609,15 @@ static int check_file_for_posting (char *file)
 	  }
 	if (sig_lines > 4)
 	  warnings[warn++ % MAX_WARNINGS] =
-	  "Please keep your signature short. 4 lines is a commonly accepted limit.";
+	  _("Please keep your signature short. 4 lines is a commonly accepted limit.");
      }
    
    fclose (fp);
 
    if (Slrn_Batch && (err != NULL))
      {
-	slrn_error ("Message is not acceptable.");
-	slrn_error ("Reason: %s", err);
+	slrn_error (_("Message is not acceptable."));
+	slrn_error (_("Reason: %s"), err);
 	return -1;
      }
    
@@ -591,29 +633,29 @@ static int check_file_for_posting (char *file)
    slrn_set_color (SUBJECT_COLOR);
    if (err != NULL)
      {
-	SLsmg_write_string ("Your message is not acceptable for the following reason:");
+	SLsmg_write_string (_("Your message is not acceptable for the following reason:"));
 	slrn_set_color (ERROR_COLOR);
 	SLsmg_gotorc (4,4); SLsmg_write_string (err);
 	if (num && (the_line != NULL))
 	  {
 	     SLsmg_gotorc (6,0);
 	     slrn_set_color (SUBJECT_COLOR);
-	     SLsmg_printf ("This message was generated while looking at line %d:", num);
+	     SLsmg_printf (_("This message was generated while looking at line %d:"), num);
 	     SLsmg_gotorc (8,0);
 	     slrn_set_color (QUOTE_COLOR);
 	     SLsmg_write_string (the_line);
 	     SLsmg_gotorc (12, 0);
 	     slrn_set_color (ARTICLE_COLOR);
-	     SLsmg_write_string ("Perhaps this error was generated because you did not separate the header");
+	     SLsmg_write_string (_("Perhaps this error was generated because you did not separate the header"));
 	     SLsmg_gotorc (13, 0);
-	     SLsmg_write_string ("section from the body by a BLANK line.");
+	     SLsmg_write_string (_("section from the body by a BLANK line."));
 	  }
 	return 1;
      }
    else
      {
 	int row = 4;
-	SLsmg_write_string ("Your message breaks the following netiquette guidelines:");
+	SLsmg_write_string (_("Your message breaks the following netiquette guidelines:"));
 	slrn_set_color (ERROR_COLOR);
 	
 	if (Slrn_Netiquette_Warnings)
@@ -629,10 +671,10 @@ static int check_file_for_posting (char *file)
 	if ((the_line != NULL) && Slrn_Reject_Long_Lines)
 	  {
 	     SLsmg_gotorc (++row, 4);
-	     SLsmg_write_string ("Lines with more than 80 characters generally need to be wrapped.");
+	     SLsmg_write_string (_("Lines with more than 80 characters generally need to be wrapped."));
 	     slrn_set_color (SUBJECT_COLOR);
 	     row += 2; SLsmg_gotorc (row, 0);
-	     SLsmg_printf ("This affects at least line %d:", num);
+	     SLsmg_printf (_("This affects at least line %d:"), num);
 	     row += 2; SLsmg_gotorc (row, 0);
 	     slrn_set_color (QUOTE_COLOR);
 	     SLsmg_write_string (the_line);
@@ -659,7 +701,7 @@ int slrn_save_file_to_mail_file (char *file, char *save_file, char *msgid)
 	
    if (NULL == (infp = fopen (file, "r")))
      {
-	slrn_error ("File not found: %s--- message not posted.", file);
+	slrn_error (_("File not found: %s--- message not posted."), file);
 	return -1;
      }
 
@@ -671,7 +713,7 @@ int slrn_save_file_to_mail_file (char *file, char *save_file, char *msgid)
    if (NULL == (outfp = slrn_open_home_file (save_file, "a", save_post_file,
 					     sizeof (save_post_file), 1)))
      {
-	slrn_message_now ("***Warning: error saving to %s", save_post_file);
+	slrn_message_now (_("***Warning: error saving to %s"), save_post_file);
 	SLang_input_pending (30);
 	SLang_flush_input ();
 	slrn_fclose (infp);
@@ -679,7 +721,10 @@ int slrn_save_file_to_mail_file (char *file, char *save_file, char *msgid)
      }
    
    time (&now);
-   fprintf (outfp, "From %s@%s %s", Slrn_User_Info.username, Slrn_User_Info.hostname, ctime(&now));
+   if ((*Slrn_User_Info.username == 0) || (*Slrn_User_Info.hostname == 0))
+     fprintf (outfp, "From nobody@nowhere %s", ctime(&now));
+   else
+     fprintf (outfp, "From %s@%s %s", Slrn_User_Info.username, Slrn_User_Info.hostname, ctime(&now));
    
    while (NULL != fgets (line, sizeof(line) - 1, infp))
      {
@@ -697,7 +742,12 @@ int slrn_save_file_to_mail_file (char *file, char *save_file, char *msgid)
 	     if ((unsigned char)*line == '\n')
 	       {
 		  if (has_from == 0)
-		    fprintf (outfp, "From: %s\n", slrn_make_from_string ());
+		    {
+		       char *from = slrn_make_from_string ();
+		       if (from == NULL) from = "";
+		       fprintf (outfp, "From: %s\n", from);
+		    }
+		  slrn_add_date_header (outfp);
 		  if ((has_messageid == 0) && (msgid != NULL))
 		    fprintf (outfp, "Message-Id: <slrn%s@%s>\n", msgid,
 			     Slrn_User_Info.posting_host);
@@ -810,12 +860,12 @@ static int saved_failed_post (char *file)
    if (NULL == (outfp = slrn_open_home_file (save_file, "a", filebuf,
 					     sizeof (filebuf), 1)))
      {
-	slrn_message_now ("***Warning: error writing to %s", filebuf);
+	slrn_message_now (_("***Warning: error writing to %s"), filebuf);
 	fclose (fp);
 	return -1;
      }
 
-   slrn_message ("Failed post saved in %s", filebuf);
+   slrn_message (_("Failed post saved in %s"), filebuf);
 
    (void) fputs ("\n\n", outfp);
 
@@ -846,13 +896,14 @@ int slrn_post_file (char *file, char *to, int is_postponed)
    int has_messageid = 0;
 #endif
 #if SLRN_HAS_SLANG
-   SLang_Name_Type *filter_hook;
+   int filter_hook;
 #endif
    int once_more;
    char *system_os_name;
+   char *responses;
 
 #if SLRN_HAS_SLANG
-   filter_hook = SLang_get_function ("post_filter_hook");
+   filter_hook = slrn_is_hook_defined (HOOK_POST_FILTER);
 #endif
 
    system_os_name = slrn_get_os_name ();
@@ -865,43 +916,70 @@ int slrn_post_file (char *file, char *to, int is_postponed)
    if (Slrn_Batch == 0) while (once_more)
      {
 #if SLRN_HAS_SLANG
-	if (filter_hook != NULL)
+	if (filter_hook != 0)
 	  {
 	     if (is_postponed)
-	       rsp = slrn_get_response ("yYnNeEsSDdFf", "Post the message? \001Yes, \001No, \001Edit, po\001Stpone, \001Delete, \001Filter");
+	       {
+  /* Note to translators:
+   * In the following strings, "yY" will mean "yes", "nN" is for "no",
+   * "eE" for "edit", "sS" for "postpone", "dD" for "delete" and "fF"
+   * for "filter". Please use the same letter for the same field everywhere,
+   * or your users will get unexpected results. As always, you can't use the
+   * default letters for other fields than those they originally stood for.
+   * Be careful not to change the length of these strings!
+   */
+		  responses = _("yYnNeEsSdDfF");
+		  if (strlen (responses) != 12)
+		    responses = "";
+		  rsp = slrn_get_response ("yYnNeEsSDdFf", responses, _("Post the message? \001Yes, \001No, \001Edit, po\001Stpone, \001Delete, \001Filter"));
+	       }
 	     else
-	       rsp = slrn_get_response ("yYnNeEsSFf", "Post the message? \001Yes, \001No, \001Edit, po\001Stpone, \001Filter");
+	       {
+		  responses = _("yYnNeEsSfF");
+		  if (strlen (responses) != 10)
+		    responses = "";
+		  rsp = slrn_get_response ("yYnNeEsSFf", responses, _("Post the message? \001Yes, \001No, \001Edit, po\001Stpone, \001Filter"));
+	       }
 	  }
 	else
 	  {
 #endif
 	     if (is_postponed)
-	       rsp = slrn_get_response ("yYnNeEsSDd", "Post the message? \001Yes, \001No, \001Edit, po\001Stpone, \001Delete");
+	       {
+		  responses = _("yYnNeEsSdD");
+		  if (strlen (responses) != 10)
+		    responses = "";
+		  rsp = slrn_get_response ("yYnNeEsSDd", responses, _("Post the message? \001Yes, \001No, \001Edit, po\001Stpone, \001Delete"));
+	       }
 	     else
-	       rsp = slrn_get_response ("yYnNeEsS", "Post the message? \001Yes, \001No, \001Edit, po\001Stpone");
+	       {
+		  responses = _("yYnNeEsS");
+		  if (strlen (responses) != 8)
+		    responses = "";
+		  rsp = slrn_get_response ("yYnNeEsS", responses, _("Post the message? \001Yes, \001No, \001Edit, po\001Stpone"));
+	       }
 #if SLRN_HAS_SLANG
 	  }
 #endif
+	rsp = slrn_map_translated_char ("yYnNeEsSdDfF", _("yYnNeEsSdDfF"), rsp) | 0x20;
+	
 	switch (rsp)
 	  {
 	   case 'n':
-	   case 'N':
 	     return -1;
 	     
 	   case 's':
-	   case 'S':
 	     
 	     if (is_postponed
 		 || (0 == postpone_file (file)))
 	       return 1;
 
 	     /* Instead of returning, let user have another go at it */
-	     sleep (1);
+	     slrn_sleep (1);
 	     break;
 	     
 	   case 'd':
-	   case 'D':
-	     rsp = slrn_get_yesno_cancel ("Sure you want to delete it");
+	     rsp = slrn_get_yesno_cancel (_("Sure you want to delete it"));
 	     if (rsp == 0)
 	       continue;
 	     if (rsp == -1)
@@ -909,7 +987,7 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 	     /* Calling routine will delete it. */
 	     return 0;
 	  
-	   case 'y': case 'Y':
+	   case 'y':
 	     if (0 == (rsp = check_file_for_posting (file)))
 	       {
 		  once_more = 0;
@@ -920,20 +998,35 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 	     
 	     SLtt_beep ();
 	     if (rsp == 1)
-	       rsp = slrn_get_response ("yYEenNcC\007", "re-\001Edit,  or \001Cancel");
+	       {
+  /* Note to translators:
+   * In the next two strings, "yY" is "yes", "eE" is "edit", "nN" is "no",
+   * "cC" is "cancel" and "fF" means "force". The usual rules apply.
+   */
+		  responses = _("yYeEnNcC");
+		  if (strlen (responses) != 8)
+		    responses = "";
+		  rsp = slrn_get_response ("yYEenNcC\007", responses, _("re-\001Edit,  or \001Cancel"));
+	       }
 	     else
-	       rsp = slrn_get_response ("EeyYnNcC\007Ff", "re-\001Edit, \001Cancel, or \001Force the posting (not recommended)");
+	       {
+		  responses = _("yYeEnNcCfF");
+		  if (strlen (responses) != 10)
+		    responses = "";
+		  rsp = slrn_get_response ("EeyYnNcC\007Ff", responses, _("re-\001Edit, \001Cancel, or \001Force the posting (not recommended)"));
+	       }
+	     
+	     rsp = slrn_map_translated_char ("yYeEnNcCfF", _("yYeEnNcCfF"), rsp) | 0x20;
 	     
 	     switch (rsp)
 	       {
-		case 'y': case 'Y': case 'E': case 'e':
-		  rsp = 'y';
-		  break;
+		case 'y': case 'e':
+		  rsp = 'y'; break;
 		  
-		case 'c': case 'C': case 7:
+		case 'c': case 7:
 		  return -1;
 		  
-		case 'F': case 'f':
+		case 'f':
 		  once_more = 0;
 	       }
 
@@ -943,44 +1036,42 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 	     /* Drop */
 	     
 	   case 'e':
-	   case 'E':
 	     if (slrn_edit_file (Slrn_Editor_Post, file, 1, 0) < 0)
 	       return -1;
 	     break;
 
 #if SLRN_HAS_SLANG
-	   case 'F':
 	   case 'f':
-	     if (filter_hook != NULL)
+	     if (filter_hook != 0)
 	       {
-		  SLang_push_string (file);
-		  if (SLang_Error == 0)
-		    SLexecute_function (filter_hook);
+                  slrn_run_hooks (HOOK_POST_FILTER, 1, file);
 		  if (SLang_Error) 
-		    filter_hook = NULL;
+		    filter_hook = 0;
 	       }
+	}
 #endif
-	  }
      }
    
-   slrn_message_now ("Posting ...");
+   slrn_message_now (_("Posting ..."));
    
 #if SLRN_HAS_GEN_MSGID
    msgid = slrn_create_message_id ();
 #endif
 
-   if (Slrn_Use_Mime & MIME_ARCHIVE) /* else: do it later */
+   if ((Slrn_Use_Mime & MIME_ARCHIVE) && /* else: do it later */
+       !Slrn_Editor_Uses_Mime_Charset)
      slrn_chmap_fix_file (file, 0);
    
    if (-1 == slrn_save_file_to_mail_file (file, Slrn_Save_Posts_File, msgid))
      return -1;
    
-   if ((Slrn_Use_Mime & MIME_ARCHIVE) == 0)
+   if (!(Slrn_Use_Mime & MIME_ARCHIVE) &&
+       !Slrn_Editor_Uses_Mime_Charset)
      slrn_chmap_fix_file (file, 0);
 
    if ((fp = fopen (file, "r")) == NULL)
      {
-	slrn_error ("File not found: %s--- message not posted.", file);
+	slrn_error (_("File not found: %s--- message not posted."), file);
 	return -1;
      }
    
@@ -989,24 +1080,25 @@ int slrn_post_file (char *file, char *to, int is_postponed)
      slrn_mime_scan_file (fp);
 #endif
 #if SLRN_HAS_SLANG
-   if (NULL != SLang_get_function ("post_file_hook"))
+   if (1 == slrn_is_hook_defined (HOOK_POST_FILE))
      {
 	fclose (fp);
-	(void) SLang_run_hooks ("post_file_hook", 1, file);
+	(void) slrn_run_hooks (HOOK_POST_FILE, 1, file);
 	if (SLang_Error)
 	  {
-	     slrn_error ("post_file_hook returned error.  %s not posted.", file);
-	     slrn_chmap_fix_file (file, 1);
+	     slrn_error (_("post_file_hook returned error.  %s not posted."), file);
+	     if (!Slrn_Editor_Uses_Mime_Charset)
+	       slrn_chmap_fix_file (file, 1);
 	     (void) saved_failed_post (file);
 	     return -1;
 	  }
 
 	if ((fp = fopen (file, "r")) == NULL)
 	  {
-	     slrn_error ("File not found: %s--- message not posted.", file);
+	     slrn_error (_("File not found: %s--- message not posted."), file);
 	     return -1;
 	  }
-	slrn_message_now ("Posting ...");
+	slrn_message_now (_("Posting ..."));
      }
 #endif
 
@@ -1015,10 +1107,11 @@ int slrn_post_file (char *file, char *to, int is_postponed)
    if (status != CONT_POST)
      {
 	fclose (fp);
-	slrn_chmap_fix_file (file, 1);
+	if (!Slrn_Editor_Uses_Mime_Charset)
+	  slrn_chmap_fix_file (file, 1);
 	(void) saved_failed_post (file);
 	if (status != -1)
-	  slrn_error ("Posting not allowed.");
+	  slrn_error (_("Posting not allowed."));
 	return -1;
      }
 
@@ -1029,7 +1122,8 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 
 #if SLRN_HAS_STRICT_FROM
    /* line is not used until now, so we can use it as a buffer */
-   slrn_strncpy(line, slrn_make_from_string (), sizeof(line));
+   if (NULL == (linep = slrn_make_from_string ())) return -1;
+   slrn_strncpy(line, linep, sizeof(line));
 
 # if SLRN_HAS_MIME
    if (Slrn_Use_Mime & MIME_DISPLAY)
@@ -1072,6 +1166,7 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 
 	     if (*white == '\n')
 	       {
+		  slrn_add_date_header (NULL);
 #if SLRN_HAS_GEN_MSGID
 		  if (has_messageid == 0)
 		    {
@@ -1141,11 +1236,12 @@ int slrn_post_file (char *file, char *to, int is_postponed)
    slrn_fclose (fp);
 
    if (0 == Slrn_Post_Obj->po_end ())
-     slrn_message ("Posting...done.");
+     slrn_message (_("Posting...done."));
    else
      {
 	/* convert back to users' charset, so it can easily be re-edited */
-	slrn_chmap_fix_file (file, 1); 
+	if (!Slrn_Editor_Uses_Mime_Charset)
+	  slrn_chmap_fix_file (file, 1); 
 	if (Slrn_Batch) 
 	  {
 	     saved_failed_post (file);
@@ -1154,13 +1250,21 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 
 	/* slrn_set_suspension (0); */
 	slrn_smg_refresh ();
-	sleep (2);
+	slrn_sleep (2);
 	slrn_clear_message (); SLang_Error = 0;
-	rsp = slrn_get_response ("RrEeCc\007", "Select one: \001Repost, \001Edit, \001Cancel");
+   /* Note to translators: Here, "rR" is "repost", "eE" is "edit" and "cC" is
+    * "cancel". Don't change the length of the string; you cannot re-use any
+    * of the default characters for different fields.
+    */
+	responses = _("rReEcC");
+	if (strlen (responses) != 6)
+	  responses = "";
+	rsp = slrn_get_response ("rReEcC\007", responses, _("Select one: \001Repost, \001Edit, \001Cancel"));
+	if (rsp == 7) rsp = 'c';
+	else rsp = slrn_map_translated_char ("rReEcC", responses, rsp) | 0x20;
 	
-	if ((rsp == 7)
-	    || (rsp == 'c') || (rsp == 'C')
-	    || (((rsp == 'e') || (rsp == 'E'))
+	if ((rsp == 'c')
+	    || ((rsp == 'e')
 		&& (slrn_edit_file (Slrn_Editor_Post, file, 1, 0) < 0)))
 	  {
 	     (void) saved_failed_post (file);
@@ -1180,7 +1284,7 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 int slrn_post (char *newsgroup, char *followupto, char *subj)
 {
    FILE *fp;
-   char file[SLRN_MAX_PATH_LEN];
+   char file[SLRN_MAX_PATH_LEN], *from;
    unsigned int header_lines;
    int ret;
    
@@ -1191,14 +1295,15 @@ int slrn_post (char *newsgroup, char *followupto, char *subj)
    
    if (fp == NULL)
      {
-	slrn_error ("Unable to create %s.", file);
+	slrn_error (_("Unable to create %s."), file);
 	return -1;
      }
    
    header_lines = 8;
    fprintf (fp, "Newsgroups: %s\n", newsgroup);
 #if ! SLRN_HAS_STRICT_FROM
-   fprintf (fp, "From: %s\n", slrn_make_from_string ()); header_lines++;
+   if (NULL == (from = slrn_make_from_string ())) return -1;
+   fprintf (fp, "From: %s\n", from); header_lines++;
 #endif
    fprintf (fp, "Subject: %s\n", subj);
    
@@ -1219,6 +1324,9 @@ int slrn_post (char *newsgroup, char *followupto, char *subj)
    slrn_add_signature (fp);
    slrn_fclose (fp);
    
+   if (Slrn_Editor_Uses_Mime_Charset)
+     slrn_chmap_fix_file (file, 0);
+   
    if (slrn_edit_file (Slrn_Editor_Post, file, header_lines, 1) >= 0)
      ret = slrn_post_file (file, NULL, 0);
    else ret = -1;
@@ -1279,7 +1387,7 @@ static int get_postpone_dir (char *dirbuf, size_t n)
        || (-1 == slrn_make_home_dirname (dir, dirbuf, n))
        || (2 != slrn_file_exists (dirbuf)))
      {
-	slrn_error_now (2, "postpone_directory not specified or does not exist");
+	slrn_error_now (2, _("postpone_directory not specified or does not exist"));
 	return -1;
      }
 
@@ -1290,7 +1398,7 @@ static int get_postpone_dir (char *dirbuf, size_t n)
 static int postpone_file (char *file)
 {
 #ifdef VMS
-   slrn_error ("Not implemented yet. Sorry");
+   slrn_error (_("Not implemented yet. Sorry"));
    return -1;
 #else
    char dir [SLRN_MAX_PATH_LEN];
@@ -1306,13 +1414,13 @@ static int postpone_file (char *file)
 	if (-1 == slrn_dircat (dir, NULL, dirfile, sizeof (dirfile)))
 	  return -1;
 
-	if (-1 == slrn_read_input ("Save to", NULL, dirfile, 1, 1))
+	if (-1 == slrn_read_filename (_("Save to: "), NULL, dirfile, 1, 1))
 	  return -1;
 	
 	status = slrn_file_exists (dirfile);
 	if (status == 1)
 	  {
-	     status = slrn_get_yesno_cancel ("File exists, overwrite");
+	     status = slrn_get_yesno_cancel (_("File exists, overwrite"));
 	     if (status == 0)
 	       continue;
 	     if (status == 1)
@@ -1324,9 +1432,9 @@ static int postpone_file (char *file)
 	if (status == 0)
 	  break;
 	if (status == 2)
-	  slrn_error_now (2, "You must specify a name");
+	  slrn_error_now (2, _("You must specify a name"));
 	else
-	  slrn_error_now (2, "Illegal filename. Try again");
+	  slrn_error_now (2, _("Illegal filename. Try again"));
      }
    
    if (-1 == slrn_move_file (file, dirfile))
@@ -1350,7 +1458,7 @@ void slrn_post_postponed (void)
    
    if (1 != slrn_file_exists (file))
      {
-	slrn_error ("%s: not a regular file");
+	slrn_error (_("%s: not a regular file"));
 	return;
      }
    
@@ -1359,9 +1467,3 @@ void slrn_post_postponed (void)
 
    slrn_free (file);
 }
-
-   
-
-	     
-	  
-	  
