@@ -19,7 +19,7 @@
  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 extern int Slrn_Apply_Score;
-extern int slrn_score_header (Slrn_Header_Type *, char *);
+extern char *Slrn_Scorefile_Open;
 extern int slrn_read_score_file (char *);
 extern void slrn_close_score (void);
 extern int slrn_open_score (char *);
@@ -27,3 +27,18 @@ extern int slrn_open_score (char *);
 #define SLRN_EXPENSIVE_SCORING 2
 extern int Slrn_Perform_Scoring;
 extern int Slrn_Prefer_Head;
+
+/* This struct holds the information which scores match the selected article */
+typedef struct Slrn_Score_Debug_Info_Type
+{
+   const char *filename;
+   int linenumber;
+   const char *description;
+   int score;
+   int stop_here;
+   struct Slrn_Score_Debug_Info_Type *next;
+}
+Slrn_Score_Debug_Info_Type;
+
+extern int slrn_score_header (Slrn_Header_Type *, char *,
+			      Slrn_Score_Debug_Info_Type **);
