@@ -99,7 +99,7 @@ static int inews_start_post (void)
     */
    if (NULL == (Fp_Inews = popen (Slrn_Inews_Pgm, "w")))
      {
-	slrn_error ("Couldn't open pipe to inews! -- Article not posted.");
+	slrn_error (_("Couldn't open pipe to inews! -- Article not posted."));
 	return -1;
      }
    return CONT_POST;
@@ -111,7 +111,7 @@ static int inews_end_post (void)
    
    if (-1 == pclose (Fp_Inews))
      {
-	slrn_error ("pclose() failed -- check if article was posted"); /* !HACK! can we do better? */
+	slrn_error (_("pclose() failed -- check if article was posted")); /* !HACK! can we do better? */
 	res = -1;
      }
    Fp_Inews=NULL;
@@ -168,7 +168,7 @@ static int inews_select_post_object (void)
    
    if (1 != slrn_file_exists (inews))
      {
-	slrn_error ("Unable to locate inews program \"%s\"", inews);
+	slrn_error (_("Unable to locate inews program \"%s\""), inews);
 	return -1;
      }
    
@@ -217,7 +217,7 @@ int slrn_init_objects (void)
    if ((server != NULL) 
        && (-1 == SLdefine_for_ifdef (server)))
      {
-	slrn_exit_error ("Unable to add preprocessor name %s.", server);
+	slrn_exit_error (_("Unable to add preprocessor name %s."), server);
      }
    
    return 0;
@@ -240,7 +240,7 @@ int slrn_select_post_object (int id)
 	if (Slrn_Use_Pull_Post
 	    && (Slrn_Server_Id == SLRN_SERVER_ID_SPOOL))
 	  {
-	     slrn_error ("inews cannot be used with an slrnpull spool.");
+	     slrn_error (_("inews cannot be used with an slrnpull spool."));
 	     return -1;
 	  }
 # endif
@@ -264,7 +264,7 @@ int slrn_select_post_object (int id)
 	if (post == NULL)
 	  post = "UNKNOWN";
 
-	slrn_error ("%s is not a supported posting agent.", post);
+	slrn_error (_("%s is not a supported posting agent."), post);
      }
    
    return -1;
@@ -304,7 +304,7 @@ int slrn_select_server_object (int id)
 	if (server == NULL)
 	  server = "UNKNOWN";
 
-	slrn_error ("%s is not a supported server object.", server);
+	slrn_error (_("%s is not a supported server object."), server);
      }
    return -1;
 }
@@ -500,7 +500,7 @@ static int pull_start_post (void)
 		   S_IREAD | S_IWRITE);
 	if ((-1 == fd) && (n == 100))
           {
-	     slrn_error ("Unable to open file in %s.", Pull_Post_Dir);
+	     slrn_error (_("Unable to open file in %s."), Pull_Post_Dir);
 	     return ERR_FAULT;
 	  }
      }
@@ -535,12 +535,15 @@ static int pull_end_post (void)
     /*
      * 1998/07/07 Sylvain Robitaille (syl@alcor.concordia.ca)
      *            We have our out.going directory setgid to news, and we
-     *            want to be sure the file is readable and writable to
+     *            want to be sure the file is readable to
      *            group news. This way, the news user will have
      *            appropriate permissions for manipulating this file.
      *            (NOTE: This has only been tested on Linux-2.0.xx)
+     * 
+     * As of version 0.9.7.3, the files is not made group writeable any
+     * longer. slrnpull should not need that permission.
      */
-    if (-1 == chmod (Pull_Post_Filename, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP))
+    if (-1 == chmod (Pull_Post_Filename, S_IRUSR|S_IWUSR|S_IRGRP))
       {
 	 return ERR_FAULT;
       }
@@ -567,7 +570,7 @@ static int pull_end_post (void)
 	  }
 #endif
        
-	slrn_error ("Unable to rename file. errno = %d.", errno);
+	slrn_error (_("Unable to rename file. errno = %d."), errno);
 	return ERR_FAULT;
      }
    
@@ -582,7 +585,7 @@ static int pull_puts (char *s)
 	
    if (len != fwrite (s, 1, len, Pull_Fp))
      {
-	slrn_error ("Write error.  File system full?");
+	slrn_error (_("Write error.  File system full?"));
 	return -1;
      }
 
@@ -601,7 +604,7 @@ static int pull_printf (char *fmt, ...)
    
    if (EOF == ret)
      {
-	slrn_error ("Write failed.  File system full?");
+	slrn_error (_("Write failed.  File system full?"));
 	return -1;
      }
    
@@ -628,7 +631,7 @@ static int pull_select_post_object (void)
    
    if (2 != slrn_file_exists (Pull_Post_Dir))
      {
-	slrn_error ("Posting directory %s does not exist.", Pull_Post_Dir);
+	slrn_error (_("Posting directory %s does not exist."), Pull_Post_Dir);
 	return -1;
      }
    
