@@ -71,7 +71,6 @@
 #include <slang.h>
 #include "jdmacros.h"
 
-
 #include <errno.h>
 
 #include "util.h"
@@ -87,6 +86,7 @@
 #include "post.h"
 #include "xover.h"
 #include "mime.h"
+#include "hooks.h"
 
 #if SLRN_USE_SLTCP
 # include "sltcp.h"
@@ -122,7 +122,6 @@ int Slrn_Batch;
 int Slrn_Suspension_Ok;
 
 char *Slrn_Newsrc_File = NULL;
-char *Slrn_Lib_Dir = NULL;
 Slrn_Mode_Type *Slrn_Current_Mode;
 
 int Slrn_Default_Server_Obj = SLRN_DEFAULT_SERVER_OBJ;
@@ -169,12 +168,12 @@ static void test_lock( char *file ) /*{{{*/
 		 )
 	       {
 #if defined(IBMPC_SYSTEM)
-		  slrn_exit_error ("\
+		  slrn_exit_error (_("\
 slrn: pid %d is locking the newsrc file. If you're not running another\n\
-      copy of slrn, delete the file %s", 
+      copy of slrn, delete the file %s"), 
 				   pid, file);
 #else
-		  slrn_exit_error ("slrn: pid %d is locking the newsrc file.", pid);
+		  slrn_exit_error (_("slrn: pid %d is locking the newsrc file."), pid);
 #endif
 	       }
 	  }
@@ -245,14 +244,14 @@ static void lock_file (int how) /*{{{*/
 #endif
 	if (-1 == make_lock (name))
 	  {
-	     slrn_exit_error ("Unable to create lock file %s.", name);
+	     slrn_exit_error (_("Unable to create lock file %s."), name);
 	  }
 	
 #if SLRN_HAS_RNLOCK
 	if (rnlock && (-1 == make_lock (file_rn)))
 	  {
 	     slrn_delete_file (name); /* delete the "normal" lock file */
-	     slrn_exit_error ("Unable to create lock file %s.", file_rn);
+	     slrn_exit_error (_("Unable to create lock file %s."), file_rn);
 	  }
 #endif       
      }
@@ -359,7 +358,7 @@ void slrn_pop_suspension (void) /*{{{*/
    
    if (Suspension_Stack_Depth == 0)
      {
-	slrn_error ("pop_suspension: underflow!");
+	slrn_error (_("pop_suspension: underflow!"));
 	return;
      }
    
@@ -424,7 +423,7 @@ static void perform_suspend (int smg_suspend_flag) /*{{{*/
 {
 #if !defined(SIGSTOP) || !defined(REAL_UNIX_SYSTEM)
    (void) smg_suspend_flag;
-   slrn_error ("Not implemented.");
+   slrn_error (_("Not implemented."));
    Want_Suspension = 0;
 #else
    
@@ -437,7 +436,7 @@ static void perform_suspend (int smg_suspend_flag) /*{{{*/
    Want_Suspension = 0;
    if (Can_Suspend == 0)
      {
-	slrn_error ("Suspension not allowed by shell.");
+	slrn_error (_("Suspension not allowed by shell."));
 	return;
      }
    
@@ -456,7 +455,7 @@ static void perform_suspend (int smg_suspend_flag) /*{{{*/
    Want_Suspension = 0;
    if (Can_Suspend == 0)
      {
-	slrn_error ("Suspension not allowed by shell.");
+	slrn_error (_("Suspension not allowed by shell."));
 	return;
      }
    
@@ -617,7 +616,7 @@ static int init_tty (void) /*{{{*/
      init_display_signals (1);
 
    if (-1 == SLang_init_tty (7, !Slrn_Use_Flow_Control, 0))
-     slrn_exit_error ("Unable to initialize the tty");
+     slrn_exit_error (_("Unable to initialize the tty"));
 
 #ifdef REAL_UNIX_SYSTEM
    SLang_getkey_intr_hook = slrn_handle_interrupts;
@@ -820,7 +819,7 @@ static void perform_cleanup (void)
 void slrn_quit (int retcode) /*{{{*/
 {
    perform_cleanup ();
-   if (retcode) fprintf (stderr, "slrn: quiting on signal %d.\n", retcode);
+   if (retcode) fprintf (stderr, _("slrn: quiting on signal %d.\n"), retcode);
    exit (retcode);
 }
 
@@ -837,10 +836,14 @@ void slrn_va_exit_error (char *fmt, va_list ap)
 	   
 	if (fmt != NULL)
 	  {
-	     fprintf (stderr, "slrn fatal error:\n");
+	     fputs (_("slrn fatal error:"), stderr);
+#ifdef IBMPC_SYSTEM
+	     fputc ('\r', stderr);
+#endif
+	     fputc ('\n', stderr);
 	     vfprintf (stderr, fmt, ap);
 	  }   
-	if (Slrn_Groups_Dirty) slrn_write_newsrc ();
+	if (Slrn_Groups_Dirty) slrn_write_newsrc (0);
 	perform_cleanup ();
      }
 
@@ -861,7 +864,7 @@ void slrn_exit_error (char *fmt, ...) /*{{{*/
 
 static void usage (char *extra) /*{{{*/
 {
-   puts ("\
+   printf (_("\
 Usage: slrn [--inews] [--nntp ...] [--spool] OPTIONS\n\
 -a              Use active file for getting new news.\n\
 -f newsrc-file  Name of the newsrc file to use.\n\
@@ -870,7 +873,7 @@ Usage: slrn [--inews] [--nntp ...] [--spool] OPTIONS\n\
 -d              Get new text descriptions of each group from server.\n\
                  Note: This may take a LONG time to retrieve this information.\n\
                  The resulting file can be several hundred Kilobytes!\n\
--i init-file    Name of initialization file to use (default: "SLRN_USER_SLRNRC_FILENAME")\n\
+-i init-file    Name of initialization file to use (default: %s)\n\
 -k              Do not process score file.\n\
 -k0             Process score file but inhibit expensive scores.\n\
 -m              Force XTerm mouse reporting\n\
@@ -884,7 +887,7 @@ Usage: slrn [--inews] [--nntp ...] [--spool] OPTIONS\n\
 --version       Show version and supported features\n\
 \n\
 NNTP mode has additional options; use \"slrn --nntp --help\" to display them.\n\
-");
+"), SLRN_USER_SLRNRC_FILENAME);
    
    if (extra != NULL)
      {
@@ -911,9 +914,9 @@ static int parse_object_args (char *obj, char **argv, int argc) /*{{{*/
    if (num_parsed < 0)
      {
 	if (num_parsed == -1)
-	  slrn_exit_error ("%s is not a supported option.", *argv);
+	  slrn_exit_error (_("%s is not a supported option."), *argv);
 	else 
-	  slrn_exit_error ("%s is not supported.", obj);
+	  slrn_exit_error (_("%s is not supported."), obj);
      }
    if ((num_parsed == 0) && (zero_ok == 0))
      usage (NULL);
@@ -934,13 +937,13 @@ static void read_score_file (void)
    
    if (1 != slrn_file_exists (file))
      {
-	slrn_message_now ("*** Warning: Score file %s does not exist", file);
+	slrn_message_now (_("*** Warning: Score file %s does not exist"), file);
 	return;
      }
 
    if (-1 == slrn_read_score_file (file))
      {
-	slrn_exit_error ("Error processing score file %s.", file);
+	slrn_exit_error (_("Error processing score file %s."), file);
      }
 }
 
@@ -962,13 +965,12 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
    char *init_file = NULL;
    char *kill_logfile = NULL;
 
-#ifdef HAVE_SETLOCALE
-# ifdef LC_CTYPE
-   (void) setlocale (LC_CTYPE, "");
-# endif
-# ifdef LC_TIME
-   (void) setlocale (LC_TIME, "");
-# endif
+#if defined(HAVE_SETLOCALE) && defined(LC_ALL)
+   (void) setlocale(LC_ALL, "");
+#endif
+#ifdef ENABLE_NLS
+   bindtextdomain(PACKAGE,LOCALEDIR);
+   textdomain(PACKAGE);
 #endif
 
    check_for_suspension ();
@@ -1033,7 +1035,7 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 		    fclose (Slrn_Debug_Fp);
                   Slrn_Debug_Fp = fopen (argv[++i], "w");
                   if (Slrn_Debug_Fp == NULL)
-                    slrn_exit_error ("Unable to open %s for debugging.", argv[i]);
+                    slrn_exit_error (_("Unable to open %s for debugging."), argv[i]);
                   setbuf (Slrn_Debug_Fp, (char *) NULL);
                }
 	     else usage (NULL);
@@ -1054,7 +1056,7 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 	  {
 	     if (-1 == SLdefine_for_ifdef (argv_i + 2))
 	       {
-		  slrn_exit_error ("Unable to add preprocessor name %s.",
+		  slrn_exit_error (_("Unable to add preprocessor name %s."),
 				   argv_i + 2);
 	       }
 	  }
@@ -1079,7 +1081,7 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 
    if (dsc_flag && create_flag)
      {
-	usage ("The -d and --create flags must not be specified together.");
+	usage (_("The -d and --create flags must not be specified together."));
      }
    
    if (Slrn_Batch == 0)
@@ -1092,12 +1094,12 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 #if SLRN_USE_SLTCP && SLRN_HAS_NNTP_SUPPORT
    /* This is necessary to ensure that gethostname works on Win32 */
    if (-1 == sltcp_open_sltcp ())
-     slrn_exit_error ("Error initializing SLtcp interface");
+     slrn_exit_error (_("Error initializing SLtcp interface"));
 #endif
 
 #if SLRN_HAS_SLANG
    if (-1 == slrn_init_slang ())
-     fprintf (stderr, "Error initializing S-Lang interpreter.\n");
+     fprintf (stderr, _("Error initializing S-Lang interpreter.\n"));
 #endif
 
    slrn_startup_initialize ();
@@ -1108,23 +1110,17 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
     */
    if (-1 == slrn_init_objects ())
      {
-	slrn_exit_error ("Error configuring server objects.");
+	slrn_exit_error (_("Error configuring server objects."));
      }
 
    slrn_get_user_info ();
 
    slrn_init_hangup_signals (1);
 
-   if (Slrn_Lib_Dir == NULL)
-     {
-	Slrn_Lib_Dir = getenv ("SLRN_LIB_DIR");
-     }
-   if (Slrn_Lib_Dir == NULL) Slrn_Lib_Dir = SLRN_LIB_DIR;
-   
 #ifdef VMS
-   slrn_snprintf (file, sizeof (file), "%s%s", Slrn_Lib_Dir, "slrn.rc");
+   slrn_snprintf (file, sizeof (file), "%s%s", SYSCONFDIR, "slrn.rc");
 #else
-   slrn_snprintf (file, sizeof (file), "%s/%s", Slrn_Lib_Dir, "slrn.rc");
+   slrn_snprintf (file, sizeof (file), "%s/%s", SYSCONFDIR, "slrn.rc");
 #endif
 
    /* Make sure terminal is initialized before setting colors.  The 
@@ -1138,15 +1134,19 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 	init_file = file;
      }
    
-   slrn_read_startup_file (init_file);
+   if ((-1 == slrn_read_startup_file (init_file)) && (init_file != file))
+     {
+	slrn_exit_error (_("\
+Could not read specified config file %s\n"), init_file);
+     }
    
    if (Slrn_Saw_Obsolete)
      {
-	slrn_message ("\n! Your configuration file contains obsolete commands or function names that\n"
+	slrn_message (_("\n! Your configuration file contains obsolete commands or function names that\n"
 		      "! will not be supported by future versions of this program.\n"
 		      "! If you have Perl installed, you can use the script slrnrc-conv to change\n"
 		      "! your configuration accordingly.  It can be found in the source distribution\n"
-		      "! or retrieved from <URL:ftp://slrn.sourceforge.net/pub/slrn/>.\n");
+		      "! or retrieved from <URL:http://slrn.sourceforge.net/data/>.\n"));
      }
 
    if (Slrn_Server_Id == 0) Slrn_Server_Id = Slrn_Default_Server_Obj;
@@ -1157,7 +1157,7 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
    if (-1 == slrn_set_charset (Slrn_Charset))
      {
 	slrn_chmap_show_supported ();
-	slrn_exit_error ("Failed to select character set.");
+	slrn_exit_error (_("Failed to select character set."));
      }
 #endif
 
@@ -1191,7 +1191,7 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
    if ((-1 == slrn_select_server_object (Slrn_Server_Id))
        || (-1 == slrn_select_post_object (Slrn_Post_Id)))
      {
-	slrn_exit_error ("Unable to select server/post object.");
+	slrn_exit_error (_("Unable to select server/post object."));
      }
 
 #if !defined(IBMPC_SYSTEM)
@@ -1207,27 +1207,27 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
    if ((NULL == Slrn_User_Info.hostname)
        || (0 == slrn_is_fqdn (Slrn_User_Info.hostname)))
      {
-	slrn_exit_error ("\
-Unable to find a valid hostname for constructing your e-mail address.\r\n\
-You will have to specify a hostname in your %s file.\r\n", 
+	slrn_exit_error (_("\
+Unable to find a valid hostname for constructing your e-mail address.\n\
+You will have to specify a hostname in your %s file.\n"), 
 			 SLRN_USER_SLRNRC_FILENAME);
      }
    if ((NULL == Slrn_User_Info.username)
        || (0 == *Slrn_User_Info.username)
        || (NULL != slrn_strchr (Slrn_User_Info.username, '@')))
      {
-	slrn_exit_error ("\
-Unable to find your user name.  This means that a valid 'From' header line\r\n\
-cannot be constructed.  Try setting the USER environment variable.\r\n");
+	slrn_exit_error (_("\
+Unable to find your user name.  This means that a valid 'From' header line\n\
+cannot be constructed.  Try setting the USER environment variable.\n"));
      }
 
    if ((NULL == Slrn_User_Info.posting_host)
        && Slrn_Generate_Message_Id)
-     slrn_message_now ("\
-***Warning: Unable to find a unique fully-qualified host name.\r\n\
-            slrn will not generate any Message-IDs.\r\n\
-            Please note that the \"hostname\" setting does not affect this;\r\n\
-            see the \"slrn reference manual\" for details.\r\n");
+     slrn_stderr_strcat (_("\
+***Warning: Unable to find a unique fully-qualified host name."), "\n", _("\
+            slrn will not generate any Message-IDs."), "\n", _("\
+            Please note that the \"hostname\" setting does not affect this;"), "\n", _("\
+            see the \"slrn reference manual\" for details."), "\n", NULL);
    
    if (no_score_file == 0) read_score_file ();
    
@@ -1235,9 +1235,9 @@ cannot be constructed.  Try setting the USER environment variable.\r\n");
      {
 	hlp_file = file;
 #ifdef VMS
-	slrn_snprintf (file, sizeof (file), "%s%s", Slrn_Lib_Dir, "help.txt");
+	slrn_snprintf (file, sizeof (file), "%s%s", SYSCONFDIR, "help.txt");
 #else
-	slrn_snprintf (file, sizeof (file), "%s/%s", Slrn_Lib_Dir, "help.txt");
+	slrn_snprintf (file, sizeof (file), "%s/%s", SYSCONFDIR, "help.txt");
 #endif
      }
 	
@@ -1255,7 +1255,7 @@ cannot be constructed.  Try setting the USER environment variable.\r\n");
 	Slrn_Newsrc_File = slrn_safe_strmalloc (file);
      }
    
-   slrn_message_now ("Using newsrc file %s for server %s.",
+   slrn_message_now (_("Using newsrc file %s for server %s."),
 		     Slrn_Newsrc_File, Slrn_Server_Obj->sv_name);
    
    if (use_active) Slrn_List_Active_File = 1;
@@ -1267,7 +1267,7 @@ cannot be constructed.  Try setting the USER environment variable.\r\n");
      {
 	if (Slrn_Server_Obj->sv_initialize () != 0)
 	  {
-	     slrn_exit_error ("Unable to initialize server.");
+	     slrn_exit_error (_("Unable to initialize server."));
 	  }
 	slrn_get_group_descriptions ();
 	Slrn_Server_Obj->sv_close ();
@@ -1281,14 +1281,18 @@ cannot be constructed.  Try setting the USER environment variable.\r\n");
 	 */
 	if (NULL == (fp = fopen (Slrn_Newsrc_File, "r")))
 	  {
-	     slrn_error ("Unable to open %s.  I will try .newsrc.", Slrn_Newsrc_File);
+#if 0 /* now disabled in group.c */
+	     slrn_error (_("Unable to open %s.  I will try .newsrc."), Slrn_Newsrc_File);
 	     if (NULL == (fp = slrn_open_home_file (".newsrc", "r", file,
 						    sizeof (file), 0)))
 	       {
-		  slrn_exit_error ("\r\nUnable to open %s.\r\n\
-If you want to create %s, add command line options:\r\n\
-   -f %s --create\r\n", file, file, file);
+#endif
+		  slrn_exit_error (_("\nUnable to open %s.\n\
+If you want to create %s, add command line options:\n\
+   -f %s --create\n"), Slrn_Newsrc_File, Slrn_Newsrc_File, Slrn_Newsrc_File);
+#if 0
 	       }
+#endif
 	  }
 	slrn_fclose (fp);
      }
@@ -1300,13 +1304,13 @@ If you want to create %s, add command line options:\r\n\
      {
 	Slrn_Kill_Log_FP = fopen(kill_logfile, "a");
 	if (Slrn_Kill_Log_FP == NULL)
-	  slrn_error ("Unable to open %s for logging.", kill_logfile);
+	  slrn_error (_("Unable to open %s for logging."), kill_logfile);
      }
    
    lock_file (1);
 
 #if SLRN_HAS_SLANG
-   (void) SLang_run_hooks ("startup_hook", 0);
+   (void) slrn_run_hooks (HOOK_STARTUP, 0);
 #endif
    
    
@@ -1316,10 +1320,10 @@ If you want to create %s, add command line options:\r\n\
 
    if (Slrn_Use_Group_Lens && (Slrn_Batch == 0))
      {
-	slrn_message ("Initializing GroupLens");
+	slrn_message (_("Initializing GroupLens"));
 	if (-1 == slrn_init_grouplens ())
 	  {
-	     fprintf (stderr, "GroupLens disabled.\n");
+	     fprintf (stderr, _("GroupLens disabled.\n"));
 	     Slrn_Use_Group_Lens = 0;
 	  }
      }
@@ -1327,13 +1331,13 @@ If you want to create %s, add command line options:\r\n\
 
    if (Slrn_Server_Obj->sv_initialize () != 0)
      {
-	slrn_exit_error ("Failed to initialize server.");
+	slrn_exit_error (_("Failed to initialize server."));
      }
    
    if (Slrn_Server_Obj->sv_has_xover == 1)
      {
 	if (0 == (Slrn_Server_Obj->sv_has_xover = slrn_read_overview_fmt ()))
-	  slrn_message ("OVERVIEW.FMT not RFC 2980 compliant -- XOVER support disabled.");
+	  slrn_message (_("OVERVIEW.FMT not RFC 2980 compliant -- XOVER support disabled."));
      }
    
    if (Slrn_Check_New_Groups || create_flag)
@@ -1346,10 +1350,10 @@ If you want to create %s, add command line options:\r\n\
    
    if (wait_for_key)
      {
-	slrn_message ("* Press Ctrl-C to quit, any other key to continue.\n");
+	slrn_message (_("* Press Ctrl-C to quit, any other key to continue.\n"));
 	slrn_set_display_state (SLRN_TTY_INIT);
 	if ('\003' == SLang_getkey ())
-	  slrn_exit_error ("Exit on user request.");
+	  slrn_exit_error (_("Exit on user request."));
      }
    else
      putc ('\n', stdout);
@@ -1376,7 +1380,7 @@ static unsigned int Mode_Stack_Depth;
 void slrn_push_mode (Slrn_Mode_Type *mode)
 {
    if (Mode_Stack_Depth == MAX_MODE_STACK_LEN)
-     slrn_exit_error ("Internal Error: Mode_Stack overflow");
+     slrn_exit_error (_("Internal Error: Mode_Stack overflow"));
 
    Mode_Stack[Mode_Stack_Depth] = Slrn_Current_Mode;
    Mode_Stack_Depth++;
@@ -1391,7 +1395,7 @@ void slrn_push_mode (Slrn_Mode_Type *mode)
 void slrn_pop_mode (void)
 {
    if (Mode_Stack_Depth == 0)
-     slrn_exit_error ("Internal Error: Mode_Stack underflow");
+     slrn_exit_error (_("Internal Error: Mode_Stack underflow"));
    
    Mode_Stack [Mode_Stack_Depth] = NULL;   /* null current mode */
    Mode_Stack_Depth--;
@@ -1440,7 +1444,7 @@ static void run_winch_functions (int old_r, int old_c)
 
 #if SLRN_HAS_SLANG
    if (SLang_Error == 0)
-     SLang_run_hooks ("resize_screen_hook", 0);
+     slrn_run_hooks (HOOK_RESIZE_SCREEN, 0);
 #endif
 }
 
@@ -1462,7 +1466,7 @@ static void slrn_hangup (int sig) /*{{{*/
 	  (*mode->hangup_fun) (sig);
      }
    
-   slrn_write_newsrc ();
+   slrn_write_newsrc (0);
    slrn_quit (sig);
 }
 
@@ -1488,7 +1492,7 @@ void slrn_call_command (char *cmd) /*{{{*/
 	list++;
      }
    
-   slrn_error ("call: %s not in current keymap.", cmd);
+   slrn_error (_("call: %s not in current keymap."), cmd);
 }
 
 /*}}}*/
@@ -1667,9 +1671,9 @@ int main (int argc, char **argv) /*{{{*/
      return 1;
 
    slrn_push_suspension (1);
-   
+
 #if SLRN_HAS_SLANG
-   (void) SLang_run_hooks ("group_mode_startup_hook", 0);
+   (void) slrn_run_hooks (HOOK_GROUP_MODE_STARTUP, 0);
 #endif
 
    if (Slrn_Batch) return 1;
@@ -1679,7 +1683,7 @@ int main (int argc, char **argv) /*{{{*/
 	if (SLKeyBoard_Quit)
 	  {
 	     SLKeyBoard_Quit = 0;
-	     slrn_error ("Quit!");
+	     slrn_error (_("Quit!"));
 	  }
 	
 	(void) slrn_handle_interrupts ();
@@ -1712,4 +1716,3 @@ int slrn_posix_system (char *cmd, int reset) /*{{{*/
 }
 
 /*}}}*/
-
