@@ -15,8 +15,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #ifndef _SLRN_SLRN_H_
 #define _SLRN_SLRN_H_
@@ -81,7 +81,6 @@ void slrn_va_exit_error (char *, va_list);
 extern int Slrn_Default_Server_Obj;
 extern int Slrn_Default_Post_Obj;
 
-extern char *Slrn_Lib_Dir;
 extern int Slrn_Batch;
 extern FILE *Slrn_Debug_Fp;
 
@@ -111,6 +110,7 @@ extern int slrn_sys_system (char *);
 #define FRAME_COLOR		22
 #define SELECT_COLOR		23
 #define VERBATIM_COLOR		24
+#define MESSAGE_COLOR		25
 #define DATE_COLOR		36
 /* SUBJECT_COLOR + 5 == HL_SUBJECT_COLOR etc. */
 #define SUBJECT_COLOR		40
