@@ -121,8 +121,11 @@
 # define SLRN_ARTICLE_FILENAME		"slrn-article.txt"
 # define SLRN_FOLLOWUP_FILENAME		"slrn-followup.txt"
 # define SLRN_SIGNATURE_FILE		".signature"
-# ifndef SLRN_LIB_DIR
-#  define SLRN_LIB_DIR			"sys$manager:"
+# ifndef SYSCONFDIR
+#  define SYSCONFDIR			"sys$manager:"
+# endif
+# ifndef SHAREDIR
+#  define SHAREDIR			"sys$manager:"
 # endif
 #else
 # if defined(IBMPC_SYSTEM)
@@ -136,8 +139,11 @@
 #  define SLRN_FOLLOWUP_FILENAME	"followup.txt"
 #  define SLRN_SIGNATURE_FILE		"signatur.txt"
 #  define SLRN_SENDMAIL_COMMAND		"sendmail -t -af"
-#  ifndef SLRN_LIB_DIR
-#    define SLRN_LIB_DIR		"C:/slrn"
+#  ifndef SYSCONFDIR
+#    define SYSCONFDIR			"C:/slrn"
+#  endif
+#  ifndef SHAREDIR
+#    define SHAREDIR			"C:/slrn"
 #  endif
 # else
 #  ifdef __unix__
@@ -154,9 +160,6 @@
 #   ifndef SLRN_SENDMAIL_COMMAND
 #    define SLRN_SENDMAIL_COMMAND	"/usr/lib/sendmail -oi -t -oem -odb"
 #   endif
-#   ifndef SLRN_LIB_DIR
-#    define SLRN_LIB_DIR		"/usr/local/lib/slrn"
-#   endif
 #  endif			       /* unix */
 # endif				       /* os2 */
 #endif				       /* vms */
@@ -189,13 +192,6 @@
 /* slrnpull failed outgoing post directory. */
 #define SLRNPULL_OUTGOING_BAD_DIR	"rejects"
 
-/* If set to 1, slrn will write files to the outgoing directory with group
- * write permission.  This assumes that the outgoing directory has been 
- * created with the setgid bit enabled, which slrnpull will do if this is
- * set to 1.  See slrn/slrnpull/setgid.txt for more information.
- */
-#define SLRNPULL_USE_SETGID_POSTS	0
-
 /* The file that will be used for killing articles as they are fetched. */
 #define SLRNPULL_SCORE_FILE	"score"
 
@@ -217,6 +213,22 @@
 # define SLRN_HAS_GROUPLENS 0
 #endif
 
+/* For the internationalisation using gettext                *
+ * This allow to write _("text") instead of gettext("text")  */
+
+#ifdef ENABLE_NLS
+# include <libintl.h>
+# define _(a) (gettext (a))
+# ifdef gettext_noop
+#  define N_(a) gettext_noop (a)
+# else
+#  define N_(a) (a)
+# endif
+#else
+# define _(a) (a)
+# define N_(a) a
+#endif
+
 /* Sanity checks.  Do not modify. */
 
 #if !defined (SLRN_DEFAULT_SERVER_OBJ)
