@@ -106,6 +106,7 @@ static int Exit_Code;
 #define SLRN_EXIT_FILEIO		20
 
 FILE *Slrn_Debug_Fp = NULL;
+int Slrn_Force_Authentication = 0;
 char *SlrnPull_Dir = SLRNPULL_ROOT_DIR;
 char *SlrnPull_Spool_News_Dir;
 char *Group_Min_Max_File;	       /* relative to group dir */
@@ -168,6 +169,7 @@ typedef struct _Active_Group_Type /*{{{*/
    char name [MAX_GROUP_NAME_LEN + 1];
    char dirname [MAX_GROUP_NAME_LEN + 1];
    struct _Active_Group_Type *next;
+
 }
 
 /*}}}*/
@@ -313,7 +315,7 @@ static int do_mkdir (char *dir, int err) /*{{{*/
 {
    if (0 == MKDIR (dir, 0777))
      {
-	log_message ("Created dir %s.", dir);
+	log_message (_("Created dir %s."), dir);
 	return 0;
      }
    
@@ -321,7 +323,7 @@ static int do_mkdir (char *dir, int err) /*{{{*/
      return 0;
 	
    if (err)
-     log_error ("Unable to create directory %s. (errno = %d)", dir, errno);
+     log_error (_("Unable to create directory %s. (errno = %d)"), dir, errno);
 
    return -1;
 }
@@ -362,7 +364,7 @@ static int read_group_min_max_file (Active_Group_Type *g) /*{{{*/
    if (NULL == fgets (line, sizeof(line), fp))
      {
 	fclose (fp);
-	log_error ("Error reading %s.", file);
+	log_error (_("Error reading %s."), file);
 	return -1;
      }
 
@@ -370,7 +372,7 @@ static int read_group_min_max_file (Active_Group_Type *g) /*{{{*/
    
    if (2 != sscanf (line, "%u %u", &min, &max))
      {
-	log_error ("Error parsing %s.", file);
+	log_error (_("Error parsing %s."), file);
 	return -1;
      }
    
@@ -390,18 +392,18 @@ static int write_group_min_max_file (Active_Group_Type *g) /*{{{*/
    fp = open_group_min_max_file (g, "w", file, sizeof (file));
    if (fp == NULL)
      {
-	log_error ("Unable to open %s for writing.", file);
+	log_error (_("Unable to open %s for writing."), file);
 	return -1;
      }
    if (EOF == fprintf (fp, "%u %u", g->min, g->max))
      {
-	log_error ("Write to %s failed.", file);
+	log_error (_("Write to %s failed."), file);
 	(void) fclose (fp);
 	return -1;
      }
    if (-1 == slrn_fclose (fp))
      {
-	log_error ("Error closing %s.", file);
+	log_error (_("Error closing %s."), file);
 	return -1;
      }
    return 0;
@@ -521,11 +523,16 @@ static int read_active_groups (void) /*{{{*/
    fp = fopen (Active_Groups_File, "r");
    if (fp == NULL)
      {
-	log_error ("Unable to open active groups file %s", Active_Groups_File);
-	return -1;
+	if (NULL == (fp = fopen (SYSCONFDIR "/" SLRNPULL_CONF, "r")))
+	  {
+	     log_error (_("Unable to open active groups file %s"), Active_Groups_File);
+	     return -1;
+	  }
+	else
+	  Active_Groups_File = SYSCONFDIR "/" SLRNPULL_CONF;
      }
    
-   log_message ("Reading %s", Active_Groups_File);
+   log_message (_("Reading %s"), Active_Groups_File);
 
    default_max_to_get = 50;
    default_expire_days = 10;
@@ -554,7 +561,7 @@ static int read_active_groups (void) /*{{{*/
 	name = argv[0]; argc--;
 	if (strlen (name) > MAX_GROUP_NAME_LEN)
 	  {
-	     log_error ("%s: line %u: group name too long.",
+	     log_error (_("%s: line %u: group name too long."),
 			 Active_Groups_File, num);
 	     fclose (fp);
 	     return -1;
@@ -571,7 +578,7 @@ static int read_active_groups (void) /*{{{*/
 	     
 	     if (uch <= 32)
 	       {
-		  log_error ("%s: line %u: Group name contains whitespace.",
+		  log_error (_("%s: line %u: Group name contains whitespace."),
 			      Active_Groups_File, num);
 		  fclose (fp);
 		  return -1;
@@ -589,7 +596,7 @@ static int read_active_groups (void) /*{{{*/
 	     if ((1 != sscanf (arg, "%d", &max_to_get))
 		 || (max_to_get < 0))
 	       {
-		  log_error ("%s: line %u: expecting positive integer in second field.",
+		  log_error (_("%s: line %u: expecting positive integer in second field."),
 			      Active_Groups_File, num);
 		  fclose (fp);
 		  return -1;
@@ -603,7 +610,7 @@ static int read_active_groups (void) /*{{{*/
 	     if ((1 != sscanf (arg, "%d", &expire_days))
 		 || (expire_days < 0))
 	       {
-		  log_error ("%s: line %u: expecting positive integer in third field.",
+		  log_error (_("%s: line %u: expecting positive integer in third field."),
 			      Active_Groups_File, num);
 		  fclose (fp);
 		  return -1;
@@ -620,7 +627,7 @@ static int read_active_groups (void) /*{{{*/
 
 	if (NULL != find_group_type (name))
 	  {
-	     log_error ("%s: line %u: group duplicated.", 
+	     log_error (_("%s: line %u: group duplicated."), 
 			 Active_Groups_File, num);
 	     fclose (fp);
 	     return -1;
@@ -628,7 +635,7 @@ static int read_active_groups (void) /*{{{*/
 	
 	if (NULL == (g = add_group_type (name)))
 	  {
-	     log_error ("%s: line %u: failed to add %s.",
+	     log_error (_("%s: line %u: failed to add %s."),
 			 Active_Groups_File, num, name);
 	     fclose (fp);
 	     return -1;
@@ -662,7 +669,7 @@ static char *do_dircat (char *dir, char *name)
    if (f == NULL)
      {
 	Exit_Code = SLRN_EXIT_MALLOC_FAILED;
-	slrn_exit_error ("Out of memory.");
+	slrn_exit_error (_("Out of memory."));
      }
 
 #if defined(__os2__) || defined(__NT__)
@@ -702,7 +709,7 @@ static int make_filenames (void) /*{{{*/
    if (Outgoing_Bad_Dir == NULL)
      {
 	Exit_Code = SLRN_EXIT_MALLOC_FAILED;
-	slrn_exit_error ("Out of memory.");
+	slrn_exit_error (_("Out of memory."));
      }
 
    if (-1 == do_mkdir (SlrnPull_Spool_News_Dir, 1))
@@ -729,8 +736,8 @@ static int *listgroup_numbers (NNTP_Type *s, char *name, unsigned int *nump) /*{
    status = nntp_listgroup (s, name);
    if (status != OK_GROUP)
      {
-	if (status == -1) log_error ("Read failed.");
-	log_error ("listgroup %s failed: %s", name, nntp_map_code_to_string (status));
+	if (status == -1) log_error (_("Read failed."));
+	log_error (_("listgroup %s failed: %s"), name, nntp_map_code_to_string (status));
 	return NULL;
      }
    
@@ -802,21 +809,21 @@ static int *list_server_numbers (NNTP_Type *s, Active_Group_Type *g,
      {
 	max = (unsigned int) server_max - g->max_to_get + 1;
 	if (server_max >= server_min)
-	  log_message ("The server may contain as many as %d articles in this group",
+	  log_message (_("The server may contain as many as %d articles in this group"),
 		       server_max - server_min + 1);
-	log_message ("Only examining last %u articles since this is a new group", g->max_to_get);
+	log_message (_("Only examining last %u articles since this is a new group"), g->max_to_get);
      }
 
    status = nntp_server_vcmd (s, "XHDR Message-Id %u-", max);
    if (status == -1)
      {
-	log_error ("Server failed to return proper code for XHDR.  The connection may be lost"); 
+	log_error (_("Server failed to return proper code for XHDR.  The connection may be lost")); 
 	return NULL;			       /* server closed? */
      }
    if (status == 224) status = OK_HEAD;/* Micro$oft broken server */
    if (status != OK_HEAD)
      {
-	log_error ("Server failed XHDR command: %s", s->rspbuf);
+	log_error (_("Server failed XHDR command: %s"), s->rspbuf);
 	return NULL;
      }
    
@@ -874,7 +881,7 @@ static int *list_server_numbers (NNTP_Type *s, Active_Group_Type *g,
 	num_numbers++;
      }
 
-   log_message ("%s: Retrieving articles %d-%d.", g->name, min, max);
+   log_message (_("%s: Retrieving articles %d-%d."), g->name, min, max);
    if (num_numbers) g->server_max = max;
    /* Otherwise, there were no unique articles in that range. */
    
@@ -897,7 +904,7 @@ static int *list_server_numbers (NNTP_Type *s, Active_Group_Type *g,
 #if 0
 	g->max = max;
 #else
-	log_message ("%s: No articles in specified range.", g->name);
+	log_message (_("%s: No articles in specified range."), g->name);
 #endif
      }
    
@@ -945,16 +952,17 @@ static void print_time_stats (NNTP_Type *s, int do_log) /*{{{*/
    if (Stdout_Is_TTY)
      {
 	
-	fprintf (stdout, "%u/%u (%u killed), Time: %02u:%02u:%02u, BPS: %lu      \r",
+	fprintf (stdout, _("%u/%u (%u killed), Time: %02u:%02u:%02u, BPS: %lu      "),
 		 Num_Articles_Received, Num_Articles_To_Receive, Num_Killed,
 		 hour, min, sec,
 		 (unsigned long) (in / elapsed_time));
+	fputc ('\r', stdout);
 	fflush (stdout);
      }
    
    if (do_log)
      {
-	log_message ("%s: %u/%u (%u killed), Time: %02u:%02u:%02u, BPS: %lu",
+	log_message (_("%s: %u/%u (%u killed), Time: %02u:%02u:%02u, BPS: %lu"),
 		     Current_Newsgroup, 
 		     Num_Articles_Received, Num_Articles_To_Receive, Num_Killed,
 		     hour, min, sec,
@@ -979,7 +987,7 @@ static int write_xover_line (FILE *fp, Slrn_XOver_Type *xov) /*{{{*/
 	   && (EOF == fprintf (fp, "\tXref: %s", xov->xref)))
        || (EOF == fputc ('\n', fp)))
      {
-	log_error ("Error writing to overview database: %s:%d.", Current_Newsgroup, xov->id);
+	log_error (_("Error writing to overview database: %s:%d."), Current_Newsgroup, xov->id);
 	return -1;
      }
 #else
@@ -1020,7 +1028,7 @@ static int write_head_and_body (Active_Group_Type *g, int n, /*{{{*/
 #endif
    if (fp == NULL)
      {
-	log_error ("Unable to open %s for writing.", file);
+	log_error (_("Unable to open %s for writing."), file);
 	return -1;
      }
    
@@ -1028,7 +1036,7 @@ static int write_head_and_body (Active_Group_Type *g, int n, /*{{{*/
        || (EOF == fputc ('\n', fp))
        || (EOF == fputs (body, fp)))
      {
-	log_error ("Error writing to %s.", file);
+	log_error (_("Error writing to %s."), file);
 	fclose (fp);
 	slrn_delete_file (file);
 	return -1;
@@ -1036,7 +1044,7 @@ static int write_head_and_body (Active_Group_Type *g, int n, /*{{{*/
    
    if (-1 == slrn_fclose (fp))
      {
-	log_error ("Error writing to %s.", file);
+	log_error (_("Error writing to %s."), file);
 	slrn_delete_file (file);
 	return -1;
      }
@@ -1161,7 +1169,7 @@ static int fetch_head (NNTP_Type *s, int n, char **headers, Slrn_XOver_Type *xov
    (void) is_msgid_cached (h.msgid, Current_Newsgroup, (unsigned int) n, 1);
 #endif
 
-   score = slrn_score_header (&h, Current_Newsgroup);
+   score = slrn_score_header (&h, Current_Newsgroup, NULL);
    if (score < Kill_Score)
      {
 	Num_Killed++;
@@ -1185,7 +1193,7 @@ static int get_heads (NNTP_Type *s, int *numbers, char **heads, /*{{{*/
 		      Slrn_XOver_Type *xovs, unsigned int num)
 {
    unsigned int i;
-   char buf[256];
+   char buf[20*SLRN_MAX_QUEUED];
    char *b;
    char *crlf;
    
@@ -1195,8 +1203,10 @@ static int get_heads (NNTP_Type *s, int *numbers, char **heads, /*{{{*/
    /* Final crlf added by nntp_start_server_cmd. */
    for (i = 0; i < num; i++)
      {
-	slrn_snprintf (b, sizeof (buf) - (size_t) (b - buf),
-		       "%shead %d", crlf, numbers[i]);
+	unsigned int len = sizeof (buf) - (size_t) (b - buf);
+	if (len < 20)
+	  slrn_exit_error (_("Internal error: Buffer in get_heads not large enough!"));
+	slrn_snprintf (b, len, "%shead %d", crlf, numbers[i]);
 	crlf = "\r\n";
 	b += strlen (b);
 	
@@ -1232,7 +1242,7 @@ static FILE *open_xover_file (Active_Group_Type *g, char *mode) /*{{{*/
      {
 	fp = fopen (ov_file, mode);
 	if (fp == NULL)
-	  log_error ("Unable to open overview file %s.\n", ov_file);
+	  log_error (_("Unable to open overview file %s.\n"), ov_file);
      }
    
    return fp;
@@ -1280,7 +1290,7 @@ static int get_articles (NNTP_Type *s, Active_Group_Type *g, int *numbers, unsig
 	  {
 	     if (-1 == slrn_fclose (fp))
 	       {
-		  log_error ("Error closing overview file for %s.", g->name);
+		  log_error (_("Error closing overview file for %s."), g->name);
 		  ret = -1;
 	       }
 	  }
@@ -1316,14 +1326,14 @@ static int get_group_articles (NNTP_Type *s, Active_Group_Type *g,
    if (((server_min > server_max) || (server_max < 0))
        || (((unsigned int)server_max <= gmax) && (gmin <= gmax)))
      {
-	log_message ("%s: no new articles available.", g->name);
+	log_message (_("%s: no new articles available."), g->name);
 	return 0;
      }
 
    Num_Duplicates = 0;
    numbers = list_server_numbers (s, g, server_min, server_max, &num_numbers);
    if (Num_Duplicates)
-     log_message ("%u duplicates removed leaving %u/%u.", 
+     log_message (_("%u duplicates removed leaving %u/%u."), 
 		  Num_Duplicates, num_numbers, num_numbers + Num_Duplicates);
 	
    if (numbers == NULL) return -1;
@@ -1335,19 +1345,19 @@ static int get_group_articles (NNTP_Type *s, Active_Group_Type *g,
    if (i == num_numbers)
      {
 	g->max = g->server_max;
-	log_message ("%s: No new articles available.", g->name);
+	log_message (_("%s: No new articles available."), g->name);
 	slrn_free ((char *) numbers);
 	return 0;
      }
    
-   log_message ("%s: %u articles available.", g->name, num_numbers - i);
+   log_message (_("%s: %u articles available."), g->name, num_numbers - i);
    
    /* Hmmm...  How shall g->max_to_get be defined?  Here I assume that it 
     * means to attempt to retrieve the last max_to_get articles.
     */
    if ((g->max_to_get != 0) && (g->max_to_get + i < num_numbers))
      {
-	log_message ("%s: Only retrieving last %u articles.", g->name, g->max_to_get);
+	log_message (_("%s: Only retrieving last %u articles."), g->name, g->max_to_get);
 	i = num_numbers - g->max_to_get;
      }
 	
@@ -1396,12 +1406,12 @@ static int pull_news (NNTP_Type *s) /*{{{*/
      {
 	int min, max;
 	
-	log_message ("Fetching articles for %s.", g->name);
+	log_message (_("Fetching articles for %s."), g->name);
 	
 	status = nntp_select_group (s, g->name, &min, &max);
 	if (status != OK_GROUP)
 	  {
-	     log_error ("Error selecting group %s.  Code = %d: %s", g->name, 
+	     log_error (_("Error selecting group %s.  Code = %d: %s"), g->name, 
 			status, nntp_map_code_to_string (status));
 
 	     if (status == -1)
@@ -1434,19 +1444,19 @@ static int post_file (NNTP_Type *s, char *file) /*{{{*/
    int status;
    char buf[8 * 1024];
    
-   log_message ("Attempting to post %s...", file);
+   log_message (_("Attempting to post %s..."), file);
 
    fp = fopen (file, "r");
    if (fp == NULL)
      {
-	log_error ("Unable to open file %s for posting.", file);
+	log_error (_("Unable to open file %s for posting."), file);
 	return -1;
      }
    
    status = nntp_post_cmd (s);
    if (status != CONT_POST)
      {
-	log_error ("Server failed post cmd.  Code = %d: %s", 
+	log_error (_("Server failed post cmd.  Code = %d: %s"), 
 		   status, nntp_map_code_to_string (status));
 	fclose (fp);
 	return -1;
@@ -1468,7 +1478,7 @@ static int post_file (NNTP_Type *s, char *file) /*{{{*/
 	if ((-1 == nntp_fputs_server (s, buf))
 	    || (-1 == nntp_fputs_server (s, "\r\n")))
 	  {
-	     log_error ("Write to server failed while posting %s.", file);
+	     log_error (_("Write to server failed while posting %s."), file);
 	     fclose (fp);
 	     return -1;
 	  }
@@ -1479,7 +1489,7 @@ static int post_file (NNTP_Type *s, char *file) /*{{{*/
    status = nntp_end_post (s);
    if (status == -1)
      {
-	log_error ("Write to server failed while posting %s.", file);
+	log_error (_("Write to server failed while posting %s."), file);
 	return -1;
      }
    
@@ -1488,22 +1498,22 @@ static int post_file (NNTP_Type *s, char *file) /*{{{*/
 	char *name;
 	char bad_file [SLRN_MAX_PATH_LEN + 1];
 	
-	log_error ("Article %s rejected. status = %d: %s.", file, status, s->rspbuf);
+	log_error (_("Article %s rejected. status = %d: %s."), file, status, s->rspbuf);
 	
 	name = slrn_basename (file);
 	if (-1 == slrn_dircat (Outgoing_Bad_Dir, name,
 			       bad_file, sizeof (bad_file)))
 	  return -1;
 	
-	log_error ("Saving article in %s...", Outgoing_Bad_Dir);
+	log_error (_("Saving article in %s..."), Outgoing_Bad_Dir);
 	if (-1 == rename (file, bad_file))
-	  log_error ("Failed to rename %s to %s.", file, bad_file);
+	  log_error (_("Failed to rename %s to %s."), file, bad_file);
 	
 	return -1;
      }
    
    if (-1 == slrn_delete_file (file))
-     log_error ("Unable to delete %s after posting.", file);
+     log_error (_("Unable to delete %s after posting."), file);
    
    return 0;
 }
@@ -1512,15 +1522,15 @@ static int post_file (NNTP_Type *s, char *file) /*{{{*/
 
 static int make_outgoing_dir (char *dir) /*{{{*/
 {
-   log_error ("%s directory does not exist.  Creating it...", dir);
+   log_error (_("%s directory does not exist.  Creating it..."), dir);
    if (-1 == MKDIR (dir, 0700))
      {
-	log_error ("Unable to create %s.", dir);
+	log_error (_("Unable to create %s."), dir);
 	return -1;
      }
 	
    if (-1 == chmod (dir, OUTGOING_DIR_MODE))
-     log_error ("chmod 0%o failed on %s.", OUTGOING_DIR_MODE, dir);
+     log_error (_("chmod 0%o failed on %s."), OUTGOING_DIR_MODE, dir);
 
    return 0;
 }
@@ -1544,7 +1554,7 @@ static int post_outgoing (NNTP_Type *s) /*{{{*/
 	
    if (s->can_post == 0)
      {
-	log_error ("Server does not permit posting at this time.");
+	log_error (_("Server does not permit posting at this time."));
 	return 0;
      }
 
@@ -1565,10 +1575,10 @@ static int post_outgoing (NNTP_Type *s) /*{{{*/
 	  continue;
 	
 	if (-1 == post_file (s, file))
-	  log_error ("Posting of %s failed.", file);
+	  log_error (_("Posting of %s failed."), file);
 	else 
 	  {
-	     log_message ("%s posted.", file);
+	     log_message (_("%s posted."), file);
 	     n++;
 	  }
      }
@@ -1590,7 +1600,7 @@ static int write_active (void) /*{{{*/
    fp = fopen (file, "w");
    if (fp == NULL)
      {
-	log_error ("Unable to create tmp active file (%s).", file);
+	log_error (_("Unable to create tmp active file (%s)."), file);
 	return -1;
      }
    
@@ -1613,14 +1623,14 @@ static int write_active (void) /*{{{*/
 	(void) slrn_delete_file (Active_File);
 	if (-1 == rename (file, Active_File))
 	  {
-	     log_error ("Failed to rename %s to %s.", file, Active_File);
+	     log_error (_("Failed to rename %s to %s."), file, Active_File);
 	     return -1;
 	  }
 	return 0;
      }
    
    write_error:
-   log_error ("Write failed to tmp active file (%s).", file);
+   log_error (_("Write failed to tmp active file (%s)."), file);
    (void) slrn_delete_file (file);
    return -1;
 }
@@ -1632,7 +1642,7 @@ static time_t Actual_Start_Time;
 
 static void connection_lost_hook (NNTP_Type *s)
 {
-   log_error ("Connection to %s lost. Performing shutdown.", s->host);
+   log_error (_("Connection to %s lost. Performing shutdown."), s->host);
    (void) write_active ();
    Exit_Code = SLRN_EXIT_CONNECTION_LOST;
    slrn_exit_error (NULL);
@@ -1644,7 +1654,7 @@ static int open_servers (char *host) /*{{{*/
 #if SLRN_USE_SLTCP
    if (-1 == sltcp_open_sltcp ())
      {
-	fprintf (stderr, "Error opening sltcp interface.\n");
+	fprintf (stderr, _("Error opening sltcp interface.\n"));
 	return -1;
      }
 #endif
@@ -1685,7 +1695,7 @@ static void print_stats (unsigned long bytes_in, unsigned long bytes_out) /*{{{*
    else
      time (&done);
 
-   log_message ("A total of %lu bytes received, %lu bytes sent in %ld seconds.",
+   log_message (_("A total of %lu bytes received, %lu bytes sent in %ld seconds."),
 		bytes_in, bytes_out, (long) done - (long) Actual_Start_Time);
 }
 
@@ -1731,7 +1741,7 @@ static void open_log_files (char *logfile, char *kill_logfile) /*{{{*/
    if (MLog_Fp == NULL)
      {
 	MLog_Fp = stdout;
-	log_error ("Unable to open %s for logging.", file);
+	log_error (_("Unable to open %s for logging."), file);
 	return;
      }
    
@@ -1743,7 +1753,7 @@ static void open_log_files (char *logfile, char *kill_logfile) /*{{{*/
    
    KLog_Fp = fopen (file, "a");
    if (KLog_Fp == NULL)
-     log_error ("Unable to open %s for logging.", file);
+     log_error (_("Unable to open %s for logging."), file);
 }
 
 /*}}}*/
@@ -1767,7 +1777,7 @@ static void close_log_files (void) /*{{{*/
 
 static void usage (char *pgm) /*{{{*/
 {
-   log_error ("%s usage: %s [options]\n\
+   log_error (_("%s usage: %s [options]\n\
  Options:\n\
   -d SPOOLDIR          Spool directory to use.\n\
   -h HOSTNAME          Hostname of NNTP server to connect to.\n\
@@ -1781,7 +1791,7 @@ static void usage (char *pgm) /*{{{*/
   --no-post            Do not post news.\n\
   --post-only          Post news, but do not pull new news.\n\
   --version            Show the version number.\n\
-",
+"),
 	      pgm, pgm);
    close_log_files ();
    exit (SLRN_EXIT_BAD_USAGE);
@@ -1791,7 +1801,7 @@ static void usage (char *pgm) /*{{{*/
 
 static void show_version (char *pgm)
 {
-   log_error ("%s version: %s\n", pgm, Slrnpull_Version);
+   log_error (_("%s version: %s\n"), pgm, Slrnpull_Version);
    close_log_files ();
    exit (0);
 }
@@ -1828,6 +1838,14 @@ int main (int argc, char **argv) /*{{{*/
    char *logfile;
    char *kill_logfile = NULL;
 
+#if defined(HAVE_SETLOCALE) && defined(LC_ALL)
+   (void) setlocale(LC_ALL, "");
+#endif
+#ifdef ENABLE_NLS
+   bindtextdomain(PACKAGE,LOCALEDIR);
+   textdomain(PACKAGE);
+#endif
+   
    pgm = argv[0];
    argv++; argc--;
    
@@ -1889,7 +1907,7 @@ int main (int argc, char **argv) /*{{{*/
 	  {
 	     if (NULL == (Slrn_Debug_Fp = fopen (*argv, "w")))
 	       {
-		  fprintf (stderr, "Unable to create debug file %s\n", *argv);
+		  fprintf (stderr, _("Unable to create debug file %s\n"), *argv);
 		  exit (1);
 	       }
              else
@@ -1907,16 +1925,16 @@ int main (int argc, char **argv) /*{{{*/
    
    if (SlrnPull_Dir == NULL)
      {
-	fprintf (stderr, "The slrnpull spool directory has not been defined.");
+	fprintf (stderr, _("The slrnpull spool directory has not been defined."));
 	return 1;
      }
    
    open_log_files (logfile, kill_logfile);
 
    if (expire_mode)
-     log_message ("slrnpull started in expire mode.");
+     log_message (_("slrnpull started in expire mode."));
    else
-     log_message ("slrnpull started.");
+     log_message (_("slrnpull started."));
    
    SLang_init_case_tables ();
    
@@ -1948,7 +1966,7 @@ int main (int argc, char **argv) /*{{{*/
    if (-1 == open_servers (host))
      {
 	Exit_Code = SLRN_EXIT_CONNECTION_FAILED;
-	slrn_exit_error ("Unable to initialize server.");
+	slrn_exit_error (_("Unable to initialize server."));
      }
 
    SLTCP_Interrupt_Hook = handle_interrupts;
@@ -1996,11 +2014,11 @@ static int get_new_groups (NNTP_Type *s)
    int num;
    char *p;
    
-   log_message ("Checking for new groups.");
+   log_message (_("Checking for new groups."));
    
    if (NULL == (fp_ng = fopen (New_Groups_File, "a")))
      {
-	log_message ("Unable to open new groups file %s.\n", New_Groups_File);
+	log_message (_("Unable to open new groups file %s.\n"), New_Groups_File);
 	return -1;
      }
    
@@ -2071,11 +2089,11 @@ static int get_new_groups (NNTP_Type *s)
 	     return 0;
 	     
 	   case ERR_COMMAND:
-	     log_message ("Server does not implement NEWGROUPS command.");
+	     log_message (_("Server does not implement NEWGROUPS command."));
 	     return 0;
 	     
 	   default:
-	     slrn_message ("Server failed to return proper response to NEWGROUPS:\n%s\n",
+	     slrn_message (_("Server failed to return proper response to NEWGROUPS:\n%s\n"),
 			   s->rspbuf);
 	     
 	     goto parse_error_label;
@@ -2087,7 +2105,7 @@ static int get_new_groups (NNTP_Type *s)
 	     if ((EOF == fputs (line, fp_ng))
 		 || (EOF == fputc ('\n', fp_ng)))
 	       {
-		  log_error ("Write to %s failed.", New_Groups_File);
+		  log_error (_("Write to %s failed."), New_Groups_File);
 		  (void) fclose (fp_ng);
 		  return -1;
 	       }
@@ -2097,14 +2115,14 @@ static int get_new_groups (NNTP_Type *s)
 	if (-1 == slrn_fclose (fp_ng))
 	  return -1;
 	
-	log_message ("%d new groups found.", num);
+	log_message (_("%d new groups found."), num);
 	
 	
 	parse_error_label:
 
 	if (parse_error)
 	  {
-	     log_message ("%s appears corrupt, expected to see see: NEWGROUPS yymmdd hhmmss GMT, I will patch the file up for you.",
+	     log_message (_("%s appears corrupt, expected to see: NEWGROUPS yymmdd hhmmss GMT, I will patch the file up for you."),
 			  New_Groups_File);
 	  }
      }
@@ -2183,10 +2201,10 @@ static int handle_interrupts (void) /*{{{*/
 {
    if (Terminate_Slrn_Pull_Requested)
      {
-	log_error ("Performing shutdown.");
+	log_error (_("Performing shutdown."));
 	write_active ();
 	Exit_Code = SLRN_EXIT_SIGNALED;
-	slrn_exit_error ("Slrn exiting on signal.");
+	slrn_exit_error (_("Slrnpull exiting on signal."));
      }
    
    return 0;
@@ -2305,7 +2323,7 @@ static int create_overview_for_dir (Active_Group_Type *g, unsigned int *nums, un
    unsigned i;
    void (*qsort_fun) (char *, unsigned int, int, int (*)(unsigned int *, unsigned int *));
 
-   log_message ("Creating Overview file for %s...", g->name);
+   log_message (_("Creating Overview file for %s..."), g->name);
    
    xov_fp = open_xover_file (g, "w");
    
@@ -2351,7 +2369,7 @@ static int create_overview_for_dir (Active_Group_Type *g, unsigned int *nums, un
 	
 	if (-1 == stat (file, &st))
 	  {
-	     log_error ("Unable to stat %s.", file);
+	     log_error (_("Unable to stat %s."), file);
 	     continue;
 	  }
 	
@@ -2408,7 +2426,7 @@ static int expire_group (Active_Group_Type *g) /*{{{*/
    dp = slrn_open_dir (dir);
    if (dp == NULL)
      {
-	log_error ("opendir %s failed.", dir);
+	log_error (_("opendir %s failed."), dir);
 	return -1;
      }
    
@@ -2435,7 +2453,7 @@ static int expire_group (Active_Group_Type *g) /*{{{*/
 	     ok_names = (unsigned int *) slrn_realloc ((char *) ok_names, max_num_ok_names * sizeof (unsigned int), 1);
 	     if (ok_names == NULL)
 	       {
-		  log_error ("malloc error. Unable to expire group %s.", g->name);
+		  log_error (_("malloc error. Unable to expire group %s."), g->name);
 		  slrn_close_dir (dp);
 		  return -1;
 	       }
@@ -2481,7 +2499,7 @@ static int expire_group (Active_Group_Type *g) /*{{{*/
 	
 	if (-1 == stat (file, &st))
 	  {
-	     log_error ("Unable to stat %s.", file);
+	     log_error (_("Unable to stat %s."), file);
 	     continue;
 	  }
 	
@@ -2498,12 +2516,12 @@ static int expire_group (Active_Group_Type *g) /*{{{*/
 	  }
 
 	if (-1 == slrn_delete_file (file))
-	  log_error ("Unable to expire %s.", file);
+	  log_error (_("Unable to expire %s."), file);
 	else
 	  num_expired++;
      }
    
-   if (num_expired) log_message ("%u articles expired in %s.", num_expired, g->name);
+   if (num_expired) log_message (_("%u articles expired in %s."), num_expired, g->name);
    
    (void) create_overview_for_dir (g, ok_names, new_num_ok_names);
    
@@ -2526,7 +2544,7 @@ static int do_expire (void) /*{{{*/
      {
 	if (Terminate_Slrn_Pull_Requested)
 	  {
-	     log_error ("Termination requested.  Shutting down.");
+	     log_error (_("Termination requested.  Shutting down."));
 	     (void) write_active ();
 	     Exit_Code = SLRN_EXIT_SIGNALED;
 	     slrn_exit_error (NULL);
@@ -2574,7 +2592,7 @@ static int read_authinfo (void)
    if ((NULL == fgets (name, sizeof (name), fp))
        || (NULL == fgets (pass, sizeof (pass), fp)))
      {
-	log_error ("Error reading name and password from %s.");
+	log_error (_("Error reading name and password from %s."), file);
 	fclose (fp);
 	
 	return -1;
@@ -2587,7 +2605,7 @@ static int read_authinfo (void)
    
    if (0 == strlen (name))
      {
-	log_error ("Empty user name.  Please check syntax of the authinfo file.");
+	log_error (_("Empty user name.  Please check syntax of the authinfo file."));
 	return -1;
      }
    
@@ -2595,12 +2613,7 @@ static int read_authinfo (void)
    Auth_Password = pass;
 
    NNTP_Authorization_Hook = get_authorization;
+   Slrn_Force_Authentication = 1;
    
    return 0;
 }
-
-	
-     
-     
-   
-   
