@@ -199,7 +199,7 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
 		  continue;
 	       }
 #endif
-	     fprintf(stderr, "%s: Unknown host.\n", host);
+	     fprintf(stderr, _("%s: Unknown host.\n"), host);
 	     return -1;
 	  }
 	
@@ -245,7 +245,7 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
 
 	this_host = (char *) inet_ntoa (s_in.sin_addr);
 	
-	if (TCP_Verbose_Reporting) fprintf (stderr, "trying %s\n", this_host);
+	if (TCP_Verbose_Reporting) fprintf (stderr, _("trying %s\n"), this_host);
 	
 	not_connected = connect (s, (struct sockaddr *)&s_in, sizeof (s_in));
 	
@@ -258,7 +258,7 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
 		    continue;
 	       }
 #endif
-	     fprintf (stderr, "connection to %s, port %d:", 
+	     fprintf (stderr, _("connection to %s, port %d:"), 
 		      (char *) this_host, port);
 	     perror ("");
 	  }
@@ -267,7 +267,7 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
    
    if (not_connected) 
      {
-	fprintf(stderr, "Unable to make connection. Giving up.\n");
+	fprintf(stderr, _("Unable to make connection. Giving up.\n"));
 	(void) SLTCP_CLOSE (s);
 	return -1;
      }
@@ -404,7 +404,7 @@ static int init_ssl_random (void)
    if (RAND_status ())
      return 0;
    
-   fprintf (stderr, "Unable to generate enough entropy\n");
+   fprintf (stderr, _("Unable to generate enough entropy\n"));
    return -1;
 }
 
@@ -423,7 +423,7 @@ static SSL *alloc_ssl (void)
 	if (c == NULL)
 	  {
 	     dump_ssl_error_0 ();
-	     fprintf (stderr, "SSL_CTX_new failed.\n");
+	     fprintf (stderr, _("SSL_CTX_new failed.\n"));
 	     return NULL;
 	  }
 	This_SSL_Ctx = c;
@@ -438,7 +438,7 @@ static SSL *alloc_ssl (void)
      return ssl;
    
    dump_ssl_error_0 ();
-   fprintf (stderr, "SSL_new failed\n");
+   fprintf (stderr, _("SSL_new failed\n"));
    return NULL;
 }
 
@@ -455,17 +455,17 @@ static void dump_ssl_error (SSL *ssl, char *msg, int status)
       case SSL_ERROR_NONE:
 	break;
       case SSL_ERROR_ZERO_RETURN:
-	fprintf (stderr, "Unexpected error: SSL connection closed\n");
+	fprintf (stderr, _("Unexpected error: SSL connection closed\n"));
 	break;
       case SSL_ERROR_SYSCALL:
-	fprintf (stderr, "System call failed: errno = %d\n", errno);
+	fprintf (stderr, _("System call failed: errno = %d\n"), errno);
 	break;
       case SSL_ERROR_SSL:
-	fprintf (stderr, "Possible protocol error\n");
+	fprintf (stderr, _("Possible protocol error\n"));
 	break;
      }
    dump_ssl_error_0 ();
-   if (msg != NULL) fprintf (stderr, "%s failed\n", msg);
+   if (msg != NULL) fprintf (stderr, _("%s failed\n"), msg);
 }
 #endif				       /* SLTCP_HAS_SSL_SUPPORT */
 
@@ -484,7 +484,7 @@ SLTCP_Type *sltcp_open_connection (char *host, int port, int use_ssl) /*{{{*/
 	if (NULL == (ssl = alloc_ssl ()))
 	  return NULL;
 #else
-	fprintf (stderr, "\n\n*** This program does not support SSL\n");
+	fprintf (stderr, _("\n\n*** This program does not support SSL\n"));
 	return NULL;
 #endif
      }
@@ -492,7 +492,7 @@ SLTCP_Type *sltcp_open_connection (char *host, int port, int use_ssl) /*{{{*/
    tcp = (SLTCP_Type *) SLMALLOC (sizeof (SLTCP_Type));
    if (tcp == NULL)
      {
-	fprintf (stderr, "Memory Allocation Failure.\n");
+	fprintf (stderr, _("Memory Allocation Failure.\n"));
 #if SLTCP_HAS_SSL_SUPPORT
 	dealloc_ssl (ssl);
 #endif
@@ -524,7 +524,7 @@ SLTCP_Type *sltcp_open_connection (char *host, int port, int use_ssl) /*{{{*/
    if (1 != SSL_set_fd (ssl, fd))
      {
 	/* Yuk.  SSL_set_fd returns 0 upon failure */
-	fprintf (stderr, "SSL_set_fd failed\n");
+	fprintf (stderr, _("SSL_set_fd failed\n"));
 	sltcp_close (tcp);
 	return NULL;
      }
@@ -533,7 +533,7 @@ SLTCP_Type *sltcp_open_connection (char *host, int port, int use_ssl) /*{{{*/
    if (status == 1)
      return tcp;
 
-   dump_ssl_error (ssl, "SSL_connect", status);
+   dump_ssl_error (ssl, _("SSL_connect"), status);
    sltcp_close (tcp);
    return NULL;
 #else
