@@ -3,10 +3,16 @@
 #include "config.h"
 #include "slrnfeat.h"
 
-#define DEBUG_SPOOL 0
-#define DEBUG_SPOOL_FILENAME "SLRNDEBUG"
+/* define this if you want to trace command-execution in spool.c.
+ * Normally, you only get errors in the debug-file. If you set this,
+ * please keep an eye on your disk-space.
+ */
+#define DEBUG_SPOOL_TRACE 1 
 
 #include <stdio.h>
+#ifndef SEEK_SET
+# define SEEK_SET	0
+#endif
 
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
@@ -26,6 +32,7 @@
 #include <time.h>
 
 #include "misc.h"
+#include "util.h"
 #include "slrn.h"
 #include "slrndir.h"
 #include "snprintf.h"
@@ -45,22 +52,26 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <assert.h>
+#include <stdarg.h>
 
 #include <limits.h>
 
+extern int Slrn_Prefer_Head;
+
 static int spool_put_server_cmd (char *, char *, unsigned int );
 static int spool_select_group (char *, int *, int *);
+static int spool_refresh_groups (Slrn_Group_Range_Type *, int);
 static void spool_close_server (void);
 static int spool_has_cmd (char *);
 static int spool_initialize_server (void);
 static int spool_read_line (char *, unsigned int );
 static int spool_xpat_cmd (char *, int, int, char *);
-static int spool_xgtitle_cmd (char *);
 static int spool_select_article (int, char *);
 static int spool_one_xhdr_command (char *, int, char *, unsigned int);
 static int spool_xhdr_command (char *, int, int);
+static int spool_list (char *);
 static int spool_list_newsgroups (void);
-static int spool_list_active (void);
+static int spool_list_active (char *);
 static int spool_send_authinfo (void);
 static int spool_read_xover (char *, unsigned int);
 static int spool_read_xpat (char *, unsigned int);
@@ -72,6 +83,7 @@ static Slrn_Server_Obj_Type Spool_Server_Obj;
 
 /* some state that the NNTP server would take care of if we were using one */
 static FILE *Spool_fh_local=NULL;
+static SLRegexp_Type *Spool_Output_Regexp=NULL;
 static char *Spool_Group=NULL;
 static char *Spool_Group_Name;
 
@@ -94,36 +106,41 @@ static int Spool_fakeactive_newsgroups=0;
 
 static unsigned int bytes_read=0;
 
-#if DEBUG_SPOOL
-# include <stdarg.h>
-static void spool_debug (char *fmt, ...)
+/* Note: Please leave debugging output untranslated.
+ *       It cannot be interpreted by the end user anyways. */
+static void debug_output (char *file, int line, char *fmt, ...)
 {
-   static FILE *fp = NULL;
    va_list ap;
-
-   if (fp == NULL)
+   if (Slrn_Debug_Fp != NULL)
      {
-	fp = fopen (DEBUG_SPOOL_FILENAME, "w");
-	if (fp == NULL)
+        if ((file != NULL) && (line != -1))
+          fprintf (Slrn_Debug_Fp, "%s:%d: ", file, line);
+        else
+#ifdef DEBUG_SPOOL_TRACE
+          fprintf (Slrn_Debug_Fp, "%lu: ", (unsigned long) clock ());
+#else
 	  return;
+#endif
+	
+        if (fmt == NULL)
+          {
+             fputs ("(NULL)", Slrn_Debug_Fp);
+          }
+        else
+          {
+	     va_start(ap, fmt);
+	     vfprintf(Slrn_Debug_Fp, fmt, ap);
+	     va_end (ap);
+          }
+	
+        putc ('\n', Slrn_Debug_Fp);
      }
-
-   fprintf (fp, "%lu: ", (unsigned long) clock ());
-
-   if (fmt == NULL)
-     {
-	fputs ("(NULL)", fp);
-     }
-   else
-     {
-	va_start(ap, fmt);
-	vfprintf(fp, fmt, ap);
-	va_end (ap);
-     }
-
-   putc ('\n', fp);
-   fflush (fp);
 }
+#ifndef __FILE__
+# define __FILE__ "spool.c"
+#endif
+#ifndef __LINE__
+# define __LINE__ 0
 #endif
 
 /* close any current file (unless it's the overview file) and NULL the FILE* */
@@ -133,6 +150,7 @@ static int spool_fclose_local (void)
 
    Spool_fhead=0;
    Spool_fFakingActive=0;
+   Spool_Output_Regexp = NULL;
 
    if (Spool_fh_local != NULL)
      {
@@ -159,6 +177,35 @@ static FILE *spool_open_nov_file (void)
    return fp;
 }
 
+static int overview_file_seek (long fp, int cur, int dest)
+{
+   int ch;
+   
+   while (cur < dest)
+     {
+        if (-1 == (fp = ftell(Spool_fh_local)))
+          {
+             debug_output (__FILE__, __LINE__, "ftell returned -1; errno %d (%s).", errno, strerror(errno));
+             return -1;
+          }
+	if (1 != fscanf (Spool_fh_local, "%d", &cur))
+	  {
+	     spool_fclose_local ();
+	     return -1;
+	  }
+	
+	while (((ch = getc (Spool_fh_local)) != '\n') && (ch != EOF))
+	  ; /* do nothing */
+     }
+   if (-1 == fseek(Spool_fh_local, fp, SEEK_SET)) /* reset to start of line */
+     {
+        debug_output (__FILE__, __LINE__, "fseek returned -1; errno %d (%s).", errno, strerror(errno));
+        return ERR_FAULT;
+     }
+   
+   return 0;
+}
+
 static int Spool_XOver_Next;
 static int Spool_XOver_Max;
 static int Spool_XOver_Min;
@@ -168,10 +215,6 @@ static int spool_nntp_xover (int min, int max)
    int i, ch;
    long fp;
 
-#if DEBUG_SPOOL
-   spool_debug ("spool_nntp_xover(%d,%d)", min, max);
-#endif
-   
    Spool_Doing_XOver = 0;
 
    spool_fclose_local ();
@@ -191,7 +234,11 @@ static int spool_nntp_xover (int min, int max)
 	/* find first record in range in overview file */
 	/* first look at the current position and see where we are */
 	/* this is worth trying as slrn will often read a series of ranges */
-	fp = ftell (Spool_fh_local);
+	if (-1 == (fp = ftell (Spool_fh_local)))
+          {
+             debug_output (__FILE__, __LINE__, "ftell returned -1; errno %d (%s).", errno, strerror(errno));
+             return ERR_FAULT;
+          }
 
 	if ((1 != fscanf (Spool_fh_local,"%d", &i))
 	    || (i > min))
@@ -215,26 +262,10 @@ static int spool_nntp_xover (int min, int max)
 	       }
 	  }
 
-#if DEBUG_SPOOL
-	spool_debug ("Starting with i=%d",i);
-#endif
-
-	while (i < min)
-	  {
-	     fp = ftell( Spool_fh_local );
-	     if (1 != fscanf(Spool_fh_local,"%d", &i))
-	       return -1;
-
-	     while (((ch = getc (Spool_fh_local)) != '\n')
-		    && (ch != EOF))
-	       ; /* do nothing */
-	  }
-#ifndef SEEK_SET
-# define SEEK_SET	0
-#endif
-	fseek (Spool_fh_local, fp, SEEK_SET); /* reset to start of line */
+	if (-1 == overview_file_seek (fp, i, min))
+	  return -1;
      }
-
+   
    Spool_XOver_Next = Spool_XOver_Min = min;
    Spool_XOver_Max = max;
    Spool_Doing_XOver = 1;
@@ -246,10 +277,6 @@ static int spool_read_xover (char *the_buf, unsigned int len)
 {
    long pos;
 
-#if DEBUG_SPOOL
-   spool_debug ("spool_read_xover");
-#endif
-
    if (Spool_Doing_XOver == 0)
      return 0;
 
@@ -263,7 +290,11 @@ static int spool_read_xover (char *the_buf, unsigned int len)
      {
 	unsigned int buflen;
 
-	pos = ftell (Spool_fh_nov);
+	if (-1 == (pos = ftell (Spool_fh_nov)))
+          {
+             debug_output (__FILE__, __LINE__, "ftell returned -1; errno %d (%s).", errno, strerror(errno));
+             return -1;
+          }	
    
 	if (NULL == fgets (the_buf, len, Spool_fh_nov))
 	  {
@@ -279,7 +310,11 @@ static int spool_read_xover (char *the_buf, unsigned int len)
 	Spool_XOver_Next = atoi (the_buf);
 	if (Spool_XOver_Next > Spool_XOver_Max)
 	  {
-	     fseek (Spool_fh_nov, pos, SEEK_SET);
+             if (-1 == fseek(Spool_fh_nov, pos, SEEK_SET))
+               {
+                  debug_output (__FILE__, __LINE__, "fseek returned -1; errno %d (%s).", errno, strerror(errno));
+                  return -1;
+               }	     
 	     Spool_Doing_XOver = 0;
 	     return 0;
 	  }
@@ -289,21 +324,11 @@ static int spool_read_xover (char *the_buf, unsigned int len)
 	
 	/* check that the article file actually exists */
 	/* if not, this nov entry is defunct, so ignore it */
-#if DEBUG_SPOOL
-	spool_debug ("Nov entry %ld", Spool_XOver_Next);
-#endif
-	
 	if (0 == spool_article_num_exists (Spool_XOver_Next))
 	  {
-#if DEBUG_SPOOL
-	     spool_debug (" - OK");
-#endif
 	     break;
 	  }
-	
-#if DEBUG_SPOOL
-	spool_debug (" skipped!");
-#endif
+	debug_output (NULL, -1, "Nov entry %ld skipped!", Spool_XOver_Next);
      }
 
    Spool_XOver_Next++;
@@ -316,9 +341,7 @@ static int spool_find_artnum_from_msgid (char *msgid)
    char *p;
    int n;
    
-#if DEBUG_SPOOL
-   spool_debug ("spool_find_artnum_from_msgid('%s')", msgid);
-#endif
+   debug_output (NULL, -1, "spool_find_artnum_from_msgid('%s')", msgid);
 
    if (Slrn_Server_Obj->sv_has_xover == 0)
      {
@@ -357,17 +380,13 @@ static int spool_find_artnum_from_msgid (char *msgid)
 
 	if (0 == strcmp(msgid, p))
 	  {
-#if DEBUG_SPOOL
-	     spool_debug ("spool_find_artnum_from_msgid() returns %d",atoi(buf));
-#endif
+	     debug_output (NULL, -1, "spool_find_artnum_from_msgid() returns %d",atoi(buf));
 	     Spool_Doing_XOver = 0;
 	     return atoi(buf);
 	  }
      }
    
-#if DEBUG_SPOOL
-   spool_debug ("spool_find_artnum_from_msgid() found no match");
-#endif
+   debug_output (NULL, -1, "spool_find_artnum_from_msgid() found no match");
    
    Spool_Doing_XOver = 0;
    return -1;
@@ -447,17 +466,13 @@ static int spool_nntp_next (int *id)
 	     Spool_cur_artnum = i;
 	     if (id != NULL) *id = i;
 	     
-#if DEBUG_SPOOL
-	     spool_debug ("NEXT found article %d",Spool_cur_artnum);
-#endif
+	     debug_output (NULL, -1, "NEXT found article %d",Spool_cur_artnum);
 
 	     return OK_NOTEXT; /* No text sent -- stat, next, last */
 	  }
      }
 
-#if DEBUG_SPOOL
-   spool_debug ("No NEXT -- %d > %d", Spool_cur_artnum, Spool_Max_Artnum);
-#endif
+   debug_output (NULL, -1, "No NEXT -- %d > %d", Spool_cur_artnum, Spool_Max_Artnum);
    
    return ERR_NONEXT; /* No next article in this group */
 }
@@ -495,10 +510,6 @@ static int spool_nntp_newgroups (char *line, char *buf, unsigned int len)
     * yr = this_year();
     * Y += ((yr-51)/100*100);
     */
-#if DEBUG_SPOOL
-   spool_debug ("%d read, %04d-%02d-%02d %d:%02d:%02d %s",
-		c,Y,M,D,h,m,s,fGMT?"GMT":"(local)");
-#endif
    
    if (Y < 1970
        || M < 1 || M > 12
@@ -548,9 +559,7 @@ static int spool_nntp_newgroups (char *line, char *buf, unsigned int len)
 #endif /* NOT HAVE_TIMEZONE */
      }
 
-#if DEBUG_SPOOL
-   spool_debug ("threshold = %d",threshold);
-#endif
+   debug_output (NULL, -1, "threshold for spool_nntp_newsgrups is %d",threshold);
 
    spool_fclose_local();
    Spool_fh_local = fopen (Slrn_ActiveTimes_File, "r");
@@ -564,15 +573,8 @@ static int spool_nntp_newgroups (char *line, char *buf, unsigned int len)
 	 * Hmm, looked into this and it seems that you can't use the mtime
 	 * or ctime to decide when a directory was created.  Hmmm.
 	 */
-#if DEBUG_SPOOL
-	spool_debug ("Couldn't open active.times");
-#endif
-#if 0
-	slrn_message ("Couldn't open active.times");
-	return OK_NEWGROUPS;
-#else
+	debug_output (NULL, -1, "Couldn't open active.times");
 	return ERR_FAULT; /* Program fault, command not performed */
-#endif
      }
 
    /* chunk size to step back through active.times by
@@ -582,13 +584,16 @@ static int spool_nntp_newgroups (char *line, char *buf, unsigned int len)
 #define SLRN_SPOOL_ACTIVETIMES_STEP 128
 
    /* find start of a line */
-   fseek (Spool_fh_local, 0, SEEK_END ); /* !HACK! check return value from this and all the rest... */
-   fpos = ftell(Spool_fh_local);
-#if DEBUG_SPOOL
-   spool_debug ("ftell=%ld errno=%d (%s)",fpos,
-		errno, strerror(errno));
-   errno = 0;
-#endif
+   if (-1 == fseek (Spool_fh_local, 0, SEEK_END ))
+     {
+        debug_output (NULL, -1, "fseek returned -1; errno %d (%s).", errno, strerror(errno));
+        return ERR_FAULT;
+     }
+   if (-1 == (fpos = ftell (Spool_fh_local)))
+     {
+        debug_output (NULL, -1, "ftell returned -1; errno %d (%s).", errno, strerror(errno));
+        return ERR_FAULT;
+     }
 
    found=0;
    first=1;
@@ -601,14 +606,13 @@ static int spool_nntp_newgroups (char *line, char *buf, unsigned int len)
 
 	if (fpos < (long)len1) len1=fpos; /* don't run of the start of the file */
 	fpos -= len1;
-	fseek (Spool_fh_local, fpos, SEEK_SET );
+        if (-1 == fseek (Spool_fh_local, fpos, SEEK_SET ))
+          {
+             debug_output (__FILE__, __LINE__, "fseek returned -1; errno %d (%s).", errno, strerror(errno));
+             return ERR_FAULT;
+          }
 	if (fpos == 0) break;
 
-#if DEBUG_SPOOL
-	spool_debug ("ftell=%ld errno=%d (%s)", ftell (Spool_fh_local),
-		     errno, strerror(errno));
-#endif
-
 	if (first)
 	  {
 	     /* on the first pass, we want to ignore the last byte \n at eof */
@@ -641,10 +645,11 @@ static int spool_nntp_newgroups (char *line, char *buf, unsigned int len)
 	  }
      }
 
-   fpos=ftell(Spool_fh_local);
-#if DEBUG_SPOOL
-   spool_debug ("ftell=%ld errno=%d (%s)",fpos, errno, strerror(errno));
-#endif
+   if (-1 == (fpos = ftell (Spool_fh_local)))
+     {
+        debug_output (NULL, -1, "ftell returned -1; errno %d (%s).", errno, strerror(errno));
+        return ERR_FAULT;
+     }
 
    while (NULL != fgets( buf1, sizeof(buf1), Spool_fh_local ))
      {
@@ -653,10 +658,18 @@ static int spool_nntp_newgroups (char *line, char *buf, unsigned int len)
 
 	if (atol(p) >= threshold) /* or just > ? !HACK! */
 	  {
-	     fseek( Spool_fh_local, fpos, SEEK_SET );
+             if (-1 == fseek (Spool_fh_local, fpos, SEEK_SET ))
+               {
+                  debug_output (__FILE__, __LINE__, "fseek returned -1; errno %d (%s).", errno, strerror(errno));
+                  return ERR_FAULT;
+               }
 	     break;
 	  }
-	fpos = ftell(Spool_fh_local);
+	if (-1 == (fpos = ftell(Spool_fh_local)))
+          {
+             debug_output (__FILE__, __LINE__, "ftell returned -1; errno %d (%s).", errno, strerror(errno));
+             return ERR_FAULT;
+          }
      }
 
    return OK_NEWGROUPS; /* New newsgroups follow */
@@ -680,9 +693,7 @@ static int spool_put_server_cmd (char *line, char *buf, unsigned int len)
 {
    Spool_NNTP_Map_Type *nntpmap;
 
-#if DEBUG_SPOOL
-   spool_debug ("spool_put_server_cmd('%s')", line);
-#endif
+   debug_output (NULL, -1, "spool_put_server_cmd('%s')", line);
 
    nntpmap = Spool_NNTP_Maps;
    while (nntpmap->name != NULL)
@@ -694,9 +705,7 @@ static int spool_put_server_cmd (char *line, char *buf, unsigned int len)
 	nntpmap++;
      }
 
-#if DEBUG_SPOOL
-   spool_debug ("Hmmm, didn't know about that command");
-#endif
+   debug_output (NULL, -1, "Hmmm, didn't know about that command ('%s')", line);
    return ERR_COMMAND;
 }
 
@@ -768,7 +777,6 @@ static int spool_read_minmax_file (int *min, int *max, char *group_dir)
    return status;
 }
 
-
 /* Get the lowest and highest article numbers by the simple method
  * or looking at the files in the directory.
  * Returns 0 on success, -1 on failure
@@ -782,7 +790,7 @@ static int spool_read_minmax_from_dir (int *min, int *max, char *dir)
    /* I suspect this is very unlikely to fail */
    if ((dp = slrn_open_dir (dir)) == NULL)
      {
-	slrn_error ("Unable to open directory %s", dir);
+	slrn_error (_("Unable to open directory %s"), dir);
 	return -1;
      }
 
@@ -827,10 +835,7 @@ static int spool_read_minmax_from_active( char *name, int *min, int *max )
 	       return -1;
 
 	     Spool_Max_Artnum = *max;
-# if DEBUG_SPOOL
-	     spool_debug ("from active:%s %d %d",
-			  name,*min,*max);
-# endif
+	     debug_output (NULL, -1, "from active:%s %d %d", name, *min, *max);
 	     return 0;
 	  }
 	buf[len] = 0;
@@ -858,23 +863,27 @@ static int spool_read_minmax_from_overview (char *name, int *min, int *max)
 
    (void) name;
    
-   /* !HACK! this assumes the overview file is rewound */
-   Spool_Server_Obj.sv_has_xover = ((Spool_fh_nov != NULL)
-				    && (1 == fscanf (Spool_fh_nov,"%d", min)));
+   if (Slrn_Prefer_Head == 2)
+     Spool_Server_Obj.sv_has_xover = 0;
+   else
+     /* !HACK! this assumes the overview file is rewound */
+     Spool_Server_Obj.sv_has_xover = ((Spool_fh_nov != NULL)
+				      && (1 == fscanf (Spool_fh_nov,"%d", min)));
 
    if (0 == Spool_Server_Obj.sv_has_xover)
      return -1;
 
    /* find start of last line */
-   fseek (Spool_fh_nov, 0, SEEK_END); /* !HACK! check return value from this and all the rest... */
-
-   fpos = ftell (Spool_fh_nov);
-
-#if DEBUG_SPOOL
-   spool_debug("ftell=%ld errno=%d (%s)",
-	       fpos, errno, strerror(errno));
-   errno = 0;
-#endif
+   if (-1 == fseek (Spool_fh_nov, 0, SEEK_END))
+     {
+        debug_output (__FILE__, __LINE__, "fseek returned -1; errno %d (%s).", errno, strerror(errno));
+        return -1;
+     }
+   if (-1 == (fpos = ftell (Spool_fh_nov)))
+     {
+        debug_output (__FILE__, __LINE__, "ftell returned -1; errno %d (%s).", errno, strerror(errno));
+        return -1;
+     }
 
    found=0;
    first=1;
@@ -889,14 +898,12 @@ static int spool_read_minmax_from_overview (char *name, int *min, int *max)
 	if (fpos < (long)len) len = fpos;
 
 	fpos -= len;
-	fseek(Spool_fh_nov, fpos, SEEK_SET);
-
-#if DEBUG_SPOOL
-	spool_debug("ftell=%ld errno=%d (%s)",
-		    ftell (Spool_fh_nov), errno, strerror(errno));
-	errno = 0;
-#endif
-
+        if (-1 == fseek(Spool_fh_nov, fpos, SEEK_SET))
+          {
+             debug_output (__FILE__, __LINE__, "fseek returned -1; errno %d (%s).", errno, strerror(errno));
+             return -1;
+          }
+	
 	if (first)
 	  {
 	     /* on the first pass, we want to ignore the last byte \n at eof */
@@ -914,21 +921,21 @@ static int spool_read_minmax_from_overview (char *name, int *min, int *max)
 	  }
      }
 
-   fseek (Spool_fh_nov, fpos + found, SEEK_SET);
+   if (-1 == fseek(Spool_fh_nov, fpos + found, SEEK_SET))
+     {
+        debug_output (__FILE__, __LINE__, "fseek returned -1; errno %d (%s).", errno, strerror(errno));
+        return -1;
+     }
 
-#if DEBUG_SPOOL
-   spool_debug("ftell=%ld errno=%d (%s)",
-	       ftell (Spool_fh_nov), errno, strerror(errno));
-   errno = 0;
-#endif
-
-   fscanf (Spool_fh_nov, "%d", max);
+   if (1 != fscanf (Spool_fh_nov, "%d", max))
+     {
+        debug_output (__FILE__, __LINE__, "Hmmm, unable to understand overview file - no integer found?");
+        return -1;
+     }
 
    rewind (Spool_fh_nov);
 
-#if DEBUG_SPOOL
-   spool_debug ("%s %d %d",name,*min,*max);
-#endif
+   debug_output (NULL, -1, "%s %d %d",name,*min,*max);
    return 0;
 }
 
@@ -949,10 +956,6 @@ static int spool_select_group (char *name, int *min, int *max)
    Spool_Group = slrn_spool_dircat (Slrn_Spool_Root, name, 1);
    Spool_Group_Name = slrn_safe_strmalloc (name);
    
-#if DEBUG_SPOOL
-   spool_debug ("spool_select_group(%s) spool_group dir = %s", name, Spool_Group);
-#endif
-
    Spool_fh_nov = spool_open_nov_file ();
 
    if ((-1 == spool_read_minmax_from_overview (name, min, max))
@@ -965,12 +968,23 @@ static int spool_select_group (char *name, int *min, int *max)
    Spool_Max_Artnum = *max;
    Spool_Min_Artnum = *min;
 
-#if DEBUG_SPOOL
-   spool_debug ("Group: %s %d - %d", name, *min, *max);
-#endif
+   debug_output (NULL, -1, "Group: %s %d - %d", name, *min, *max);
    return OK_GROUP;
 }
 
+static int spool_refresh_groups (Slrn_Group_Range_Type *gr, int n)
+{
+   while (n)
+     {
+	if (-1 == spool_select_group (gr->name,
+				      &(gr->min), &(gr->max)))
+	  gr->min = -1;
+	n--;
+	gr++;
+     }
+   return 0;
+}
+
 static char *spool_current_group (void)
 {
    return Spool_Group_Name == NULL ? "" : Spool_Group_Name;
@@ -980,10 +994,6 @@ static int Spool_Server_Inited = 0;
 
 static void spool_close_server (void)
 {
-#if DEBUG_SPOOL
-   spool_debug ("spool_close_server()");
-#endif
-
    slrn_free (Spool_Group);
    Spool_Group = NULL;
 
@@ -1007,13 +1017,9 @@ static int spool_initialize_server (void)
 {
    if (Spool_Server_Inited) spool_close_server ();
 
-#if DEBUG_SPOOL
-   spool_debug ("spool_initialize_server(%s)", host);
-#endif
-
    if (2 != slrn_file_exists (Slrn_Spool_Root))
      {
-	slrn_error ("Local spool directory '%s' doesn't exist.", Slrn_Spool_Root);
+	slrn_error (_("Local spool directory '%s' doesn't exist."), Slrn_Spool_Root);
 	return -1;
      }
 
@@ -1027,23 +1033,20 @@ static int spool_initialize_server (void)
    return 0;
 }
 
-static int spool_read_line (char *line, unsigned int len)
+static int spool_read_fhlocal (char *line, unsigned int len)
 {
-   if (Spool_Doing_XPat) return spool_read_xpat (line, len);
-   
-   if (Spool_Doing_XOver) return spool_read_xover (line, len);
-   
-   if (NULL != Spool_XHdr_Field) return spool_read_xhdr (line, len);
-   
-   if (Spool_fFakingActive) return spool_fakeactive_read_line (line, len);
-   
-   if ((NULL == Spool_fh_local)
-       || (NULL == fgets (line, len, Spool_fh_local))
-       || (Spool_fhead && (line[0]=='\n')))
+   do
      {
-	spool_fclose_local();
-	return 0;
+	if ((NULL == Spool_fh_local)
+	    || (NULL == fgets (line, len, Spool_fh_local))
+	    || (Spool_fhead && (line[0]=='\n')))
+	  {
+	     spool_fclose_local();
+	     return 0;
+	  }
      }
+   while ((Spool_Output_Regexp != NULL) &&
+	  (NULL == slrn_regexp_match (Spool_Output_Regexp, line)));
 
    len = strlen(line);
 
@@ -1055,6 +1058,39 @@ static int spool_read_line (char *line, unsigned int len)
    return 1;
 }
 
+static int spool_read_line (char *line, unsigned int len)
+{
+   if (Spool_Doing_XPat) return spool_read_xpat (line, len);
+   
+   if (Spool_Doing_XOver) return spool_read_xover (line, len);
+   
+   if (NULL != Spool_XHdr_Field)
+     {
+	char buf[NNTP_BUFFER_SIZE];
+	int retval = -1;
+	
+	if (buf == NULL) return -1;
+	if ((len > 33) &&
+	    (1 == (retval = spool_read_xhdr (buf, sizeof (buf)))))
+	  {
+	     int numlen;
+#ifdef HAVE_ANSI_SPRINTF
+	     numlen =
+#endif
+	     sprintf (line, "%d ", (Spool_XOver_Next - 1)); /* safe */
+#ifndef HAVE_ANSI_SPRINTF
+	     numlen = strlen (line);
+#endif
+	     slrn_strncpy (line + numlen, buf, len - numlen);
+	  }
+	return retval;
+     }
+   
+   if (Spool_fFakingActive) return spool_fakeactive_read_line (line, len);
+   
+   return spool_read_fhlocal (line, len);
+}
+
 typedef struct
 {
    int xover_field;
@@ -1077,13 +1113,14 @@ static int spool_xpat_match (char *str, char *pat)
 
 static int spool_read_xpat (char *buf, unsigned int len)
 {
-   char tmpbuf [8129];
-
+   char tmpbuf [8192];
+   
+   Spool_Doing_XPat = 0;
+   
    if (Spool_XPat_Struct.xover_field == -1)
      {
 	unsigned int num;
 	
-	Spool_Doing_XPat = 0;
 	for (num = Spool_XPat_Struct.rmin; num <= Spool_XPat_Struct.rmax; num++)
 	  {
 	     if (-1 == spool_one_xhdr_command (Spool_XPat_Struct.header, (int) num,
@@ -1108,16 +1145,62 @@ static int spool_read_xpat (char *buf, unsigned int len)
 	  }
 	
 	Spool_XPat_Struct.rmin = Spool_XPat_Struct.rmax + 1;
-	return 0;
+     }
+   else
+     {
+	/* read the overview file until the right field matches the pattern */
+	while (NULL != (fgets (tmpbuf, sizeof (tmpbuf), Spool_fh_local)))
+	  {
+	     int field = Spool_XPat_Struct.xover_field + 1;
+	     /* first field is article number; "+1" skips it */
+	     unsigned int num = (unsigned int) atoi (tmpbuf);
+	     char *b = tmpbuf, *end;
+	     
+	     if (num > Spool_XPat_Struct.rmax)
+	       {
+		  spool_fclose_local ();
+		  return 0;
+	       }
+	     
+	     while (field)
+	       {
+		  b = strchr (b, '\t');
+		  /* If we reach the end of the overview line before finding
+		   * the correct field, the overview file must be corrupt. */
+		  if (b == NULL)
+		    {
+		       debug_output (NULL, -1, "Overview file corrupt? Unable to find "
+				     "field %d in overview line %s.",
+				     Spool_XPat_Struct.xover_field, tmpbuf);
+		       spool_fclose_local ();
+		       return -1;
+		    }
+		  b++;
+		  field--;
+	       }
+	     end = strchr (b, '\t');
+	     if (end != NULL)
+	       *end = '\0';
+	     
+	     if (-1 != spool_xpat_match (b, Spool_XPat_Struct.pat))
+	       {
+		  unsigned int blen;
+		  
+		  Spool_Doing_XPat = 1;
+		  slrn_snprintf (buf, len, "%u ", num);
+		  blen = strlen (buf);
+		  
+		  strncpy (buf + blen, b, len - blen);
+		  buf[len - 1] = 0;
+		  return 1;
+	       }
+	  }
+	spool_fclose_local ();
      }
    
-   /* Not implemented Yet. */
-   Spool_Doing_XPat = 0;
-   spool_fclose_local ();
    return 0;
 }
 
-   
 static int spool_xpat_cmd (char *hdr, int rmin, int rmax, char *pat)
 {
    static char *overview_headers [] = 
@@ -1163,13 +1246,13 @@ static int spool_xpat_cmd (char *hdr, int rmin, int rmax, char *pat)
 	  }
      }
    
-   Spool_XPat_Struct.xover_field = -1;
-   
    if (Spool_XPat_Struct.xover_field != -1)
      {
 	Spool_fh_local = spool_open_nov_file ();
 	if (Spool_fh_local == NULL)
 	  return ERR_COMMAND;
+	if (-1 == overview_file_seek (0, -1, Spool_XPat_Struct.rmin))
+	  return -1;
      }
    
    Spool_Doing_XPat = 1;
@@ -1177,12 +1260,6 @@ static int spool_xpat_cmd (char *hdr, int rmin, int rmax, char *pat)
    return OK_HEAD;
 }
 
-static int spool_xgtitle_cmd (char *pattern)
-{
-   (void) pattern;
-   return ERR_COMMAND;
-}
-
 static int spool_select_article (int n, char *msgid)
 {
    /*    printf("spool_select_article(%d,%s)\n",n,msgid); */
@@ -1222,21 +1299,23 @@ static int spool_one_xhdr_command (char *hdr, int num, char *buf,
    
    colon = strlen (hdr);
 
-   while (1 == spool_read_line (tmpbuf, sizeof (tmpbuf)))
+   while (1 == spool_read_fhlocal (tmpbuf, sizeof (tmpbuf)))
      {
 	char *b;
 	if (slrn_case_strncmp ((unsigned char *) tmpbuf, (unsigned char *) hdr, colon)
 	    || (tmpbuf[colon] != ':'))
 	  continue;
 
-	/* HACK!!  This needs fixed for folded lines. */
-	if (buflen > sizeof (tmpbuf))
-	  buflen = sizeof (tmpbuf);
-	
 	b = tmpbuf + (colon + 1);
 	if (*b == ' ') b++;
-	strncpy (buf, b, buflen - 1);
-	buf[buflen - 1] = 0;
+	slrn_strncpy (buf, b, buflen);
+	while ((1 == spool_read_fhlocal (tmpbuf, sizeof (tmpbuf))) &&
+	       ((*tmpbuf == ' ') || (*tmpbuf == '\t')))
+	  {
+	     unsigned int len = strlen (buf);
+	     b = tmpbuf + 1;
+	     slrn_strncpy (buf + len, b, buflen - len);
+	  }
 	return 0;
      }
    
@@ -1245,9 +1324,7 @@ static int spool_one_xhdr_command (char *hdr, int num, char *buf,
 
 static int spool_xhdr_command (char *field, int min, int max)
 {
-#if DEBUG_SPOOL
-   spool_debug ("spool_xhdr_command(%s,%d,%d)", field, min, max);
-#endif
+   debug_output (NULL, -1, "spool_xhdr_command(%s,%d,%d)", field, min, max);
    
    spool_fclose_local ();
    
@@ -1268,12 +1345,8 @@ static int spool_read_xhdr (char *the_buf, unsigned int len)
 {
    int retval = -1;
    
-#if DEBUG_SPOOL
-   spool_debug ("spool_read_xhdr");
-#endif
-
    if (Spool_XHdr_Field == NULL)
-     return 0;
+     return -1;
 
    if (Spool_XOver_Next > Spool_XOver_Max) 
      {
@@ -1285,7 +1358,16 @@ static int spool_read_xhdr (char *the_buf, unsigned int len)
      retval = spool_one_xhdr_command (Spool_XHdr_Field, Spool_XOver_Next++,
 				      the_buf, len);
    
-   return retval;
+   if (Spool_XOver_Next > Spool_XOver_Max)
+     Spool_XHdr_Field = NULL;
+   
+   return (retval == -1) ? -1 : 1;
+}
+
+static int spool_list (char *what)
+{
+   (void) what;
+   return ERR_FAULT;
 }
 
 static int spool_list_newsgroups (void)
@@ -1306,10 +1388,32 @@ static int spool_list_newsgroups (void)
    return OK_GROUPS;
 }
 
-static int spool_list_active (void)
+/* Having an independant buffer for the spool code might spare us mysterious
+ * bugs, so I'm not simply using slrn_compile_regexp_pattern */
+static SLRegexp_Type *spool_compile_regexp_pattern (char *pat)
+{
+   static unsigned char compiled_pattern_buf [512];
+   static SLRegexp_Type re;
+   
+   re.pat = (unsigned char *) pat;
+   re.buf = compiled_pattern_buf;
+   re.buf_len = sizeof (compiled_pattern_buf);
+   re.case_sensitive = 1;
+   
+   if (0 != SLang_regexp_compile (&re))
+     {
+	slrn_error (_("Invalid regular expression or expression too long."));
+	return NULL;
+     }
+   return &re;
+}
+
+static int spool_list_active (char *pat)
 {
    spool_fclose_local();
    Spool_fh_local=fopen (Slrn_Active_File,"r");
+   if (pat != NULL)
+     Spool_Output_Regexp=spool_compile_regexp_pattern (slrn_fix_regexp (pat));
    if (!Spool_fh_local)
      {
 	spool_fake_active(Slrn_Spool_Root);
@@ -1513,6 +1617,7 @@ int Slrn_Spool_Check_Up_On_Nov;
 static int spool_init_objects (void)
 {
    Spool_Server_Obj.sv_select_group = spool_select_group;
+   Spool_Server_Obj.sv_refresh_groups = spool_refresh_groups;
    Spool_Server_Obj.sv_current_group = spool_current_group;
    Spool_Server_Obj.sv_read_line = spool_read_line;
    Spool_Server_Obj.sv_close = spool_close_server;
@@ -1522,8 +1627,8 @@ static int spool_init_objects (void)
    Spool_Server_Obj.sv_put_server_cmd = spool_put_server_cmd;
    Spool_Server_Obj.sv_xpat_cmd = spool_xpat_cmd;
    Spool_Server_Obj.sv_xhdr_command = spool_one_xhdr_command;
-   Spool_Server_Obj.sv_xgtitle_cmd = spool_xgtitle_cmd;
    Spool_Server_Obj.sv_has_cmd = spool_has_cmd;
+   Spool_Server_Obj.sv_list = spool_list;
    Spool_Server_Obj.sv_list_newsgroups = spool_list_newsgroups;
    Spool_Server_Obj.sv_list_active = spool_list_active;
    Spool_Server_Obj.sv_send_authinfo = spool_send_authinfo;
@@ -1580,26 +1685,3 @@ static int spool_select_server_object (void)
    
    return 0;
 }
-
-
-static void spool_usage (void)
-{
-   fputs ("--spool options:\n\
-",
-	  stdout);
-   exit (0);
-}
-
-static int spool_parse_args (char **argv, int argc)
-{
-   int i;
-   
-   for (i = 0; i < argc; i++)
-     {
-	if (!strcmp (argv[i], "--help"))
-	  spool_usage ();
-	else break;
-     }
-   
-   return i;
-}
