@@ -44,7 +44,7 @@
 #include "startup.h"
 #include "score.h"
 #include "util.h"
-#include "uudecode.h"
+#include "decode.h"
 #include "mime.h"
 #if SLRN_HAS_GROUPLENS
 # include "grplens.h"
@@ -55,6 +55,7 @@
 #include "server.h"
 #include "chmap.h"
 #include "print.h"
+#include "snprintf.h"
 
 #ifdef VMS
 # include "vms.h"
@@ -160,9 +161,9 @@ int Slrn_Saw_Obsolete = 0;
 static void exit_malloc_error (void) /*{{{*/
 {
    if (This_File == NULL)
-     slrn_exit_error ("Memory Allocation Failure");
+     slrn_exit_error (_("Memory Allocation Failure"));
    
-   slrn_exit_error ("%s: Line %d\n%sMemory Allocation Failure",
+   slrn_exit_error (_("%s: Line %d\n%sMemory Allocation Failure"),
 		    This_File, This_Line_Num, This_Line);
 }
 
@@ -181,7 +182,7 @@ static char *safe_malloc (unsigned int n) /*{{{*/
 
 static void exit_unknown_object (void) /*{{{*/
 {
-   slrn_exit_error ("%s: Error encountered processing line %d\n%s",
+   slrn_exit_error (_("%s: Error encountered processing line %d\n%s"),
 		    This_File, This_Line_Num, This_Line);
 }
 
@@ -190,9 +191,9 @@ static void exit_unknown_object (void) /*{{{*/
 static void issue_obsolete_message (void) /*{{{*/
 {
    Slrn_Saw_Obsolete = 1;
-   slrn_message ("%s: Command is obsolete on line %d:\n%s",
+   slrn_message (_("%s: Command is obsolete on line %d:\n%s"),
 		 This_File, This_Line_Num, This_Line);
-   slrn_message ("The new usage is:\nset %s\n", This_Line);
+   slrn_message (_("The new usage is:\nset %s\n"), This_Line);
 }
 
 /*}}}*/
@@ -266,7 +267,7 @@ static int setkey_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 	obsolete_names = Art_Obsolete_Names;
      }
    else if (!strcmp (map, "readline")) kmap = Slrn_RLine_Keymap;
-   else slrn_exit_error ("%s: line %d:\n%sNo such keymap: %s", This_File, This_Line_Num, This_Line, map);
+   else slrn_exit_error (_("%s: line %d:\n%sNo such keymap: %s"), This_File, This_Line_Num, This_Line, map);
    
    if ((kmap == Slrn_Keymap_RLI->keymap) &&
        (NULL == SLang_find_key_function(fun, kmap)))
@@ -281,7 +282,7 @@ static int setkey_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
    
    if (failure)
      {
-	slrn_exit_error ("%s: line %d:\n%serror defining key.", This_File, This_Line_Num, This_Line);
+	slrn_exit_error (_("%s: line %d:\n%serror defining key."), This_File, This_Line_Num, This_Line);
      }
    else if (obsolete_names != NULL)
      {
@@ -300,8 +301,8 @@ static int setkey_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 	  }
 	
 	if (new_name != NULL)
-	  slrn_message ("%s: Obsolete function name on line %d: %s\n"
-			"The new name is: %s",
+	  slrn_message (_("%s: Obsolete function name on line %d: %s\n"
+			"The new name is: %s"),
 			This_File, This_Line_Num, old_name, new_name);
      }
 
@@ -320,7 +321,8 @@ static int unsetkey_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
    
    if (!strcmp (map, "group")) kmap = Slrn_Group_Keymap;
    else if (!strcmp (map, "article")) kmap = Slrn_Article_Keymap;
-   else slrn_exit_error ("%s: line %d:\n%sNo such keymap: %s",
+   else if (!strcmp (map, "readline")) kmap = Slrn_RLine_Keymap;
+   else slrn_exit_error (_("%s: line %d:\n%sNo such keymap: %s"),
 			 This_File, This_Line_Num, This_Line, map);
    
    SLang_undefine_key (key, kmap);
@@ -355,7 +357,7 @@ static SLRegexp_Type *compile_quote_regexp (char *str) /*{{{*/
    
    if (SLang_regexp_compile (r))
      {
-	slrn_exit_error ("%s: line %d:\n%sInvalid regular expression.",
+	slrn_exit_error (_("%s: line %d:\n%sInvalid regular expression."),
 			 This_File, This_Line_Num, This_Line);
      }
    
@@ -371,7 +373,7 @@ static void generic_regexp_fun (int argc, SLcmd_Cmd_Table_Type *cmd_table,
    
    if (argc > SLRN_MAX_REGEXP + 1)
      {
-	slrn_exit_error ("%s: line %d:\nToo many expressions specified.",
+	slrn_exit_error (_("%s: line %d:\n%sToo many expressions specified."),
 			 This_File, This_Line_Num, This_Line);
      }
    
@@ -462,16 +464,7 @@ static int set_comp_charsets_fun (int argc, SLcmd_Cmd_Table_Type *table)
 
 /*{{{ Setting/Getting Variable Functions */
 
-typedef struct /*{{{*/
-{
-   char *what;
-   int *valuep;
-}
-
-/*}}}*/
-Set_Int_Type;
-
-static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
+Slrn_Int_Var_Type Slrn_Int_Variables [] = /*{{{*/
 {
      {"hide_verbatim_marks",	&Slrn_Verbatim_Marks_Hidden},
      {"hide_signature",	&Slrn_Signature_Hidden},
@@ -516,6 +509,7 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
      {"show_article", &Slrn_Startup_With_Article},
      {"smart_quote", &Slrn_Smart_Quote},
      {"no_backups", &Slrn_No_Backups},
+     {"no_autosave", &Slrn_No_Autosave},
      {"beep", &SLtt_Ignore_Beep},
      {"unsubscribe_new_groups", &Slrn_Unsubscribe_New_Groups},
      {"check_new_groups", &Slrn_Check_New_Groups},
@@ -608,16 +602,7 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
 
 /*}}}*/
 
-typedef struct /*{{{*/
-{
-   char *what;
-   char **svaluep;
-}
-
-/*}}}*/
-Set_String_Type;
-
-static Set_String_Type String_Things_To_Set [] = /*{{{*/
+Slrn_Str_Var_Type Slrn_Str_Variables [] = /*{{{*/
 {
      {"failed_posts_file", &Slrn_Failed_Post_Filename},
 #if ! SLRN_HAS_STRICT_FROM
@@ -767,7 +752,7 @@ static Set_String_Type String_Things_To_Set [] = /*{{{*/
 
 int slrn_set_string_variable (char *name, char *value) /*{{{*/
 {
-   Set_String_Type *sp = String_Things_To_Set;
+   Slrn_Str_Var_Type *sp = Slrn_Str_Variables;
 	
    while (sp->what != NULL)
      {
@@ -777,8 +762,8 @@ int slrn_set_string_variable (char *name, char *value) /*{{{*/
 	     
 	     if ((This_File != NULL) && !strcmp (name, "followup"))
 	       {
-		  slrn_message ("%s: Obsolete variable name on line %d: followup\n"
-				"The new name is followup_string",
+		  slrn_message (_("%s: Obsolete variable name on line %d: followup\n"
+				"The new name is followup_string"),
 				This_File, This_Line_Num);
 		  Slrn_Saw_Obsolete = 1;
 	       }
@@ -803,7 +788,7 @@ int slrn_set_string_variable (char *name, char *value) /*{{{*/
 
 int slrn_set_integer_variable (char *name, int value) /*{{{*/
 {
-   Set_Int_Type *ip = Int_Things_To_Set;
+   Slrn_Int_Var_Type *ip = Slrn_Int_Variables;
    
    while (ip->what != NULL)
      {
@@ -815,23 +800,23 @@ int slrn_set_integer_variable (char *name, int value) /*{{{*/
 		      !strcmp (name, "display_author_realname") ||
 		      !strcmp (name, "display_score"))
 		    {
-		       slrn_message ("%s: Obsolete variable on line %d: %s\n"
-				     "Please make use of the header_display_format command instead.",
+		       slrn_message (_("%s: Obsolete variable on line %d: %s\n"
+				     "Please make use of the header_display_format command instead."),
 				     This_File, This_Line_Num, name);
 		       Slrn_Saw_Obsolete = 1;
 		    }
 		  else if (!strcmp (name, "group_dsc_start_column") ||
 			   !strcmp (name, "show_descriptions"))
 		    {
-		       slrn_message ("%s: Obsolete variable on line %d: %s\n"
-				     "Please make use of the group_display_format variable instead.",
+		       slrn_message (_("%s: Obsolete variable on line %d: %s\n"
+				     "Please make use of the group_display_format variable instead."),
 				     This_File, This_Line_Num, name);
 		       Slrn_Saw_Obsolete = 1;
 		    }
 		  else if (!strcmp (name, "process_verbatum_marks"))
 		    {
-		       slrn_message ("%s: Obsolete spelling on line %d:\n"
-				     "Use \"process_verbatim_marks\" instead of \"process_verbatum_marks\".",
+		       slrn_message (_("%s: Obsolete spelling on line %d:\n"
+				     "Use \"process_verbatim_marks\" instead of \"process_verbatum_marks\"."),
 				     This_File, This_Line_Num);
 		       Slrn_Saw_Obsolete = 1;
 		    }
@@ -839,7 +824,7 @@ int slrn_set_integer_variable (char *name, int value) /*{{{*/
 			   !strcmp (name, "query_reconnect") ||
 			   !strcmp (name, "use_xgtitle"))
 		    {
-		       slrn_message ("%s: Obsolete variable on line %d: %s",
+		       slrn_message (_("%s: Obsolete variable on line %d: %s"),
 				     This_File, This_Line_Num, name);
 		       Slrn_Saw_Obsolete = 1;
 		    }
@@ -857,10 +842,10 @@ int slrn_set_integer_variable (char *name, int value) /*{{{*/
 
 int slrn_get_variable_value (char *name, int *type, char ***sval, int **ival) /*{{{*/
 {
-   Set_String_Type *sp;
-   Set_Int_Type *ip;
+   Slrn_Str_Var_Type *sp;
+   Slrn_Int_Var_Type *ip;
    
-   sp = String_Things_To_Set;
+   sp = Slrn_Str_Variables;
    while (sp->what != NULL)
      {
 	if (!strcmp (sp->what, name))
@@ -872,7 +857,7 @@ int slrn_get_variable_value (char *name, int *type, char ***sval, int **ival) /*
 	sp++;
      }
    
-   ip = Int_Things_To_Set;
+   ip = Slrn_Int_Variables;
    while (ip->what != NULL)
      {
 	if (!strcmp (ip->what, name))
@@ -955,6 +940,7 @@ static Color_Handle_Type Color_Handles[] = /*{{{*/
      {"italicstext", 	ITALICS_COLOR,		"magenta", 	DEF_BG, 0},
      {"menu",		MENU_COLOR,		"yellow",	"blue", SLTT_REV_MASK},
      {"menu_press",	MENU_PRESS_COLOR,	DEF_FG,		"yellow", 0},
+     {"message",	MESSAGE_COLOR,		DEF_FG,		DEF_BG, 0},
      {"neg_score",	NEG_SCORE_COLOR,	"green",	DEF_BG,	0},
      {"normal",		0,			DEF_FG,		DEF_BG, 0},
      {"pos_score",	POS_SCORE_COLOR,	"blue",		DEF_BG,	SLTT_REV_MASK},
@@ -997,7 +983,7 @@ static char *Color_Names [16] =
 };
 /*}}}*/
 
-int slrn_set_object_color (char *name, char *fg, char *bg)
+int slrn_set_object_color (char *name, char *fg, char *bg) /*{{{*/
 {
    Color_Handle_Type *ct = Color_Handles;
 
@@ -1009,8 +995,8 @@ int slrn_set_object_color (char *name, char *fg, char *bg)
 	     
 	     if ((This_File != NULL) && (ct == Color_Handles))
 	       {
-		  slrn_message ("%s: Obsolete spelling on line %d:\n"
-				"Use \"verbatim\" instead of \"verbatum\"",
+		  slrn_message (_("%s: Obsolete spelling on line %d:\n"
+				"Use \"verbatim\" instead of \"verbatum\""),
 				This_File, This_Line_Num);
 		  Slrn_Saw_Obsolete = 1;
 	       }
@@ -1049,9 +1035,41 @@ int slrn_set_object_color (char *name, char *fg, char *bg)
 	return 0;
      }
 
-   slrn_error ("%s is not a color object", name);
+   slrn_error (_("%s is not a color object"), name);
    return -1;
 }
+/*}}}*/
+
+char *slrn_get_object_color (char *name, int want_bg) /*{{{*/
+{
+#if !defined(__unix__) || defined(IBMPC_SYSTEM)
+   (void) name; (void) want_bg;
+   slrn_error (_("Due to a limitation in S-Lang, this feature is only available on Unix."));
+#else
+   Color_Handle_Type *ct = Color_Handles;
+   
+   while (ct->name != NULL)
+     {
+	if (!strcmp (ct->name, name))
+	  {
+	     SLtt_Char_Type color;
+	     color = SLtt_get_color_object (ct->value);
+	     /* 0xFF is the internal representation of "default" in S-Lang */
+	     if (((color >> (want_bg ? 16 : 8)) & 0xFF) == 0xFF)
+	       return "default";
+	     if (color & SLTT_BOLD_MASK)
+	       color |= 0x8 << 8;
+	     if (color & SLTT_BLINK_MASK)
+	       color |= 0x8 << 16;
+	     return Color_Names[(color >> (want_bg ? 16 : 8)) & 0xF];
+	  }
+	ct++;
+     }
+   slrn_error (_("%s is not a color object"), name);
+#endif
+   return NULL;
+}
+/*}}}*/
 
 static int color_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 {
@@ -1139,7 +1157,7 @@ static int set_posting_host (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 	return 0;
      }
    
-   slrn_error ("%s is not a valid fully-qualified host name.", hostname);
+   slrn_error (_("%s is not a valid fully-qualified host name."), hostname);
    return -1;
 }
 
@@ -1256,34 +1274,52 @@ static Server_List_Type *find_server (char *host) /*{{{*/
 {
    Server_List_Type *s;
    char *port;
-
-   /* host may contain information about the port in the form "address:port".
-    * First try to find a match with the port information.  If that fails, 
-    * find a match without it.
+   int i, has_ssl = 0;
+   
+   if (0 == strncmp (host, "snews://", 8))
+     {
+	has_ssl = 1;
+	host += 8;
+     }
+   else if (0 == strncmp (host, "news://", 7))
+     host += 7;
+   
+   /* host may contain information about the port in the form "address:port"
+    * and about whether to use SSL with a prefix like "snews://"
+    * First try to find a match with exactly the same information.  If that
+    * fails, find a match without it.
     */
    host = slrn_safe_strmalloc (host);
    port = slrn_strchr (host, ':');
 
-   while (1)
+   for (i = 0; i < 2; i++)
      {
 	s = Server_List;
 	while (s != NULL)
 	  {
-	     if (0 == slrn_case_strcmp ((unsigned char *)host, 
-					(unsigned char *) s->host))
+	     char *this_host = s->host;
+	     int this_has_ssl = 0;
+	     
+	     if (0 == strncmp (this_host, "snews://", 8))
+	       {
+		  this_has_ssl = 1;
+		  this_host += 8;
+	       }
+	     else if (0 == strncmp (this_host, "news://", 7))
+	       this_host += 7;
+	     
+	     if ((0 == slrn_case_strcmp ((unsigned char *)host, 
+					 (unsigned char *) this_host)) &&
+		 ((i == 2) || (has_ssl == this_has_ssl)))
 	       break;
 	     s = s->next;
 	  }
 
-	if (s != NULL)
-	  break;
-
-	if (port == NULL)
+	if ((s != NULL) || (port == NULL))
 	  break;
 	
 	/* Try again without port information */
 	*port = 0;
-	port = NULL;
      }
 
    slrn_free (host);
@@ -1411,7 +1447,7 @@ int slrn_get_authorization (char *host, char **name, char **pass) /*{{{*/
 	     do_newline = 0;
 	  }
 
-	if (-1 == slrn_read_input ("Username: ", NULL, buf, 1, 0))
+	if (-1 == slrn_read_input (_("Username: "), NULL, buf, 1, 0))
 	  return -1;
 	slrn_free (s->username);
 	s->username = slrn_safe_strmalloc (buf);
@@ -1426,7 +1462,7 @@ int slrn_get_authorization (char *host, char **name, char **pass) /*{{{*/
 	       putc ('\n', stdout);
 	  }
 
-	if (-1 == slrn_read_input_no_echo ("Password: ", NULL, buf, 1, 0))
+	if (-1 == slrn_read_input_no_echo (_("Password: "), NULL, buf, 1, 0))
 	  return -1;
 	slrn_free (s->password);
 	s->password = slrn_safe_strmalloc (buf);
@@ -1455,8 +1491,8 @@ void slrn_startup_initialize (void) /*{{{*/
    Color_Handle_Type *h;
    int i;
 
-#if SLRN_HAS_SLANG && defined(SLRN_MACRO_DIR)
-   Slrn_Macro_Dir = slrn_safe_strmalloc (SLRN_MACRO_DIR);
+#if SLRN_HAS_SLANG
+   Slrn_Macro_Dir = slrn_strdup_strcat (SHAREDIR, "/macros", NULL);
 #endif
 
    slrn_init_modes ();
@@ -1518,18 +1554,18 @@ int slrn_read_startup_file (char *name) /*{{{*/
 
    if (-1 == slrn_init_readline ())
      {
-	slrn_exit_error ("Unable to initialize S-Lang readline library.");
+	slrn_exit_error (_("Unable to initialize S-Lang readline library."));
      }
    
    if (-1 == SLprep_open_prep (&pt))
      {
-	slrn_exit_error ("Error initializing S-Lang preprocessor.");
+	slrn_exit_error (_("Error initializing S-Lang preprocessor."));
      }
    
    fp = fopen (name, "r");
    if (fp == NULL) return -1;
    
-   slrn_message ("Reading startup file %s.", name);
+   slrn_message (_("Reading startup file %s."), name);
    
    save_this_file = This_File;
    save_this_line = This_Line;
@@ -1555,11 +1591,11 @@ int slrn_read_startup_file (char *name) /*{{{*/
    
    if ((Server_Object != NULL)
        && (-1 == (Slrn_Default_Server_Obj = slrn_map_name_to_object_id (0, Server_Object))))
-     slrn_exit_error ("Server object '%s' is not supported.", Server_Object);
+     slrn_exit_error (_("Server object '%s' is not supported."), Server_Object);
    
    if ((Post_Object != NULL) 
        && (-1 == (Slrn_Default_Post_Obj = slrn_map_name_to_object_id (1, Post_Object))))
-     slrn_exit_error ("Post object '%s' is not supported.", Post_Object);
+     slrn_exit_error (_("Post object '%s' is not supported."), Post_Object);
    
    This_File = save_this_file;
    This_Line = save_this_line;
@@ -1578,7 +1614,7 @@ static int include_file_fun (int argc, SLcmd_Cmd_Table_Type *table)
    slrn_make_home_filename (table->string_args [1], file, sizeof (file));
 
    if (-1 == slrn_read_startup_file (file))
-     slrn_exit_error ("%s: line %d:\n%sFile not opened: %s",
+     slrn_exit_error (_("%s: line %d:\n%sFile not opened: %s"),
 		      This_File, This_Line_Num, This_Line, file);
 
    return 0;
@@ -1600,4 +1636,3 @@ static int interpret_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 }
 
 /*}}}*/
-
