@@ -14,10 +14,11 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #include "config.h"
+#include "slrnfeat.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -53,4 +54,3 @@ void slrn_tty_error (char *fmt, ...)
    slrn_tty_vmessage (stderr, fmt, ap);
    va_end (ap);
 }
-
